 /*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient");

dojo.require("com.ibm.team.repository.web.foundation.Status");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.RestRequest");
dojo.require("com.ibm.team.repository.web.transport.ExternalTeamServerClient");

(function() {

var Status						= com.ibm.team.repository.web.foundation.Status;

var ServiceRequest				= com.ibm.team.repository.web.transport.ServiceRequest;
var RestRequest					= com.ibm.team.repository.web.transport.RestRequest;

var ExternalTeamServerClient	= com.ibm.team.repository.web.transport.ExternalTeamServerClient;

var ROA_SERVICE_URI				= "com.ibm.team.workitem.service.internal.roa.IRestService";

var METHOD_GET					= "GET";
var METHOD_POST					= "POST";
var METHOD_PUT					= "PUT";
var METHOD_DELETE				= "DELETE";

var RESOURCE_PROJECTAREAS		= "projectareas";
var RESOURCE_WORKITEM			= "workitems";
var RESOURCE_WORKITEM_DRAFT		= "drafts/workitems";
var RESOURCE_WORKITEM_ATTRIBUTE	= "attributes/workitem";

com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient= {

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getProjectAreas: function(serverDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Get Project Areas", false, true);
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_PROJECTAREAS, METHOD_GET);
		config.invoke(request, serverDescription, responseCallback, config.label, config.cancel);
	},

	getProjectArea: function(projectAreaId, serverDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Get Project Area", false, true);
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_PROJECTAREAS + "/" + projectAreaId, METHOD_GET);
		config.invoke(request, serverDescription, responseCallback, config.label, config.cancel);
	},

	postDraftWorkItem: function(workItemAttributes, projectAreaDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Create Draft Work Item", false, true);
		workItemAttributes["projectArea"]= projectAreaDescription.getProjectAreaItemId();
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_WORKITEM_DRAFT, METHOD_POST, workItemAttributes);
		config.invoke(request, projectAreaDescription, responseCallback, config.label, config.cancel);
	},

	modifyWorkItem: function(workItemId, workItemAttributes, projectAreaDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Modify Work Item", false, true);
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_WORKITEM + "/" + workItemId, METHOD_PUT, workItemAttributes);
		config.invoke(request, projectAreaDescription, responseCallback, config.label, config.cancel);
	},
	
	modifyWorkItemLinks: function(workItemId, workItemAttributes, projectAreaDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Modify Work Item Links", false, true);
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_WORKITEM + "/" + workItemId + "/links", METHOD_POST, workItemAttributes);
		config.invoke(request, projectAreaDescription, responseCallback, config.label, config.cancel);
	},
	
	queryWorkItems: function(query, projectAreaDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Query Work Items", false, true);
		query["projectArea"]= projectAreaDescription.getProjectAreaItemId();
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_WORKITEM, METHOD_GET, query);
		config.invoke(request, projectAreaDescription, responseCallback, config.label, config.cancel);
	},

	queryWorkItemAttributes: function(attributeQuery, projectAreaDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Query Work Item Attribute Definitions", false, true);
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_WORKITEM_ATTRIBUTE, METHOD_GET, attributeQuery);
		config.invoke(request, projectAreaDescription, responseCallback, config.label, config.cancel);
	},

	addWorkItemAttribute: function(attributeAttributes, projectAreaDescription, responseCallback, options) {
		var config= this._processRequestOptions(options, "Add Work Item Attribute Definition", false, true);
		attributeAttributes["projectArea"]= projectAreaDescription.getProjectAreaItemId();
		var request= RestRequest.create(ROA_SERVICE_URI, RESOURCE_WORKITEM_ATTRIBUTE, METHOD_POST, attributeAttributes);
		config.invoke(request, projectAreaDescription, responseCallback, config.label, config.cancel);
	},


	
	ensureAttribute: function(externalProjectArea, responseHandler, appArgs) {
		var serviceObj= {
			success: function(collection){ 
			},
			failure: function() { 
				responseHandler.doErrorCallback(arguments); 
			}
		};
		this.get(externalProjectArea, new ServiceResponseHandler(serviceObj, "success", "failure"), "attributes/workitem", appArgs);
	},
	
	createAttribute: function(externalProjectArea, responseHandler, appArgs) {
		this.post(externalProjectArea, responseHandler, "attributes/workitem", appArgs);
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	_processRequestOptions: function(options, label, cancel, check) {
		if (options) {
			if (typeof options.requestLabel == "string") {
				label= options.requestLabel;
			}
			if (typeof options.cancelIfNotLoggedIn == "boolean") {
				cancel= options.cancelIfNotLoggedIn;
			}

			// for now, we only offer user-checked requests!
			//	if (typeof options.checkLoggedInUser == "boolean") {
			// 		check= options.checkLoggedInUser;
			//	}
			check= true;
		}
		
		return {
			label: label,
			cancel: cancel,
			invoke: ExternalTeamServerClient[check ? "invokeService" : "invokeServiceUnchecked"]
		}
	},

	__sentinel: null // terminates this class definition
};

})();