/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("com.ibm.asq.common.web.client.internal._catalog.Part");
dojo.require("com.ibm.asq.common.web.client.internal._catalog._NodeBase");

var _tagCounter = 0;

dojo.declare("com.ibm.asq.common.web.client.internal._catalog.Part", com.ibm.asq.common.web.client.internal._catalog._NodeBase, 
{
    constructor: function(document, parent, instanceRoot, templateObj, path){
        this.isResolved = false;
        if (instanceRoot != null) {
            this.instanceObj = instanceRoot.AssetType;
            if(instanceRoot.uuid != null) {
                this.isResolved = true;
            }
        }
        if (path == null) {
            this.path = templateObj.type;
        }
        else {
            this.path = path + "." + templateObj.type;
        }
        this.attributeMap = {};
        if(this.instanceObj != null){
            this._initialize();
        }
        this.tag = _tagCounter++;
    },
    // Get the name of the Part
    getName: function(){
        return this.templateObj.name;
    },
    // Get the name of the Part
    getType: function(){
        return this.templateObj.type;
    },
    // Get the tag for the Part
    getTag: function(){
        return this.tag;
    },
    // Get the children of this Part.  For example, RedHat 
    // Linux might be a child of Linux.
    getChildParts: function(){
        if(!this.subParts == null) {
            return this.subParts;
        }
        var partArrayObj = this._util._getArrayObject(this.templateObj, "Subtypes", "AssetType");
        var partArray = new Array();
        for(var i=0; i<partArrayObj.length; i++) {
            var part = new com.ibm.asq.common.web.client.internal._catalog.Part(this.document, this, null, partArrayObj[i], null);
            partArray.push(part);
        }
        return partArray;
    },
    // Get a list of the names of all parts in the subtree of this
    // part.  Returns an array of strings.
    getAllSubPartNames: function() {
        return this._getAllSubPartNames().sort();
    },
    _getAllSubPartNames: function() {
        var partNames = new Array();
        var childParts = this.getChildParts();
        for(var i=0; i < childParts.length; i++) {
            partNames.push(childParts[i].templateObj.name);
            partNames = partNames.concat(childParts[i]._getAllSubPartNames());
        }
        return partNames;
    },
    // Get the parent of this Part.  This will return the template
    // if the Part is a root Part.
    getParent: function(){
        return this.parent;
    },
    // Get the names of the attributes for the Part.
    getAttributeNames: function(){
        return this._getAttributeNames().sort();
    },
    // Walk the parent Parts to get all the attribute names.
    _getAttributeNames: function(){
        var attNames = new Array();
        if(this.getParent() instanceof com.ibm.asq.common.web.client.internal._catalog.Part) {
            attNames = this.getParent()._getAttributeNames();
        }
        var attributeArrayObject = this._util._getArrayObject(this.templateObj, "Attributes", "Attribute");
        for(var i=0; i<attributeArrayObject.length; i++) {
            attNames.push(attributeArrayObject[i].name);
        }
        return attNames;
    },
    // Mark the attribute as ignored in a search config
    setAttributeIgnored: function(attributeName, ignore){
        this.getAttribute(attributeName).setIgnored(ignore);
    },
    // Get if the attribute is ignored in a search config
    isAttributeIgnored: function(attributeName){
        return this.getAttribute(attributeName).isIgnored();
    },
    // Get an attribute of the Part by its name.
    getAttribute: function(attributeName){
        // Parts in the template must create new attributes
        if (this.instanceObj == null) {
            return (this._initializeAttribute(attributeName));
        }
        else {
            return (this.attributeMap[attributeName]);
        }
    },
    // Determine if an attribute is an array.
    isAttributeArray: function(attributeName){
        return this.getAttribute(attributeName).isArray();
    },
    // Get an attribute value of the Part by its name.
	// If createPartIfMissing is true and the attribute value is null,
	// and the attribute represents a Part, a generic instance of the 
	// Part will be created and then returned.  This method is
    // only valid for non-arrays.
    getAttributeValue: function(attributeName, createPartIfMissing){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var value = null;
		var attr = this.getAttribute(attributeName);
		if (attr){
			value =  attr.getValue();
			if (createPartIfMissing && value == null){
				if (attr.getType() != 'ConfigurationPartType'){
					throw new Error("createPartIfMissing only applies to configuration parts: "+attributeName)
				}
				var typeName = attr.getPartTypeName();
				this.setAttributeValue(attributeName, typeName);
				attr = this.getAttribute(attributeName);
				value = attr.getValue();
			}
		}
		return value;
    },
    // Get an attribute value of the Part by its name.
	// If createPartIfMissing is true and the attribute value is null,
	// and the attribute represents a Part, a generic instance of the 
	// Part will be created and then returned.  This method is
    // only valid for non-arrays.
    getAttributeArrayValues: function(attributeName, createPartIfMissing){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var values = null;
		var attr = this.getAttribute(attributeName);
		if (attr){
			values =  attr.getArrayValues();
			if (createPartIfMissing && (values == null || values.length == 0)){
				if (attr.getType() != 'ConfigurationPartType'){
					throw new Error("createPartIfMissing only applies to configuration parts: "+attributeName)
				}
				var typeName = attr.getPartTypeName();
				this.addAttributeArrayValue(attributeName, typeName);
				attr = this.getAttribute(attributeName);
				values = attr.getArrayValues();
			}
		}
		return values;
    },
    // Get an attribute value of the Part by its name and part tag.
    // This method is only valid for part type attributes.
    // If createPartIfMissing is true and the attribute value is null,
    // and the attribute represents a Part, a generic instance of the 
    // Part will be created and then returned.  This method is
    // only valid for arrays.
    getAttributeArrayValueByTag: function(attributeName, tag, createPartIfMissing){
        var attr = this.getAttribute(attributeName);
        var index = attr.getIndexByTag(tag);
        return this.getAttributeArrayValue(attributeName, index, createPartIfMissing);
    },
    // Get an attribute value of the Part by its name.
	// If createPartIfMissing is true and the attribute value is null,
	// and the attribute represents a Part, a generic instance of the 
	// Part will be created and then returned.  This method is
    // only valid for arrays.
    getAttributeArrayValue: function(attributeName, index, createPartIfMissing){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var value = null;
		var attr = this.getAttribute(attributeName);
		if (attr){
			var values =  attr.getArrayValues();
			if (createPartIfMissing && (values == null || values.length == 0)){
				if (attr.getType() != 'ConfigurationPartType'){
					throw new Error("createPartIfMissing only applies to configuration parts: "+attributeName)
				}
				var typeName = attr.getPartTypeName();
				this.addAttributeArrayValue(attributeName, typeName);
				attr = this.getAttribute(attributeName);
				value = attr.getArrayValue(0);
			} else {
                value = values[index];
            }
		}
		return value;
    },
    // Set an attribute value of the Part.
    setAttributeValue: function(attributeName, value){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.setValue(value);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Add an attribute value of the Part.
    addAttributeArrayValue: function(attributeName, value){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.addArrayValue(value);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Set an attribute value of the Part.
    // This method is only valid for part type attributes.
    setAttributeArrayValueByTag: function(attributeName, tag, value){
        var attr = this.getAttribute(attributeName);
        var index = attr.getIndexByTag(tag);
        return this.setAttributeArrayValue(attributeName, index, value);
    },
    // Set an attribute value of the Part.
    setAttributeArrayValue: function(attributeName, index, value){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.setArrayValue(index, value);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Remove an attribute value of the Part by the part tag.  This will
    // re-index the array of values.
    // This method is only valid for part type attributes.
    removeAttributeArrayValueByTag: function(attributeName, tag){
        var attr = this.getAttribute(attributeName);
        var index = attr.getIndexByTag(tag);
        return this.removeAttributeArrayValue(attributeName, index);
    },
    // Remove an attribute value of the Part.  This will
    // re-index the array of values.
    removeAttributeArrayValue: function(attributeName, index){
        if(this.isTemplatePart == true){
            throw new Error("Template Parts do not have attribute values!");
        }
        var att = this.getAttribute(attributeName);
		if (att){
			att.removeArrayValue(index);
		}else{
			throw Error( "invalid attribute");	
		}
    },
    // Get the template that contains the Part
    getTemplate: function(){
        var currParent = this.getParent();
        while(currParent instanceof com.ibm.asq.common.web.client.internal._catalog.Part) {
            currParent = currParent.getParent();
        }
        if(!(currParent instanceof com.ibm.asq.common.web.client.internal._catalog.Template)) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> improperly constructed Part.  No template at root!");
            throw new Error("improperly constructed Part.  No template at root!");
        }
        return currParent;
    }, 
    // Get the root Part from the template that contains this 
    // Part. 
    getRoot: function(){
        var current = this;
        while(current.parent instanceof com.ibm.asq.common.web.client.internal._catalog.Part) {
            current = current.parent;
        }
        return current;
    },
    // Check if the part is a generic (ie Root) part
    isGeneric: function(){
        if(this.parent instanceof com.ibm.asq.common.web.client.internal._catalog.Part) {
            return false;
        } else {
            return true;
        }
    }, 
    // Determine if this is a resolved config part 
    _isResolved: function(){
		var uuid = this.instanceRoot.uuid;
        return(uuid != null);
    },
    // Get the enum values for this type starting with
    // this part and working up the tree.
    _getEnumValues: function(name){
        return this._getEnumValuesInner(name).sort();
    },
    _getEnumValuesInner: function(name){
        var values = this._getAttributeValues(name);
        if(values != null){
            return values;
        } else{
            var par = this.getParent();
            if(par instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
                return par._getEnumValues(name);
            } else{
                return new Array();
            }
        }
    },
    // Determine if a given attribute is an enum
    _isEnum: function(name){
        var hasValues = this._hasAttributeValues(name);
        if(hasValues){
            return true;
        } else{
            var par = this.getParent();
            if(par instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
                return par._isEnum(name);
            } else{
                return false;
            }
        }
    },
    // Get a set of attribute values
    _getAttributeValues: function(name){
        var ranges = this.templateObj.AttributeValueRanges;
        if(ranges == null || ranges == ""){
            return null;
        }
        var rangeArray = ranges.AttributeValueRange;
		
		if (!dojo.isArray(rangeArray)) {
			rangeArray = [rangeArray];
		}
		
        var found = -1;
        for(var i=0; i<rangeArray.length && found == -1; i++){
            if(rangeArray[i].name == name){
                found = i;
            }
        }
        if(found == -1){
            return null;
        }
        return rangeArray[found].value.string;
    },
    // Determine if there are attribute values
    _hasAttributeValues: function(name){
        if(this._getAttributeValues(name) == null){
            return false;
        }
        else return true;
    },
    // Determine if the values used by an attribute are
    // from the named part
    _usingAttributeValuesFrom: function(partName, attributeName){
        if(this.templateObj.name == partName){
            return true;
        }
        if(this._hasAttributeValues(attributeName)){
            return false;
        }
        return this.parent._usingAttributeValuesFrom(partName, attributeName);
    },
    // Create a Part that is a "clone" of this part and
    // can be added to a Configuration.
    _createInstancePart: function(document, instanceRoot, path) {
        var partRoot = null;
        if (instanceRoot != null) {
            partRoot = instanceRoot;
        }
        else {
            partRoot = {
                AssetType: {
                    type: this.templateObj.type,
                    name: this.templateObj.name,
                    Attributes: {
                        Attribute: new Array()
                    }
                }
            }
        }
        var part = new com.ibm.asq.common.web.client.internal._catalog.Part(
                document,
                this.parent,
                partRoot,
                this.templateObj,
                path);
        return part;
    },
    // Create Parts and Attributes for all objects in the part's attribute tree.
    _initialize: function(){
        var attNames = this._getAttributeNames();
        for(var i=0;i<attNames.length;i++){
            var att = this._initializeAttribute(attNames[i]);
            this.attributeMap[attNames[i]] = att;
        }
    },
    _initializeAttribute: function(attributeName){
        // First find the attribute in the template.  
        var part = this;
        var templateAttributeObject = null;
        while(templateAttributeObject == null &&
              part instanceof com.ibm.asq.common.web.client.internal._catalog.Part) {
                  
            // See if the attribute is in this part
            var templateAttributeArrayObject = part._util._getArrayObject(part.templateObj, "Attributes", "Attribute");
            for(var i=0; (i< templateAttributeArrayObject.length && templateAttributeObject == null); i++) {
                if(templateAttributeArrayObject[i].name == attributeName) {
                    templateAttributeObject = templateAttributeArrayObject[i];
                }
            }
            part = part.getParent();
        }
        if(templateAttributeObject == null) {
            console.debug("*************!!!!!!!!!!!!!!!!>>>>>>>>>>>>> Invalid attribute name - " + attributeName + "!");
            throw Error( "Attribute not found");    
        }
        
        // Now see if there is an instance object for this attribute
        var attributeObject = null;
        if (this.instanceObj != null) {
            var attributeArrayObject = this._util._getArrayObject(this.instanceObj, "Attributes", "Attribute");
            for(var i=0; (i< attributeArrayObject.length && attributeObject == null); i++) {
                if(attributeArrayObject[i].name == attributeName) {
                    attributeObject = attributeArrayObject[i];
                }
            }
        }
        var att = new com.ibm.asq.common.web.client.internal._catalog.Attribute(this.document, this, attributeObject, templateAttributeObject, this.path);
        return att;    
    },
    _getCommonRoot: function(part) {
        // Create a hash of the names in this parts parent hierarchy
        var partHash = {};
        var name = this.templateObj.name;
        partHash[name] = this;
        var node = this.parent;
        while(node instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
            name = node.templateObj.name;
            partHash[name] = node;
            node = node.parent;
        }
        // Find the node in the passed in part that is in the hash
        node = part;
        while(node instanceof com.ibm.asq.common.web.client.internal._catalog.Part){
            name = node.templateObj.name;
            var commonPart = partHash[name];
            if(commonPart != null){
                return partHash[name];
            }
            node = node.parent;
        }
        return null;
    },
	// _setValuesFromPart is used when we want to change the type of
	// a part but maintain the values.  It will copy all of the values
	// and the uuids.  For part type values, it will reuse the parts, not
	// make copies of them.
    _setValuesFromPart: function(oldPart){
        // Transfer the uuid and stateId first as the serialization
		// needs them to be the first attributes.
        var oldUuid = oldPart.instanceRoot.uuid;
        if(oldUuid!= null){
	        // Need to force this ordering
            var instanceRoot = this.instanceRoot;
	        var at = instanceRoot.AssetType;
	        this.instanceRoot = {};
            instanceRoot = this.instanceRoot;
	        instanceRoot.uuid = oldUuid;
            instanceRoot.stateId = oldPart.instanceRoot.stateId;
	        instanceRoot.AssetType = at;
            this.instanceObj = instanceRoot.AssetType;
        }
        // Get the common root
        var commonRoot = oldPart._getCommonRoot(this);
        var commonRootName = commonRoot.getName();
        // Set the value for each of the attributes in the common root
        var attNames = commonRoot._getAttributeNames();
        for(var i=0;i<attNames.length;i++){
            var name = attNames[i];
            var oldAtt = oldPart.getAttribute(name);
            var newAtt = this.getAttribute(name);
            var commonRootAtt = commonRoot.getAttribute(name);
            // If this is an enum, get the available values
            var enumVals = null;
            if(newAtt.isEnum()){
                // We will get the enum values from the common root
                // only if neither the new attribute nor the old
                // attribute has overridden those values.
                if (this._usingAttributeValuesFrom(commonRootName, name) &&
                    oldPart._usingAttributeValuesFrom(commonRootName, name)) {
                    enumVals = {};
                    var valArr = commonRootAtt.getEnumValues();
                    for (var j = 0; j < valArr.length; j++) {
                        enumVals[valArr[j]] = valArr[j];
                    }
                }
            }
            if(oldAtt.isArray()){
                var values = oldAtt.getArrayValues();
                for(var j=0;j<values.length;j++){
                    if (enumVals == null) {
                        newAtt._addArrayValue(values[j], false);
                    }
                    else {
                        // Check if the value is in the enum
                        if(enumVals[values[j]] != null){
                            newAtt._addArrayValue(values[j], false);
                        }
                    }
                }
            }else{
                if (enumVals == null) {
	                // Don't need to set nulls
	                var value = oldAtt.getValue();
	                if (value != null) {
	                    newAtt._setValue(oldAtt.getValue(), false);
	                }
                }
                else {
	                // Check if the value is in the enum
	                var value = oldAtt.getValue();
	                if(value != null) {
	                    if (enumVals[value] != null) {
	                        newAtt._setValue(value, false);
	                    }
	                }
	            }
            }
        }
    },
    
    // _copyValuesFromPart is used when we want to make a duplicate of the
	// part.  It will copy all of the values but not the uuids.  For part
	// type values, it will create new part objects, it will not reuse
    // the existing objects.  The type and name of the old part must be
	// the same as this or the method will fail.
    _copyValuesFromPart: function(oldPart){
        // Set the value for each of the attributes.
        var attNames = this.getAttributeNames();
        for(var i=0;i<attNames.length;i++){
            var name = attNames[i];
            var oldAtt = oldPart.getAttribute(name);
            var isPart = (oldAtt.getType() == "ConfigurationPartType");
            if(oldAtt.isArray()){
                var values = oldAtt.getArrayValues();
				if (values.length > 0) {
					var newAtt = this.getAttribute(name);
					for (var j = 0; j < values.length; j++) {
    					newAtt._addArrayValue(values[j], false);
						if(isPart){
							var part = newAtt.getArrayValue(j);
							part._copyValuesFromPart(values[j]);
						}
					}
				}
            }else{
                // Don't need to set nulls
                var value = oldAtt.getValue();
                if (value != null) {
                    var newAtt = this.getAttribute(name);
                    newAtt._setValue(oldAtt.getValue(), false);
                    if(isPart){
                        var part = newAtt.getValue();
                        part._copyValuesFromPart(value);
                    }
                }
            }
        }
    },
    
    isAncestor: function(parentName){
        var parent = this.getParent();
        if(!(parent instanceof com.ibm.asq.common.web.client.internal._catalog.Part)){
            return false;
        }
        if(parent.getName() == parentName){
            return true;
        }
        return parent.isAncestor(parentName);
    }

}   
)
})();
