/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

dojo.require("dojo.date.locale");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.rational.test.lm.web.client.internal.admin");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

// define a message bundle id for RTLM
var MessageBundleRTLM = "RTLM";

dojo.declare("com.ibm.rational.test.lm.web.ui.internal._MessageLoader", null,
{
    constructor: function(){
        // initializes the rtlm message bundle
        MessageUtil.registerRequireBundle(MessageBundleRTLM, "com.ibm.rational.test.lm.web.ui.internal", "messages");
        console.debug("Registered RTLM bundle: \"" + MessageUtil.getMessage(MessageBundleRTLM, "bundle_description") + "\"" );
    }
});

// create an instance to ensure the message bundle is loaded
com.ibm.rational.test.lm.web.ui.internal.MessageLoader = new com.ibm.rational.test.lm.web.ui.internal._MessageLoader();

com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil = {
	
    TYPE_CONFIG: "config",
    TYPE_MACHINE: "machine",
    TYPE_VM: "virtual_image",
    TYPE_REQUEST: "request",
    TYPE_GROUP: "group",
    TYPE_VMI_COLLECTION: "virtual_image_collection",
    TYPE_VM_COLLECTION: "virtual_machine_collection",
    
    showWorkingDialog: function(msg)
    {
        var label = document.getElementById("progress_label");
        var progress = document.getElementById("progress_indicator");
        var header = document.getElementById("form_main_header");
        if (label && progress) {
            // use in page progress indicator if the elements are in the page
            if (header) {
                // there may not be a default header in the page
                header.style.display = "none";
            }
            label.innerHTML = msg;
            progress.style.display = "inline";
        } else {
            // use progress dialog if the in page progress indicator elements are not in the page
            var workingDialog = dijit.byId('progress_dialog');
            var workingMsg = document.getElementById("progress_dialog_msg");
            if (workingDialog && workingMsg) {
                workingMsg.innerHTML = msg;
                workingDialog.show();
            }
        }
    },
    
    hideWorkingDialog: function(delay)
    {
        var label = document.getElementById("progress_label");
        var progress = document.getElementById("progress_indicator");
        var header = document.getElementById("form_main_header");
        if (label && progress) {
            // use in page progress indicator if the elements are in the page
            var timeout = delay ? delay : "200";
            command = "document.getElementById('progress_label').innerHTML = '';";
            command += "document.getElementById('progress_indicator').style.display = 'none';";
            if (header) {
                // there may not be a default header in the page
                command += "document.getElementById('form_main_header').style.display = 'inline';";
            }
            setTimeout(command, timeout);
        } else {
            // use progress dialog if the in page progress indicator elements are not in the page
            var workingDialog = dijit.byId('progress_dialog');
            var timeout = delay ? delay : "200";
            if (workingDialog) {
                setTimeout("dijit.byId('progress_dialog').hide()", timeout);
            }
        }
    },
    
    setComboContent: function(w_id, values, selectValue, createEmpty, force) 
    {
        var combo = w_id;
        
        if(dojo.isString(w_id))
            combo = dojo.byId(w_id);
            
        if(combo) {
            // only load the data to the combo box if it has not been initialized.
            if(combo.length == 0 || force) {
                var pos = 0;

	            if (combo.options == null) // TODO - review this to see if this makes sense?
	                combo.options = [];
                
                // create an empty selection
                if(createEmpty != null && createEmpty == true) {
                    combo.options[0] = new Option("");
                    pos = 1;
                }     
                var data = null;
                if(values && values.length > 0) {
                    for (var i = 0; i < values.length; i++) {
                        var name = values[i];
                        var value = name;
                        
                        if(dojo.isObject(values[i])) {
                            name = values[i].name;
                            value = (values[i].value) ? values[i].value : values[i].name; 
                        }
                        // the display text for the combo option
                        // is the same as the value that we want to
                        // retrieve. We can just use the option's 'text'
                        // property, i.e., don't need to set the option's
                        // 'value' property.
                        combo.options[i+pos] = new Option(name, value);
                        
                        // make this option selected (shown) if a 
                        // selectValue is specified and it matches this
                        // option.
                        if(selectValue && value == selectValue)
                            combo.options[i+pos].selected = true;
                    }
                }
                // don't select anything if we don't have a default value
                // and there's not an empty option.
                if(selectValue == null && !createEmpty)
                    combo.selectedIndex = -1;
            }
        }
    },

	//A special handler for user combos to weed out the archived users
    setUserComboContent: function(w_id, users, selectValue, createEmpty, force) 
    {
        var combo = w_id;
        
        if(dojo.isString(w_id))
            combo = dojo.byId(w_id);
            
        if(combo) {
            // only load the data to the combo box if it has not been initialized.
            if(combo.length == 0 || force) {
                var pos = 0;

	            if (combo.options == null) // TODO - review this to see if this makes sense?
	                combo.options = [];
                
                // create an empty selection
                if(createEmpty != null && createEmpty == true) {
                    combo.options[0] = new Option("");
                    pos = 1;
                }     
                var data = null;
                if(users && users.length > 0) {
                    for (var i = 0; i < users.length; i++) {
                        // the display text for the combo option
                        // is the same as the value that we want to
                        // retrieve. We can just use the option's 'text'
                        // property, i.e., don't need to set the option's
                        // 'value' property.
						
						//Don't include archived users UNLESS the archived user is the default value
						if (users[i].archived)
						{
							if (!selectValue || users[i].name != selectValue)
								continue;
						}
						
						var option = new Option(users[i].name, users[i].itemId);
						option.disabled = users[i].archived;  //disable archived users
                        combo.options[pos] = option;
                        
                        // make this option selected (shown) if a 
                        // selectValue is specified and it matches this
                        // option.
                        if(selectValue && users[i].name == selectValue)
                            combo.options[pos].selected = true;
							
						pos++;
                    }
                }
                // don't select anything if we don't have a default value
                // and there's not an empty option.
                if(selectValue == null && !createEmpty)
                    combo.selectedIndex = -1;
            }
        }
    },


    // Summary:
    //    Scans the page the for the list of nodes with the specified tag. If a scanned node
    //    has an id that starts with the specified pattern, it uses the node's id as the key to retrieve
    //    the message from the message bundle and loads the returned string into the node's
    //    content.
    //
    loadPageLabelsByTagName: function(tag, pattern)
    {
        var nodes = document.getElementsByTagName(tag);
        for (var i = 0; i < nodes.length; i++) {
            var id = nodes[i].id;
            if(id && id.indexOf(pattern) >= 0) {
                var label = this.getMessage(id);
                if(!label)
                    label = "<b>missing Label: "+id+"</b>";
                nodes[i].innerHTML = label;
            } 
        }
    },
    
    // Summary:
    //    Displays server error message in a popup dialog. Normalizes the server message to 
    //    replace encoded character strings (%2D), etc.
    //
    showServerErrorMessage: function(message)
    {
        var c = new RegExp("\%[a-zA-Z0-9]{2}");
        var ss = message.split(c);
        var str = "";
        for(var i = 0; i < ss.length; i++)
            str = str + ss[i];
           
        alert(str);
    },
    
    // Summary:
    //    Normalizes the timestamp string to the timezone format expected by
    //    dojo.date.stamp.fromISOString. The server returned timestamp contains timezone
    //    format as [-+][hhmm]. The dojo timestamp expects [-+][hh:mm]. 
    //
    normalizeTimeStampString: function(/* timestamp string */timestamp)
    {
        var timeZone = new RegExp("[-+][0-9]{4}");
        
        if(timestamp && timestamp.length) {
            var indx = timestamp.search(timeZone);
        
            if(indx > -1) {
                var str = timestamp.substring(0, indx + 3) + ":" + timestamp.substring(indx + 3, indx + 5);
                return str;
            }
        }
        return timestamp;
    },
    
    // Summary:
    //    Converts a timestamp returned by the server into a Date object.
    //    If the timestamp is null, or empty, today's date object is returned.
    //
    convertTimeStampToDate: function(/* timestamp string */timestamp)
    {   
        var date = new Date();
             
        if(timestamp && timestamp.length) {
            var str = this.normalizeTimeStampString(timestamp);
            date = dojo.date.stamp.fromISOString(str);
        }
        return date; 
    },
	
	getDateDisplay: function(dateValue, dateOnly)
    {
		var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;

        // Constant that represents the date equal to "-1".  This is the
        // convention used to represent a null date.  This convention is needed
        // so that we can indicate that we want to remove a date from a date field.
        var _NODATE = "1969-12-31T18:59:59.999-0500";
        
        if(dateValue == null)
            return "";
            
        // Check if the date is -1.  If it is we have no date specified
        // and we should display a blank.
        if( dateValue == _NODATE ) {
            return "";
        }
        var date = RtlmUiUtil.convertTimeStampToDate(dateValue);
        
        if (dateOnly) {
            var style = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatStyle();
            return dojo.date.locale.format(date, style);
        }    
        return RtlmUiUtil.getDisplayDateTimeString(date);
    },
    
    // Summary:
    //    Returns a locale specific date/time displayable string. This method should be used
    //    to obtain a locale display string for date/time. If there's a need to deviate from the
    //    default locale display format, please use the dojo.date.locale object to do the formatting. 
    //
    getDisplayDateTimeString: function(/* the Date object */datetime)
    {
        var length = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatLength();
        var str = dojo.date.locale.format(datetime, {formatLength:length});
        return str;
    },
    
    // the version number of the application
    _versionNumber: null,
    
    _showVersionDialog: function(msg) {
        var versionLabel = document.getElementById('version_label');
        if (versionLabel) {
            versionLabel.innerHTML = msg;
        }
    /*
        // not using dialog
        var aboutDialog = dijit.byId('about_dialog');
        var aboutMsg = document.getElementById('about_msg');
        var copyrightMsg = document.getElementById('copyright_msg');
        var okButton = document.getElementById("about_OK_button");
        var appTitle = document.getElementById("About_RTLM");
        if (aboutDialog && aboutMsg) {
            aboutMsg.innerHTML = msg;
            if (appTitle) {
                appTitle.innerHTML = getMessage("appTitle");
            }
            if (copyrightMsg) {
                copyrightMsg.innerHTML = getMessage("appCopyright");
            }
            if (okButton) {
              okButton.innerHTML = getMessage("buttonLabelOK");
            }
            aboutDialog.show();
            if (okButton) {
              okButton.focus();
            }
        }
    */
    },
        
    /**
     * Hide the about dialog.
     */
    hideAbout: function() {
        var aboutDialog = dijit.byId('about_dialog');
        if (aboutDialog) {
            aboutDialog.hide();
        }
    },
    
    /**
     * Show the about dialog from key event.
     */
    showAboutFromKey: function(evt) {
      var charCode = (evt.which) ? evt.which : evt.keyCode
      // from <a> link, enter key follow the link
      if (charCode == 13) {
        this.showAbout();
      }
    },

    /**
     * Show the about dialog.
     */
    showAbout: function() {
        if (this._versionNumber != null) {
            this._showVersionDialog(getMessage("appVersion", this._versionNumber));
            return;
        }
        this._errorMessage = null;
        var callbackObj = {
            parent: this,
            successCallback: function(response) {
                this.parent._versionNumber = response;
                console.debug("Successful server response: " + response);
                this.parent._showVersionDialog(getMessage("appVersion", response));
            },
            errorCallback: function(error, errorMessage) {
                console.debug("Error server response: " + errorMessage);
                this.parent._showVersionDialog(getMessage("appfailGetVersion"));
            }
        };
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(callbackObj, "successCallback", "errorCallback");
        var admin = new com.ibm.rational.test.lm.admin();
        admin.getVersion(responseHandler);
    },
    
    /**
     * Return the message from the message bundle with the specified keyword.
     * Optional token arguments are for replacing place holders in the message
     * definition.  The place holders have the syntax "%number" with the number
     * starting from 1.  For example, if the message "myStr" is defined as
     * "This is %1 of %2 message".  getMessage("myStr", "one", "my many") will return
     * "This is one of my many message".
     */
    getMessage: function(keyword, tokens) {
        var args = [MessageBundleRTLM];
        for(var i = 0; i < arguments.length; i++){ 
             args.push(arguments[i]);
        }
		
        return MessageUtil.getMessage.apply(MessageUtil, args);
    },

    /**
     * Returns the message stored in the common message bundle.
     * @param {Object} keyword
     * @param {Object} tokens
     */
    getCommonMessage: function(keyword, tokens) {
        var args = [MessageBundle.COMMON];
        for( var i = 0; i < arguments.length; i++){ 
            args.push(arguments[i]);
        }
        return MessageUtil.getMessage.apply(MessageUtil, args);
    },
    
    // Dojo handles the high contrast mode by adding "dijit_ally" to
    // the document body class.
    //
    isHighContrast: function()
    {
//        dojo.a11y.checkAccessible();  // can't find this anywhere in dojo!?
        return document.body.className.search(/dijit_a11y/) != -1;
    },
	
	assertExists: function(value, valueName)
	{
		if(typeof value == "undefined") {
			throw new Error(valueName + " must not be undefined");
		}
	},
    
    getCurrentTimeString: function() {
        var now = new Date();
        return dojo.string.pad(now.getHours(), 2, '0') + ":" 
             + dojo.string.pad(now.getMinutes(), 2, '0') + ":" 
             + dojo.string.pad(now.getSeconds(), 2, '0');       
    }       
    
};
})();
