/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.adapter.rtc.web.ui.internal.config.PageProjectArea");

dojo.require("dijit._Templated");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.ValidationTextBox");

dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ExternalTeamServerDescription");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.rqm.defects.web.ui.internal.wizard.WizardPage");
dojo.require("com.ibm.team.repository.web.foundation.Status");
dojo.require("com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient");

dojo.require("com.ibm.rqm.adapter.rtc.web.ui.internal.RTCAdapterUIUtil");

(function(){

var RemoteTeamConcertClient				= com.ibm.rqm.adapter.rtc.web.client.internal.RemoteTeamConcertClient;
var ServiceResponseHandler				= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var Status								= com.ibm.team.repository.web.foundation.Status;
var ExternalTeamServerDescription		= com.ibm.team.repository.web.transport.ExternalTeamServerDescription;
var RTCAdapterUIUtil 					= com.ibm.rqm.adapter.rtc.web.ui.internal.RTCAdapterUIUtil;

dojo.declare("com.ibm.rqm.adapter.rtc.web.ui.internal.config.PageProjectArea", 
			 [com.ibm.rqm.defects.web.ui.internal.wizard.WizardPage, dijit._Templated], {

	_context: null,
	_rootNode: null,
	_projectAreaLabel: null,
	_projectAreaSelection: null,
	
	_cachedProjectAreas: null,
	_cachedProjectAreasServer: null,

	constructor: function() {
		this.setPageComplete(false);
	},

	// ---- WizardPage implementation --------------------------------------------------------------------------------------	

	getContentNode: function() {
		return this._rootNode;
	},
	
	aboutToShow: function(runWhenReady) {
		
		if (this._cachedProjectAreasServer !== this._context.url) {
			dojox.data.dom.removeChildren(this._projectAreaSelection);
			this._cachedProjectAreasServer= null;

			this.runWithProgress(dojo.hitch(this, function(monitor) {
				monitor.beginTask(RTCAdapterUIUtil.getMessage("labelFetchingProjectAreas"));

				var teamConcertServer= new ExternalTeamServerDescription(this._context.name, this._context.description, this._context.url);
				
				var serviceCallback= dojo.hitch(this, function(status, result) {
					try {
						var serviceResult= status;
						try {
							if (status.isOK()) {
								this._cachedProjectAreasServer= this._context.url;
								this._displayProjectAreas(result.collection);
								this._update();
							} else {
								serviceResult= new Status(Status.ERROR, RTCAdapterUIUtil.getMessage("labelFailedConnectServer"));
							}
						} finally {
							runWhenReady(Status.OK_STATUS);
						}
					} finally {
						monitor.done();
					}
				});
		
				RemoteTeamConcertClient.getProjectAreas(teamConcertServer, serviceCallback, { requestLabel: RTCAdapterUIUtil.getMessage("labelConfigureDefectsProvider") } );
			}));
		} else {
			runWhenReady(Status.OK_STATUS);
		}
	},
	
	aboutToHide: function() {
		this._context.projectArea= this._projectAreaSelection.value;
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.rqm.adapter.rtc.web", "ui/internal/config/templates/PageProjectArea.html"),
	widgetsInTemplate: true,

	postCreate: function() {
		this.connect(this._projectAreaSelection, "onchange", "_update");
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_update: function() {
		this.setPageComplete(dojo.isString(this._projectAreaSelection.value) && this._projectAreaSelection.value.length > 0);
	},
	
	_displayProjectAreas: function(projectAreaDescs) {
		
		console.group('_displayProjectAreas');
		console.dir(projectAreaDescs);
		console.groupEnd();
		
		dojo.forEach(projectAreaDescs, dojo.hitch(this, function(projectAreaDesc) {
			var projectAreaOption= document.createElement("option");
			projectAreaOption.appendChild(document.createTextNode(projectAreaDesc.projectArea.name.value));
			projectAreaOption.setAttribute("value", projectAreaDesc.projectArea.name.value);
			
			if (projectAreaDesc.projectArea.name.value == this._context.projectArea) {
				projectAreaOption.setAttribute("selected", "selected");
			}

			this._projectAreaSelection.appendChild(projectAreaOption);

		}));
	},

	__sentinel: null // terminates this class definition
});

com.ibm.rqm.adapter.rtc.web.ui.internal.config.PageProjectArea.create= function(context) {
	var args= com.ibm.rqm.defects.web.ui.internal.wizard.WizardPage.constructorArgs(RTCAdapterUIUtil.getMessage("labelProjectArea"), RTCAdapterUIUtil.getMessage("labelSelectProjectArea"));
	dojo.mixin(args, {
		_context: context
	});
	return new com.ibm.rqm.adapter.rtc.web.ui.internal.config.PageProjectArea(args, null);
};

})();
