/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.page.planning.PlanningPageController");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.rqm.planning.web.client.internal.QueryClient");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorFactory");

dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.mapper.TestPlanMapper");

(function()
{
	var ServiceResponseHandler 		= com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
	var QueryClient            		= com.ibm.rqm.planning.web.client.internal.QueryClient;
	var EditorFactory 				   = com.ibm.rqm.planning.web.ui.internal.util.EditorFactory;
	var TestScriptClient       		= com.ibm.rqm.planning.web.client.internal.construction.TestScriptClient;
	var TestCaseClient         		= com.ibm.rqm.planning.web.client.internal.construction.TestCaseClient;
	var TestGroupClient        		= com.ibm.rqm.planning.web.client.internal.construction.TestGroupClient;
	var PlanningPageClient     		= com.ibm.rqm.planning.web.client.internal.planning.PlanningPageClient;
	var TestPlanMapper				= com.ibm.rqm.planning.web.client.internal.mapper.TestPlanMapper;
	var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

	/**
	 * Main page controller
	 */
	dojo.declare(
		"com.ibm.rqm.planning.web.ui.internal.page.planning.PlanningPageController", 
		null, 
		{
			constructor: function(myView) 
			{
				this._view = myView;
			},
	
		   /*
			* Provide access to an EditorFactory instance.
			*/
			getEditorFactory: function() {
				return EditorFactory;
			},
			
			/*
			* The following methods support the URI navigation control
			*/
			
			/*
			* Initial Navigation.
			*/
			defaultView: function()
			{
				// Initial navigation is set 
				var name = getAuthenticatedContributor().name;
				
				if (name == "Joe Tester")
				{
					this.viewTestScripts({name: "My Test Scripts"});
				}
				else if (name == "Mary Testlead")
				{
					this.viewTestGroups({name: "My Test Case Groups"});
				}
				else
				{
					this.viewTestPlans({name: "My Test Plans"});
				}
			},

			/**************************************************************************************
			 * Search Methods
			 * 
			 * @param {Object} args
			 **************************************************************************************/
			genericSearch: function(args)
			{
				var responseHandler = new ServiceResponseHandler(this, "_doGenericSearch", "_error");
				QueryClient.getGenericSearch(responseHandler, args.searchStr);
			},
			
			_doGenericSearch: function(results)
			{
				this._view.showGenericSearchResults(results);
			},
			
			newQuery: function()
			{
				this._view.newQuery();
			},
			
			/**************************************************************************************
			 * Work Item methods
			 *
			 * @param {Object} args
			 **************************************************************************************/
			viewWorkItem: function(args)
			{
				this._view.showWorkItem(args);
			},
							
			/* 
			 * TEST PLAN URI ACTIONS
			 */	
			newTestPlan: function(kwArgs) 
			{
				this._view.showNewTestPlan(kwArgs.name);
			},		
			 			
			viewTestPlan: function(kwArgs) 
			{	
				console.debug("PlanningPageController: "+kwArgs.id+kwArgs.currentId);			
				var responseHandler = new ServiceResponseHandler(this, "_doViewTestPlan", "_error");
				PlanningPageClient.getTestPlansWithWorkItems(responseHandler, kwArgs);
			},
			
			copyTestPlan: function(kwArgs) 
			{	
				console.debug("PlanningPageController: "+kwArgs.id+kwArgs.currentId);			
				var responseHandler = new ServiceResponseHandler(this, "_doCopyTestPlan", "_error");
				PlanningPageClient.getTestPlansWithWorkItems(responseHandler, kwArgs);
			},
			
			_doViewTestPlan: function(plans)
			{
				if (plans.length > 1)
				{
					alert("PlanningPageController: too many Plans returned, should be 1.");
				}
				this._view.showTestPlan(plans[0]);
			},	
			
			_doCopyTestPlan: function(plans)
			{
				if (plans.length > 1)
				{
					alert("PlanningPageController: too many Plans returned, should be 1.");
				}
				this._view.copyTestPlan(plans[0]);
			},
				
			viewTestPlans: function(kwArgs) 
			{
				var self = this;
				var handler = 
				{
					success: function(plans)
					{
						self._view.showTestPlanList(plans, kwArgs.name);
					},
					error: function(e) 
					{
						self._error(e);
					}
				}
				
				var responseHandler = new TestPlanMapper(handler, "success", "error");			
				PlanningPageClient.getTestPlansSummary(responseHandler, kwArgs.name);				
			},	

			/* 
			 * TEST GROUP URI ACTIONS
			 */	
			newTestGroup: function(kwArgs) 
			{
				this._view.showNewTestGroup(kwArgs.name);
			},		
			 			
			viewTestGroup: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doViewTestGroup", "_error");
				if (kwArgs.uid!=null && kwArgs.revision!=null) {
					TestGroupClient.getTestGroupRevision(responseHandler, kwArgs.uid, kwArgs.revision);		
				} else {
					TestGroupClient.getTestGroup(responseHandler, kwArgs.id);		
				}		
			},
			
			copyTestGroup: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doCopyTestGroup", "_error");			
				TestGroupClient.getTestGroupRevision(responseHandler, kwArgs.uid, kwArgs.revision);		
			},
			
			_doViewTestGroup: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}			
				this._view.showTestGroup(result);	
			},				
			
			_doCopyTestGroup: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}			
				this._view.copyTestGroup(result);	
			},				
			
			viewTestGroups: function(kwArgs) 
			{
				var self = this;

				var handler = 
				{
					success: function(groups)
					{
						self._view.showTestGroupList(groups, kwArgs.name);
					},
					error: function(e) 
					{
						self._error(e);
					}
				}
				
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");			
//				var responseHandler = new ServiceResponseHandler(this, "_doViewTestGroups", "_error");			
				TestGroupClient.getTestGroups(responseHandler);				
			},	
			
//			_doViewTestGroups: function(groups) {
//				this._view.showTestGroupList(groups);
//			},				
			
			/*
			 * TEST PLAN GLOBAL SAVE
			 */
			saveTestPlan: function(){
				this._view.saveTestPlan();
			},
			
			/* 
			 * TEST SCRIPT URI ACTIONS
			 */				
			newTestScript: function(kwArgs) 
			{
				this._view.showNewTestScript(kwArgs.name);
			},
								
			viewTestScript: function(kwArgs) 
			{				
				var responseHandler = new ServiceResponseHandler(this, "_doViewTestScript", "_error");			
				TestScriptClient.getTestScript(responseHandler, kwArgs.id);
			},
			
			_doViewTestScript: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}
				this._view.showTestScript(result);				
			},
			
			viewTestScripts: function(kwArgs) 
			{
				var self = this;

				var handler = 
				{
					success: function(scripts)
					{
						self._view.showTestScriptList(scripts, kwArgs.name);
					},
					error: function(e) 
					{
						self._error(e);
					}
				}
				
				var responseHandler = new ServiceResponseHandler(handler, "success", "error");			
//				var responseHandler = new ServiceResponseHandler(this, "_doViewTestScripts", "_error");			
				TestScriptClient.getTestScripts(responseHandler);				
			},	
			
//			_doViewTestScripts: function(scripts) {
//				this._view.showTestScriptList(scripts);
//			},
			
			/* 
			 * SIZING RECORD URI ACTIONS
			 */
         /*
			viewTestSizing: function(kwArgs) 
			{	
				var responseHandler = new ServiceResponseHandler(this, "_doViewTestSizing", "_error");			
				SizingClient.getArtifactWithSizing(responseHandler, kwArgs);
			},
			
			_doViewTestSizing: function(result) 
			{
				if ( dojo.isArray(result) ) 
				{
					result = result[0];	
				}
				this._view.showTestSizing(result);				
			},
			 */
         
			/*
			 * EXCUTION WORK ITEM URI ACTIONS
			 */
			generateEWI: function(param) {
				this._view.generateEWI(param);
			},
			
			showConfigurationList: function(param){	
				this._view.showConfigurationList(param);
			},
			
			showEWILists: function(param){
				this._view.showEWILists(param);
			},
			
			backToDefaultView: function(){
				this._view.openLeftNav();
				this.defaultView();
			},
			
			_error: function(error) 
			{
				alert("PlanningPageController: Error invoking service; message: " + error.message);
			}
		}
	);
})();
