/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.view.common.CollapsibleArea");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

(function() { // module container

var	ImageCache			= com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var	TooltipBuilder		= com.ibm.asq.common.web.ui.internal.util.TooltipBuilder;

dojo.declare(
	"com.ibm.asq.common.web.ui.internal.view.common.CollapsibleArea",
	[dijit._Widget, dijit._Templated], 
	{
		
	templatePath: dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/templates/common/CollapsibleArea.html"),
	//templateCssPath: dojo.uri.dojoUri("../com.ibm.asq.common.web/ui/internal/view/templates/common/CollapsibleArea.css"),
	
	constructor: function() {
		///////////////////////////////////////////////////////////////////////////////////
		// Initialization parameters	
		//////////////////////////////////////////////////////////////////////////////////	
		this.label = null;
		this.labelTooltip = null;
		//this.id = null;
		this.sectionWidgets = null;
		this.open = false;		
		this.showTwistie = true;
		this.showBorder = false;
		this.flagStatus = 0;
		
		this.editorOpen = false;
		this.editorAvailable = true;
		this.showActionsForEditor = true;
		
		//////////////////////////////////////////////////////////////////////////////////
		// Actions
		//////////////////////////////////////////////////////////////////////////////////
		this.actions = [];	//action objects should include: 'text' && ('href' || ('obj' && 'function'))
		
		//////////////////////////////////////////////////////////////////////////////////
		// States
		//////////////////////////////////////////////////////////////////////////////////
		this._isExpanded = false;
		this._isEditorOpen = false;
		
		//////////////////////////////////////////////////////////////////////////////////
		// Expand/Edit options
		//////////////////////////////////////////////////////////////////////////////////
		this.isExpandable = true;
		this.isEditable = true;
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Called on creation
	///////////////////////////////////////////////////////////////////////////////////
	postCreate: function() { },
	
	initialize: function(args) {
		//allow mixin of args on initialize()
		for (var element in args) {
			this[element] = args[element];
		}

		//if editable (editor available) connect proper events
		if (this.editorAvailable && this.editorAvailable == true) {
			//this._editImg.src = ImageCache.EDIT;
			dojo.connect(this.labelSpan, "onclick", this, this._toggleEditor);
			//dojo.connect(this._editImg, "onclick", this, this._toggleEditor);
			dojo.addClass(this.labelSpan, "cursor-pointer");

			if (this.sectionWidgets) {
	            if ( this.sectionWidgets.length == 1 ) {
	                var editor = this.sectionWidgets[0];
	                if (editor && editor["setNameListenerNode"]) {
	                    editor.setNameListenerNode(this.labelSpan);
	                }
	            }
			}
			
		} else {
			this.editorOpen = false;
		}
		this._initializeWidget();
	},

	_initializeWidget: function() {
		//set initial collapsed/expanded state
		if (this.open && this.open == true) {
			var children = this.sectionWidgets[0].domNode.childNodes;
			for (var i = 0; i < children.length; i++) {
				var child = children[i];
				if ((child.className != null) && (child.className != "")) {
					if (dojo.hasClass("hierarchy") && dojo.hasClass("hidden") ) {
						dojo.removeClass("hierarchy");
					}
				}
			}
			this.expand();
		}
		else
		{
			this.collapse();
		}
		//set initial editor state
		if (this.editorOpen) {
			if (this.showBorder) {
				this._titleRow.className = "title-row-expanded";
				this._sectionGroupBorderTop.className = "border-top";
				this._sectionGroupBorderBottom.className = "border-bottom hidden";		
				this._sectionGroupContainer.className = "section-group-collapsed";
			} else {
				var children = this.sectionWidgets[0].domNode.childNodes;
				for (var i = 0; i < children.length; i++) {
					var child = children[i];
					if ((child.className != null) && (child.className != "")) {
						if (dojo.hasClass("editor") &&  dojo.hasClass("shaded") && dojo.hasClass("hidden") ) {
							child.className = "editor shaded bottom-border";
							this.labelSpan.className = "label cursor-pointer shaded";
						}
					}
				}
			}
		}
		
		this._updateTwistie();
		this._updateFlag();
		
		if (this.icon) {
			this.iconImg.src = this.icon;
		}

		this.setLabel(this.label);
		if ( this.labelTooltip != null ) {
			this.setLabelTooltip(this.labelTooltip);	
		}
		
		if (this.sectionWidgets) {
			for (var i= 0; i < this.sectionWidgets.length; i++) {
				this.sectionsDiv.appendChild(this.sectionWidgets[i].domNode);
			}
		}
		this._showActions();		
		
	},
			
	///////////////////////////////////////////////////////////////////////////////////
	// API
	///////////////////////////////////////////////////////////////////////////////////
	collapse: function() {
		this._titleRow.className = "title-row-collapsed";
		if (!this.editorOpen || !this.showActionsForEditor) {
			this._actionsBar.className = "hidden";	
		}		
		if (!this.editorOpen) {
			this._sectionGroupContainer.className = this.showBorder ? "section-group-collapsed top-border-line" : "section-group-collapsed" ;
		}
		this._sectionGroupBorderTop.className = "";
		this._sectionGroupBorderBottom.className = "";		
	},
	
	expand: function() {
		if (this.showBorder) {
			this._titleRow.className = "title-row-expanded";
			this._sectionGroupBorderTop.className = "border-top";
			this._sectionGroupBorderBottom.className = "border-bottom hidden";		
		}
		this._actionsBar.className = "actions-bar";
		this._sectionGroupContainer.className = "section-group-expanded";
	},
	
	isExpanded: function() {
		return this._isExpanded;
	},
	
	setExpression: function(/* ExpressionDTO[] */ expressions) {
		var hasExpression= false;
		for (var i= 0; i < this.sectionWidgets.length; i++) {
			hasExpression|= this.sectionWidgets[i].setExpression(expressions);
		}
		this._setDisplay(hasExpression);
	},
	
	addAction: function(action) {
		this.actions.push(action);
		this._showActions();
	},
	
	registerAction: function(text, href, obj, method) {
		var action = {
			text: text, href: href, obj: obj, method: method
		}
		this.addAction(action);
	},
	
	setLabel: function(labelText)
	{
		this.label = labelText;
		this.labelSpan.innerHTML = this.label;		
	},

	setLabelTooltip: function(tooltip)
	{
		TooltipBuilder.addLabelTooltip(tooltip, this.labelSpan, this);
	},
	
	setWorkItemUrl: function(wi, name, url)
	{
		dojox.data.dom.removeChildren(this._workItemUrl);
		
		var anchor = document.createElement("A");
		anchor.id = this.label;
		anchor.className = "";
		anchor.href = url;
		var tn = document.createTextNode(name);
		anchor.appendChild(tn);
		this._workItemUrl.appendChild(anchor);
		
		TooltipBuilder.addWorkItemTip(wi, anchor);
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Expand/Edit option capability setters
	///////////////////////////////////////////////////////////////////////////////////
	
	//canEditor: function(value) {
	setEditable: function(value) {
		//can be edited
		if(value == true) {
			this.isEditable = true;
			this._editImg.src = ImageCache.EDIT;
		}
		//can't be edited
		if(!value) {
			this.isEditable = false;
			this._editImg.src = "";			
		}
	},
	
	//canExpand: function(value) {
	setExpandable: function(value) {
		//can be expanded
		if(value && value == true) {
			this.isExpandable = true;
			this.twistieImg.src = this._isVisible() ? ImageCache.TWISTIE_OPEN : ImageCache.TWISTIE_CLOSED;
		} else {
			this.isExpandable = false;
			this.twistieImg.src = "";
		}
	},
	
	closeEditor: function() {
		var section = this.labelSpan.parentNode.parentNode.parentNode.childNodes[3].childNodes[0];
		for (var i = 0; i < section.childNodes.length; i++)
		{
			var child = section.childNodes[i];
			if ((child.className != null) &&(child.className != "")){
				if (child.className == "editor shaded bottom-border")
				{
					child.className = "editor shaded hidden";
					this.labelSpan.className = "label";
				}
			}
		}
	},
	
	needTitleRow: function(show) {
		if(!show) {
			this._titleRow.className = "hidden";
		}
		else
			this._titleRow.className = "title-row-collapsed";
	},	
	
	///////////////////////////////////////////////////////////////////////////////////
	// Helpers
	///////////////////////////////////////////////////////////////////////////////////
	_updateTwistie: function() {
		if (this.showTwistie) {
			this.twistieImg.src = this.isExpanded() ? ImageCache.TWISTIE_OPEN : ImageCache.TWISTIE_CLOSED;
		} else {
			this.twistieImg.src = "";
		}
	},
	
	_updateFlag: function() {
		switch(this.flagStatus) {
			case 0: img = ImageCache.FLAG_STATUS_INCOMPLETE;
				break;
			case 1: img = ImageCache.FLAG_STATUS_REQ_NOT_COMP;
				break;
			case 2: img = ImageCache.FLAG_STATUS_DRAFT_COMP;
				break;
			case 3: img = ImageCache.FLAG_STATUS_COMPLETED;
				break;
			default:
				//default to NO flag
				img = "";
		}		
		//this.flagImg.src = img;
	},
	
	_showActions: function() {
		dojox.data.dom.removeChildren(this._actionsBar);
		if ( this.actions != null && this.actions.length > 0 ) {
			for ( var i = 0; i < this.actions.length; i++ ) {
				var action = this.actions[i];
				var actionLink = document.createElement("a");
				actionLink.innerHTML = action.text;
				if ( action.href ) {
					actionLink.href = action.href;
				} else if ( action.obj && action.method ) {
					dojo.connect(actionLink, "onclick", action.obj, action.method);	
				}
				this._actionsBar.appendChild(actionLink);
			}
		}
	},
	
	_setDisplay: function(show) {
		//dojo.html.setDisplay(this.sectionsDiv, show);
		this._updateTwistie();
	},
	  
	///////////////////////////////////////////////////////////////////////////////////
	// Event handlers
	///////////////////////////////////////////////////////////////////////////////////
	_toggleExpand: function(e) {
		if(this.isExpandable) {
			var section = e.target.parentNode.parentNode.parentNode.childNodes[3].childNodes[0];
			for (var i = 0; i < section.childNodes.length; i++)
			{
				var child = section.childNodes[i];
				if ((child.className != null) &&
					(child.className != ""))
				{
					if (child.className == "hierarchy hidden")
					{
						this._isExpanded = true;
						child.className = "hierarchy";
						this.expand();
					}
					else if (child.className == "hierarchy")
					{
						this._isExpanded = false;
						child.className = "hierarchy hidden";
						this.collapse();
					}
				}
			}
			this._updateTwistie();
		}
	},
			
	_toggleEditor: function(e)
	{
		if(this.isEditable) {
		e.preventDefault();
		
		var section = e.target.parentNode.parentNode.parentNode.childNodes[3].childNodes[0];
		for (var i = 0; i < section.childNodes.length; i++) {
			var child = section.childNodes[i];
			
			if ((child.className != null) && (child.className != "")) {
				if (child.className == "editor shaded bottom-border") {
					//hide the editor
					child.className = "editor shaded hidden";
					this.labelSpan.className = "label cursor-pointer";
					
					if (!this._isExpanded) {
						this._sectionGroupContainer.className = this.showBorder ? "section-group-collapsed top-border-line" : "section-group-collapsed";
						this._actionsBar.className = "hidden";	
					}					
					this.editorOpen = false;					
				}
				else if (child.className == "editor")
				{
					//hide the editor
					child.className = "editor hidden";
					this.labelSpan.className = "label cursor-pointer";
					this._titleRow.className = "title-row-collapsed";
					if (!this._isExpanded) {
						this._actionsBar.className = "hidden";	
					}
					this._sectionGroupContainer.className = this.showBorder ? "section-group-collapsed top-border-line" : "section-group-collapsed";
					this._sectionGroupBorderTop.className = "";
					this._sectionGroupBorderBottom.className = "";
					this.editorOpen = false;	
				}
				else if (child.className == "editor hidden")
				{
					//show the editor
					child.className = "editor";					
					if (this.showBorder)
					{
						this._titleRow.className = "title-row-expanded";
						if (this.showActionsForEditor) {
							this._actionsBar.className = "actions-bar";	
						}
						this._sectionGroupBorderTop.className = "border-top";
						this._sectionGroupBorderBottom.className = "border-bottom hidden";		
						this._sectionGroupContainer.className = "section-group-collapsed";
					}
					//this.labelSpan.className = "label";
					this.editorOpen = true;
				}
				else if (child.className == "editor shaded hidden")
				{
					//show the editor
					child.className = "editor shaded bottom-border";
					this.labelSpan.className = "label cursor-pointer shaded";
					this._sectionGroupContainer.className = "section-group-expanded";
					if (this.showActionsForEditor) {
						this._actionsBar.className = "actions-bar";	
					}
					this.editorOpen = true;
				}
			}
		}
		}
	}
});

})();
