/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Loading com.ibm.rqm.planning.web.ui.internal.view.common.ContentArea");
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.ContentArea");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.TabContainer");
//xojo.require("dijit.layout.SplitContainer");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function() 
{
    var ACTION_ROOT = "com.ibm.rqm.planning.home.";
    var DISPATCHER  = "actionDispatcher";
    var HOME_ACTION = "viewUserHome";
	var ImageCache 	= com.ibm.rqm.planning.web.ui.internal.view.common.ImageCache;
	var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;
    var EditorUtil = com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	
    var capitalize = function(word){
        return word.substring(0,1).toUpperCase() + word.substring(1);
    };

    var size = function(widget, dim){
        // size the child
        widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

        // record child's size, but favor our own numbers when we have them.
        // the browser lies sometimes
        dojo.mixin(widget, dojo.marginBox(widget.domNode));
        dojo.mixin(widget, dim);
    };

    // Subclass to initiate destroy on our content widget when the tab is destroyed.
    dojo.declare(
        "ContentArea.ContentPane",
        [dijit.layout.ContentPane],
        {
            destroy: function()
            {
                this.inherited("destroy",arguments);
                
                // clean up the widget reference.
                if(this.contentWidget) {
                    if (this.contentWidget.destroyRecursive && dojo.isFunction(this.contentWidget.destroyRecursive)) {
                         this.contentWidget.destroyRecursive();
                    }    
                    delete this.contentWidget;
                }
            }
        }
    );
    
    // Subclass from the base TabContainer to override the layout method to not
    // to force a size change on the tab children. This allows the current dsiplaying
    // tab to assume its full height and would force the entire window to grow/shrink.
    // 
    dojo.declare(
        "ContentArea.TabContainer",
        [dijit.layout.TabContainer],
        {
			lastSelectedPage: null,
			
			postCreate: function(){	
                dijit.layout.TabContainer.superclass.postCreate.apply(this, arguments);
				this.tablist = new ContentArea.TabContainerController(
				{
					id: this.id + "_tablist",
					tabPosition: this.tabPosition,
					doLayout: this.doLayout,
					containerId: this.id
				});
                this.tabListParent.appendChild(this.tablist.domNode);		
			},
			
            myChldrenLayout: function(/*DomNode*/ container, /*Object*/ dim, /*Object[]*/ children){
                /**
                 * summary
                 *      Layout a bunch of child dom nodes within a parent dom node
                 * container:
                 *      parent node
                 * dim:
                 *      {l, t, w, h} object specifying dimensions of container into which to place children
                 * children:
                 *      an array like [ {domNode: foo, layoutAlign: "bottom" }, {domNode: bar, layoutAlign: "client"} ]
                 */
        
                // copy dim because we are going to modify it
                dim = dojo.mixin({}, dim);
        
                dojo.addClass(container, "dijitLayoutContainer");
        
                // Move "client" elements to the end of the array for layout.  a11y dictates that the author
                // needs to be able to put them in the document in tab-order, but this algorithm requires that
                // client be last.
                children = dojo.filter(children, function(item){ return item.layoutAlign != "client"; })
                    .concat(dojo.filter(children, function(item){ return item.layoutAlign == "client"; }));
        
                // set positions/sizes
                dojo.forEach(children, function(child){
                    var elm = child.domNode,
                        pos = child.layoutAlign;
        
                    // set elem to upper left corner of unused space; may move it later
                    var elmStyle = elm.style;
                    elmStyle.left = dim.l+"px";
                    elmStyle.top = dim.t+"px";
                    elmStyle.bottom = elmStyle.right = "auto";
                    
                    /*
                     * HACK
                     * By not using position:absolute (defined in dijitAlignClient css) and
                     * use "relative", this allows the content pane to be layout properly below
                     * the tab buttons and most importantly, the client height to not be clipped.
                     * The natural size of the content pane would force the scrollbar on the 
                     * window.
                     */
                    if(pos != "client") 
                        dojo.addClass(elm, "dijitAlign" + capitalize(pos));
                    else 
                        elmStyle.position = "relative";
                    
                    // set size && adjust record of remaining space.
                    // note that setting the width of a <div> may affect it's height.
                    if(pos=="top" || pos=="bottom"){
                        size(child, { w: dim.w });
                        dim.h -= child.h;
                        if(pos=="top"){
                            dim.t += child.h;
                        }else{
                            elmStyle.top = dim.t + dim.h + "px";
                        }
                    }else if(pos=="left" || pos=="right"){
                        size(child, { h: dim.h });
                        dim.w -= child.w;
                        if(pos=="left"){
                            dim.l += child.w;
                        }else{
                            elmStyle.left = dim.l + dim.w + "px";
                        }
                    }else if(pos=="client"){
                        /*
                         * HACK
                         * Don't resize the client node. This will let the
                         * content to assume its desired height. This is to 
                         * avoid having the content to create an inner scrollbar
                         */
                        //size(child, dim);
                    }
                });
            },
            
            layout: function(){
                // Summary: Configure the content pane to take up all the space except for where the tabs are
                if(!this.doLayout){ return; }
        
                // position and size the titles and the container node
                var titleAlign=this.tabPosition.replace(/-h/,"");
                var children = [
                    // we're hosting the tab list in a outside div.
                    // This is to avoid the layout code to push the container
                    // area (client) down below the tab button list. 
                    // When the content area is pushed down, the bottom of the
                    // container would become clipped.
                    //
                    //{domNode: this.tablist.domNode, layoutAlign: titleAlign},
                    {domNode: this.containerNode, layoutAlign: "client"}
                ];
                
                // use this modified children layout routine to not set the size
                // on the client node.
                this.myChldrenLayout(this.domNode, this._contentBox, children);
                
                // Compute size to make each of my children.
                // children[1] is the margin-box size of this.containerNode, set by layoutChildren() call above
                this._containerContentBox = dijit.layout.marginBox2contentBox(this.containerNode, children[0]);
        
                if(this.selectedChildWidget){
                    this._showChild(this.selectedChildWidget);
                    if(this.doLayout && this.selectedChildWidget.resize){
                        this.selectedChildWidget.resize(this._containerContentBox);
                    }
                }
            },
            
            // Summary:
            //   Override the TabContainer/StackContainer's removeChild to show the
            //   last tab page instead of showing the first tab page when a tab page
            //   is being removed.
            //
            removeChild: function(/*Widget*/ page){
                var needToUpdateTab = (this.selectedChildWidget === page);

                dijit._Container.prototype.removeChild.apply(this, arguments);
        
                // If we are being destroyed than don't run the code below (to select another page), because we are deleting
                // every page one by one
                if(this._beingDestroyed){ return; }
        
                if(this._started){
                    // this will notify any tablists to remove a button; do this first because it may affect sizing
                    dojo.publish(this.id+"-removeChild", [page]);
        
                    // in case the tab titles now take up one line instead of two lines
                    this.layout();
                }
                
                var children = this.getChildren();
                
                // If the tab being removed is the current tab, or
                // we only has one tab left, select the last tab.
                // But only select a tab if we have children. This should
                // be always the case, since we alway creates the home
                // tab. But in case we ever allow the home tab to be removed,
                // we won't get an exception here.
                //
                if((children.length > 0) && (needToUpdateTab || children.length == 1)){
                    this.selectChild(children[children.length - 1]);
                }
            },
			
			selectChild: function(/*Widget*/ page){
				//remove all css change
				var children = this.getChildren();
				if(children && children.length){
					for(var i=0; i< children.length; i++){
						children[i].controlButton.domNode.style.borderTopColor = "";
						children[i].controlButton.domNode.style.borderBottomColor = "";
						children[i].controlButton.domNode.style.backgroundImage = "";
						children[i].controlButton.containerNode.style.fontWeight = "";
					}
				}
				
				page.controlButton.domNode.style.borderBottomColor = "#fff";
							 
//				alert("page"+page.domNode.getAttribute("class"));
				var cc =  page.domNode.firstChild;
//				if (cc.getAttribute("dojoattachpoint") != null) 
//					alert("page first child:" + cc.getAttribute("dojoattachpoint"));
					
				cc.style.display = "block";
				this.inherited("selectChild",arguments); 
				page.controlButton.domNode.style.borderTopColor = "#7898a0";
				page.controlButton.containerNode.style.fontWeight = "bold";
				
				this.lastSelectedPage = page;
        	}    
    	}
	);
	
    dojo.declare("ContentArea.TabContainerController",	dijit.layout.TabController,	{
		buttonWidget: "ContentArea.TabContainerButton",
		
		postMixInProperties: function(){
			this["class"] = "dijitTabLabels-" + this.tabPosition + (this.doLayout ? "" : " dijitTabNoLayout");
			this.inherited("postMixInProperties",arguments);
		},
		
		onCloseButtonClick: function(/*Widget*/ page){
			if(page.contentWidget && page.contentWidget.isDirty && page.contentWidget.isDirty())
			{
				var self = this;
				var arg = arguments;
				var args = {
					title: Constants.getMessage("labelConfirmation"),
					message: Constants.getMessage("msgCloseTab"),
					yesCallback: function(){
						self.inherited("onCloseButtonClick", arg);
					},
					noCallback: null
				}
				var dialog = new YesNoDialog(args);
	            dialog.setSecondaryTitle(args.title);
	            dialog.open();
			}else{
				this.inherited("onCloseButtonClick", arguments);
			}
		},
		
		//Override onkeypress so that when edit boxes have the focus, the ctrl arrow
		//keys will work properly within the edit boxes.
		onkeypress: function(/*Event*/evt){
			var e = evt;

			if( !evt )
			{
				e = window.event;
			}
			
			var target = this._getTarget( e );
			
			if( this._isEditableInputArea( target ) )
			{
				return;
			}
				
			this.inherited( "onkeypress", arguments );
		},
		
		_getTarget: function( e )
		{
			var target = null;
			if( e.originalTarget )
				target = e.originalTarget;
			else if( e.target )
				target = e.target;
			else if( e.srcElement )
				target = e.srcElement;
				
			return( target );
		},
		
		_isEditableInputArea: function( target )
		{
			if (target) 
			{
				if (target.tagName.toLowerCase() == "input") {
					return (true);
				}
				
				var parent = target;
				while (parent) {
					if (parent.id) {
						var widget = dijit.byId(parent.id);
						if (widget && (widget.declaredClass == "com.ibm.asq.common.web.ui.internal.widgets.Textarea")) 
							return (true);
					}
					
					parent = (dojo.isIE) ? parent.parentElement : parent.parentNode;
				}
			}
			
			return( false );
		}	
	});

	dojo.declare("ContentArea.TabContainerButton",	dijit.layout._TabButton, {
	
		templateString:"<div dojoAttachEvent='onclick:onClick,onmouseenter:_onMouse,onmouseleave:_onMouse'>\n    <div class='dijitTabInnerDiv' dojoAttachPoint='innerDiv'>\n        <span dojoAttachPoint='containerNode,focusNode'><a dojoAttachPoint='link' style='color: #000; text-decoration: none;'>${!label}</a></span>\n        <span dojoAttachPoint='closeButtonNode' class='closeImage' dojoAttachEvent='onmouseenter:_onMouse, onmouseleave:_onMouse, onclick:onClickCloseButton' stateModifier='CloseButton'>\n            <span dojoAttachPoint='closeText' class='closeText'>x</span>\n        </span>\n    </div>\n</div>\n",

		postCreate: function(){
			this.closeButtonNode.style.display="none";
			this.inherited("postCreate",arguments); 
			dojo.setSelectable(this.containerNode, false);
			this.domNode.style.borderTopColor = "#7898a0";
			this.containerNode.style.fontWeight = "bold";
			this.containerNode.style.color = "#000";
			
			if (EditorUtil.isDirectionRtl() && dojo.isIE) {
				//set button's width in RTL direction in IE
				var maxLen = Math.max(0, dojo.marginBox(this.containerNode).w);
				this.innerDiv.style.width = maxLen + 'px';
			}
			if(!dojo.isIE)
				this.link.setAttribute("href", "javascript:{}");     
			else {
				dojo.connect(this.link, "onclick", this, "onClick");
			}
			
			dojox.data.dom.removeChildren(this.link);
			this.link.appendChild(this._createAllySpan());
			this.link.appendChild(document.createTextNode(this.label));
		},
		
		_onMouse : function(/*Event*/ event){
			var mouseNode = event.currentTarget;
			if(mouseNode && mouseNode.getAttribute){
				this.stateModifier = mouseNode.getAttribute("stateModifier") || "";
			}

			if(!this.disabled){
				switch(event.type){
					case "mouseenter":	
					case "mouseover":
						this._hovering = true;
						this._active = this._mouseDown;
						this.closeButtonNode.style.display = "";
						break;
					case "mouseout":
					case "mouseleave":
						this._hovering = false;
						this._active = false;
						if(mouseNode.className != "closeImage")
							this.closeButtonNode.style.display = "none";
						break;
					case "mousedown" :
						this._active = true;
						this._mouseDown = true;
						var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
							this._active = false;
							this._mouseDown = false;
							this._setStateClass();
							this.disconnect(mouseUpConnector);
						});
						if(this.isFocusable()){ this.focus(); }
						break;
				}
				this._setDivStateClass(mouseNode);
			}
		},
		
        setChecked: function(args)
        {
            this.inherited("setChecked", arguments);
            if(this.checked) {
                this.domNode.style.borderBottomColor = "#fff";
            }
        },
	
		//@Overridden to remove behavior in base Button where the style display:none is set
		//and then back to its original style as a workaround to some mozilla table behavior;
		//caused concurrency issues with leaving the tab blank; we don't use tables, safe to override
		setLabel: function(content) {
			this.label = content;
			dojox.data.dom.removeChildren(this.link);
			this.link.appendChild(this._createAllySpan());
			this.link.appendChild(document.createTextNode(this.label));
			
	        if (this.showLabel == false){
                this.titleNode.title=dojo.trim(this.containerNode.innerText || this.containerNode.textContent);
	        }			
		},
        
		//change focusNode from TabButton to ContentArea. 
		setConentFocus: function(content) {
			if(content)
				 this.focusNode = content;
		},
		
		_setDivStateClass: function(mouseNode){
			this._setStateClass();
			
			if (mouseNode.tagName != "SPAN") {
				if (!this.checked && !this._hovering) {
					this.domNode.style.borderTopColor = "";
					this.domNode.style.borderBottomColor = "";
					this.domNode.style.backgroundImage = "";
					this.containerNode.style.fontWeight = "";
				}
				if (this._hovering && !this.checked) {
					this.domNode.style.backgroundColor = "#fff";
					this.domNode.style.backgroundImage = "none";
					this.domNode.style.borderColor = "#c8c8c8";
					this.containerNode.style.fontWeight = "";
				}
			}
			this.containerNode.style.color = "#000";
		},
		
		_createAllySpan: function() {
			var node = document.createElement("span");
			node.style.left = "-3000px";
			node.style.position = "absolute";
			node.style.display = "inline";
			node.innerHTML = Constants.getMessage("labelTabAlly");
			return node;
		}
	});
	
	dojo.declare(
		"com.ibm.rqm.planning.web.ui.internal.view.common.ContentArea",
		[dijit._Widget, dijit._Templated],
		{
            // Create a separate div area for the tab container's tab control list
            // and the tab content area. 
            templateString: "<div dojoAttachPoint=\"_contentArea\">" +
                            "    <div class=\"tab-container-button-area\" dojoAttachPoint=\"_tabButtonArea\">" +
                            "    </div>" +       
                            "    <div class=\"tab-container-content-area\" dojoAttachPoint=\"_tabContentArea\">" +
                            "    </div>" +      
                            "</div>",       
	
			constructor: function()
			{
			},
	
			postCreate: function(args)
			{
			},
			
			initialize: function(args)
			{
				dojo.connect(window, "onbeforeunload", this, "_onBeforeUnload");
//				window.onbeforeunload = dojo.hitch(this, this._onBeforeUnload);		// CML: hack

                // initialize object to false to enforce the onBeforeUnload confirmation
                //dojo.setObject("com-ibm-rqm-planning-web-logout-in-progress", false);
				this._tabIndex = 0;
				this._contentDiv = null;
				
				dojo.addClass(this._contentArea, "tabbed-content-area");

			    var tabDiv = document.createElement("div");
			    tabDiv.style.width= "90%";
			    this._tabContentArea.appendChild(tabDiv);
			    
                var tabArgs = (args && args.tabContainerStyle) ? 
                    {style: args.tabContainerStyle} : {style: "min-height: 700px"};
                
                tabArgs['tabListParent'] = this._tabButtonArea;
			    this._tabs = new ContentArea.TabContainer(tabArgs , tabDiv);
			},
			
            getTabController: function() {
                return this._tabs ? this._tabs.tablist : null;
            },			

			_onBeforeUnload: function(e)
			{
                // Set a returnValue for the event will trigger the Jazz handler to present
                // a confirmation dialog to leave the page.  We don't want the confirmation
                // on logout.
                if (!dojo.getObject("com-ibm-rqm-planning-web-logout-in-progress") && this._hasUnsavedChanges())
                    e.returnValue = Constants.getMessage("promptLostUnsavedChanges");
			},
			
			_hasUnsavedChanges: function() {
				var tabs = this._tabs.getChildren();
				for (var i = 0; i < (dojo.isArray(tabs) ? tabs.length : 0); i++) {
                    var contentWidget = tabs[i] ? tabs[i].contentWidget : null;  
					if (contentWidget && dojo.isFunction(contentWidget.isDirty) && contentWidget.isDirty()) {
						return true;
					}
				}				
				return false;
			},
			
			_confirmClose: function()
			{
				return confirm(Constants.getMessage("promptCloseTab"));
			},
			
			addPage: function(args, tabPosition)
			{
				if (tabPosition == undefined)
				{
					tabPosition = "last";
				}
				
				var tabArgs =
				{
				   title: args.name,
//				   id: "tab" + this._tabIndex,
				   index: this._tabIndex,
				   refreshOnShow: true,
//				   onClose: dojo.hitch(this, "_confirmClose"),
				   parseOnLoad: false,
				   onHide: this._onHide
				};

				if (args.closable)
				{
				   tabArgs.closable = args.closable;
				}
				
				if(args.browserTitle)
				{
					tabArgs.browserTitle = args.browserTitle;
				}

				var tab = new ContentArea.ContentPane(tabArgs,
				                                      document.createElement("div"));
				
				//remove overflow: auto; style. Hide Tab's scrollbar  
				dojo.removeClass(tab.domNode, "dijitContentPane");
                dojo.addClass(tab.domNode, "tab-pane-content");
                
                tab = this._setTabContent(tab, args.content);		
				
				//close event
                var controller = this.getTabController();                                      
                if (controller && dojo.isFunction(args.onFocus)) {
                    dojo.connect(controller, "onButtonClick", null, args.onFocus);
                }                                                      
                if (controller && dojo.isFunction(args.onClose)) {
                    dojo.connect(controller, "onCloseButtonClick", null, args.onClose);
                }
                
				tab.startup();

				this._tabs.addChild(tab, tabPosition);
				this._tabs.startup();
				
				if (!args.noSelect)
				{
					this._tabs.selectChild(tab);
				}
				
				this._tabIndex++;
                
                // associate the page with the curernt id that created this page.
                
				var uri = "";
				if (args.uri)
				{
					uri = args.uri;
				}
				else
				{
					uri = window.location.hash;
				}
    
	            tab.uri = uri; 
				tab.controlButton.setConentFocus(this._contentDiv);
//				window.onbeforeunload = dojo.hitch(this, this._onBeforeUnload);		// CML: hack
             
                // initialize the scroll top position for the tab. Always put
                // the new tab at the initial scroll position at the top. 
                window.scrollTo(0, 0);                
                
                return tab;
			},
			
			_setTabContent: function(tab, content) {
                if (dojo.isString(content))
                {
                    // content is a text string, set it in a div as the content
                    var contentDiv = document.createElement("div");
                    contentDiv.id = "tab" + this._tabIndex;
                    contentDiv.innerHTML = content;
                    tab.setContent(contentDiv);  
					this._contentDiv = contentDiv;
                }
                else if (content && content.domNode)
                {
                    // content is a widget, store the widget and set its domNode as the content
                    tab.contentWidget = content;
                    tab.setContent(content.domNode);
					this._contentDiv = content.domNode;
					content.page = tab;
                }
                else
                {
                    tab.setContent(content);
					this._contentDiv = content;
                }		
				return tab;
			},
            
            // Summary:
            //    Bring a tab to the front of the stack given its uri ID.
            //    
            showPageByUri: function(uri)
            {
				var tab = this.getPageByUri(uri);
				if (tab != null) {
					this._tabs.selectChild(tab);
				}
            },
			
			// Summary:
			//   Find & return the page/tab corresponding to a given URI
			//
			getPageByUri: function(uri) {
				var tabs = this._tabs.getChildren();
                for(var i = 0; i < tabs.length; i++) {
                    if(tabs[i].uri == uri) {
                        return tabs[i];
                    }
                }				
				return null;
			},
            
            // Summary:
            //   Return true if a tab with the specified uri ID is already in the tab stack. 
            //    
            hasPage: function(uri)
            {
                return (this.getPageByUri(uri) != null);
            },
			
			hasHomeTab: function()
			{
				var uri = "#action=" + ACTION_ROOT + DISPATCHER + "&subAction=" + HOME_ACTION;
				
				return (this.hasPage(uri));
			},

            // Summary:
            //   Updates a page's uri id to the new specified uri. If updated, returns true, otherwise false.
            //    
            updatePageUri: function(uri, newUri)
            {		
				var tab = this.getPageByUri(uri);
				if (tab != null) {
					tab.uri = newUri;
					return true
				}
                return false;
            },

            // Summary:
            //   Updates the page's tab button's label given its URI id with the specified new label.
            //    
            updatePageLabel: function(uri, label, iconClass, dirty, skipCompactLabel)
            {
				var tab = this.getPageByUri(uri);
				if (tab != null) {
	                if (label == null || label.length == 0) {
	                    // no label specified so must be toggling the tab dirty bit
	                    label = tab.controlButton.label;
	                    if (dirty) {
	                        // ensure the tab is not already marked dirty before setting it
	                        if (label.substring(label.length - 1) != "*") {
	                            label += " *";
	                        }
	                    } else {
	                        // ensure the tab is marked dirty before removing it
	                        if ((label.length > 1) && (label.substring(label.length - 2) == " *")) {
	                            label = label.substring(0, label.length - 2);
	                        }
	                    }
	                } else {
	                    if (!skipCompactLabel && (label.length > 20)) {
	                        label = label.substring(0, 20);
	                        label += "...";
	                    }
	                    if (dirty && dirty == true) {
	                        label += " *";
	                    }
	                }
	                tab.controlButton.setLabel(label);
					
	                if(iconClass) {
	                    tab.controlButton.containerNode.className = iconClass;                            
	                }
	                // force a layout to align the tab buttons correctly.
	                setTimeout(dojo.hitch(this._tabs, 'layout'), 10);
				}

            },
			
            // Summary:
            //   Update the content of the tab corresponding to a given URI
            //			
			updatePageContent: function(uri, content) {
				var tab = this.getPageByUri(uri);
				tab = this._setTabContent(tab, content);				
				return tab;
			},
            
            // Summary:
            //   Updates the pages on artifacts changed.  Only the tabs with content that have reference of
            //   a widget that has the "onArtifactChanged" handler defined can be updated.  The changeType
            //   is either "GlobalEvents.ArtifactsUpdated", "GlobalEvents.ArtifactsAdded",
            //   or "GlobalEvents.ArtifactsDeleted".
            //
            pageContentChanged: function(changeType, artifactType, artifactArray, fromURI)
            {
                var tabs = this._tabs.getChildren();
                
                for (var i = 0; i < tabs.length; i++) {
                    if (tabs[i].uri != fromURI) {
                        // will skip the tab that posted the change event
                        if (tabs[i].contentWidget && tabs[i].contentWidget.onArtifactChanged &&
                        dojo.isFunction(tabs[i].contentWidget.onArtifactChanged)) {
                            // invoke the content widget's "onArtifactChanged" handler
                            tabs[i].contentWidget.onArtifactChanged(changeType, artifactType, artifactArray);
                        }
//                        else console.debug("============> skipped page without handler " + tabs[i].uri);
                    }
//                    else console.debug("============> skipped page posting page " + tabs[i].uri);
                }
            },

            closeTab: function(uri)
            {
                var tab = this.getPageByUri(uri);
                if (tab != null) {
                    this._tabs.removeChild(tab);
                    return;
                }
            },
			
			_onHide: function()
			{
				if( dojo.isIE && this.domNode.childNodes.length == 1 )
				{
					//For some reason hiding the non-selected tabs does not
					//work for the dashboard page, when the dashboard as been
					//added to it.  If the dashboard page is also not displayed, then
					//the tab to display can be set correctly by the widget.
					//Hiding the homePage (dashboard page)
					this.domNode.childNodes[ 0 ].style.display = "none";
					this.domNode.style.display = "none";
				}
			},

			getLastSelectedTab: function()
			{
				return this._tabs.lastSelectedPage;
			}
		}
	);
}
)();
