/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.common.Section");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");


(function(){
	var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	
    dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.common.Section", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.rqm.planning.web", "ui/internal/view/templates/common/Section.html"),
        //	templateCssPath: dojo.uri.dojoUri("../com.ibm.rqm.planning.web/ui/internal/view/templates/common/Section.css"),
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Initialization parameters
        
        label: Constants.getMessage("labelNoLabel"),
        identifier: "",
        alternativeIdentifier: "",
        controlWidget: null,
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Attach points
        
        headerDiv: null,
        labelSpan: null,
        controlDiv: null,
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Private fields
        
        
        ///////////////////////////////////////////////////////////////////////////////////
        // Called on creation
        
        postCreate: function(){
            this.labelSpan.innerHTML = this.label;
            if (this.controlWidget) 
                this.controlDiv.appendChild(this.controlWidget.domNode);
        },
        
        ///////////////////////////////////////////////////////////////////////////////////
        // API
        
        /* ExpressionDTO */
        getExpression: function(){
            var values = this.controlWidget.getSelection(); //UIItemDTO[]
            if (values.length > 0) {
                var expression = new Object();
                expression.attribute = {
                    id: this.identifier
                }; //UIItemDTO
                expression.values = values
                return expression;
            }
            return null;
        },
        
        setExpression: function(/* ExpressionDTO[] */expressions){
            this.controlWidget.clearSelection();
            this._buttonWidget.resetSelection();
            if (expressions) {
                for (var i = 0; i < expressions.length; i++) {
                    if (expressions[i].attribute.id == this.identifier) {
                        this.controlWidget.setSelection(expressions[i].values);
                        this._buttonWidget.setSelection(expressions[i].operator);
                        return true;
                    }
                }
            }
            return false;
        },
        
        setValues: function(/* UIItemSetDTO[] */values){
            for (var i = 0; i < values.length; i++) {
                if (values[i].attributeName == this.identifier || values[i].attributeName == this.alternativeIdentifier) {
                    this.controlWidget.setValues(values[i].uiItems);
                    break;
                }
            }
        },
        
        setVariables: function(/* UIItemSetDTO[] */values){
            for (var i = 0; i < values.length; i++) {
                if (values[i].attributeName == this.identifier || values[i].attributeName == this.alternativeIdentifier) {
                    this.controlWidget.setVariables(values[i].uiItems);
                    break;
                }
            }
        },
        
        setOperators: function(/* OperatorSetDTO */values){
            for (var i = 0; i < values.length; i++) {
                if (values[i].attributeId == this.identifier) {
                    this._buttonWidget.setValues(values[i].operators);
                    break;
                }
            }
        },
        
        getSummary: function(){
            var expression = this.getExpression(); // ExpressionDTO
            if (expression) {
                var values = new Array();
                for (var i = 0; i < expression.values.length; i++) {
                    values.push(expression.values[i].label);
                }
                return [this.label, expression.operator.label, values.join(" or ")].join(" ");
            }
            return null;
        }
        
    });
    
})();
