/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.extension.EWISection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");

dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");

dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");
dojo.require("com.ibm.rqm.execution.web.ui.internal.view.execution.EWIList");
dojo.require("com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

(function(){ // module container
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var GenerateEWIWizard = com.ibm.rqm.configuration.web.ui.internal.view.configuration.GenerateEWIWizard;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
	var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
    
    dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.extension.EWISection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: "<div dojoAttachPoint=\"_ewiSection\" class=\"com-ibm-rqm-execution-web-ui-internal-view-extension-EWISection\"></div>",
        
		_ewiSectionInitialized: false,
		
        constructor: function(args){
            this.dialogIndex = 0;
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();			
        },
        
		//@Overriden from AbstractArtifactEditorSection
        initialize: function(isEsitable) {
            if(!this._ewiSectionInitialized){
				var self = this;
	            dojox.data.dom.removeChildren(this._ewiSection);
	            var listArgs = {
	                isEditorSection: true,
	                showTestCase: false,
					showArchived: this.getParentEditor().isSnapshot(),
	                editable: this.isEditable() && !this.isPrintView(),             
	                testCase: self.getArtifact(),
	                tableActions: this.getTableActions(),
					parentSection: this
	            }
	            this._listWidget = new com.ibm.rqm.execution.web.ui.internal.view.execution.EWIList(listArgs);
	            this._ewiSection.appendChild(this._listWidget.domNode);
				this._ewiSectionInitialized = true;
				
			} else{
				// was new.. now its saved.. so update the listwidget with the testcase
				this._listWidget.testCase = this.getArtifact();
				this._listWidget.initialize();
			}	
			
        },
		
        getTableActions: function(){
            var actions = [];
            var ewi_type = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_EXECUTION_EWI;
            var ewi_perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(ewi_type);
            
            if (!ewi_perm || ewi_perm.create) {
                var action = {
                    actionId: "generate",
                    label: Constants.getMessage("labelGenEWIs"),
                    actionHandler: dojo.hitch(this, "_generateEWI"),
                    iconClass: "generate-ewis-icon",
                    disabledOnReadOnly: true
                }
				if(!this.canGenerateEWI()){
					action.disabled = true;
				}
				actions.push(action);
            }			
            return actions;
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.addWorkItemSupport();
			if(this._ewiSectionInitialized){
				this.initialize();
			}			
        },
        
        update: function(){
            this._listWidget.initialize();
			this.setDirty(true);
        },
        
        _generateEWI: function(parentTestPlanId, parentTestPlanUUID){
            if (dojo.isFunction(this.getArtifact) && this.getArtifact() && this.getArtifact().id) {
                var self = this;
                var args = {
                    title: Constants.getMessage("labelGenEWIs"),
                    parentController: this.parentController,
                    callback: {
                        context: self,
                        method: self.update
                    },
                    testCases: [this.getArtifact()]
                }
                this._ewiWizard = new GenerateEWIWizard(args);
                
            } else {
                var msg = Constants.getMessage("msgSaveBeforeGenTER", this.getArtifactType());
				this._displayAlertDialog(msg);
            }
        },
		
		_contains: function(item) {
            var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;         
            return this._getItemById(itemId) != null;
        },
		
		 _getItemById: function(id) {
		 	this._items = [];
			if(this._listWidget && this._listWidget.ewis)
				this._items = this._listWidget.ewis;
			for (var i = 0; i < this._items.length; i++) {
                var item = this._items[i];
                var itemId = item.versionableItemId ? item.versionableItemId : item.itemId;         
                if (itemId == id) {
                    return this._items[i];
                }
            }
            return null;
        },
		
		_displayAlertDialog: function(message, handler)
		{
			var args = {
                'message': message
            };
			if(handler){
				args.okCallback = handler;
			}
			var alertDialog = new AlertDialog(args);
			
            alertDialog.setSecondaryTitle("");
            alertDialog.open();
		},
		validate: function(){
			if(this._listWidget && this._listWidget._isBeingEdited()){
				this._displayAlertDialog(Constants.getMessage("msgSaveDiscardTERChanges"));
				this.getParentEditor().setSectionFocus("com.ibm.rqm.execution.editor.section.ewis");
				return false;
			}
			return true;
		},
		canGenerateEWI:function(){
			if (this.parentEditor && this.parentEditor.isSaveableState) {
				return this.parentEditor.isSaveableState();
			}
			return true;			
		}
        
    });
    
})();
