/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.requirement.web.ui.internal.view.extension.RequirementsSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane");
dojo.require("com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection");

dojo.require("com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementSelectionDialog");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementsSource");
dojo.require("com.ibm.rqm.requirement.web.ui.internal.view.table.RequirementTableView");
dojo.require("com.ibm.asq.common.web.ui.internal.util.CSHelp");

(function(){ // module container
    var CSHelp = com.ibm.asq.common.web.ui.internal.util.CSHelp;
    var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    var Utility = com.ibm.rqm.planning.web.ui.internal.util.Utility;
    var AbstractArtifactEditorSection = com.ibm.rqm.planning.web.ui.internal.view.extension.AbstractArtifactEditorSection;
    var AlertDialog = com.ibm.asq.common.web.ui.internal.widgets.AlertDialog;
    var Constants = com.ibm.rqm.planning.web.client.internal.Constants;
    
    var RequirementWorkItemClient = com.ibm.rqm.requirement.web.client.internal.requirement.RequirementWorkItemClient;
    var RequirementTableView = com.ibm.rqm.requirement.web.ui.internal.view.table.RequirementTableView;
    
    var REQ_WORKITEM_TYPE_ID = "requirement";
    
    dojo.declare("com.ibm.rqm.requirement.web.ui.internal.view.extension.RequirementsSection", [dijit._Widget, dijit._Templated, AbstractArtifactEditorSection], {
        templateString: '<div class="com-ibm-rqm-test-requirement-plan-section">' +
                        '   <div dojoAttachPoint="_tableContainer"></div>' +
                        '</div>',
        
        constructor: function(args){
            this.RequirementWorkItemClient = RequirementWorkItemClient;
            this._reqTableViewer = null;
        },
        
        postCreate: function(){
            this.inherited('postCreate', arguments);
            this.addWorkItemSupport();
        },
        
        //@Overriden from AbstractArtifactEditorSection
        initialize: function(isEditable){
            this._requirements = [];
            this._requirementsAdded = [];
            this._requirementsRemoved = [];
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
                this._linkType = "com.ibm.rqm.requirement.linkType.testPlan";
            }
            else {
                this._linkType = "com.ibm.rqm.requirement.linkType.testCase";
            }
            
            this.showRequirementList();
        },
        
        //@Overridden from AbstractArtifactEditorSection
        saveFinished: function(savedItem){
            this.inherited('saveFinished', arguments);
            this.initialize();
            this.addWorkItemSupport();
            if (this._reqTableViewer) {
                this._reqTableViewer.setEditable(this.isEditable() && this.getArtifact());
            }
        },
        
        //actions to be displayed in the table header
        getTableActions: function(){
            var actions = [];
            actions.push({
                label: Constants.getMessage("labelRefresh"),
                actionHandler: dojo.hitch(this, '_loadRequirements'),
                iconClass: "refresh-icon-image",
                disabledOnReadOnly: false
            });
            
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
                actions.push({
                    label: Constants.getMessage("labelImportRequirements"),
                    actionHandler: dojo.hitch(this, "_handleImportRequirementClicked"),
                    iconClass: "import-reqirement-image",
                    disabledOnReadOnly: true
                });
            }
            
            actions.push({
                label: Constants.getMessage("labelAssociateRequirements"),
                actionHandler: dojo.hitch(this, "_handleAssociateRequirementClicked"),
                iconClass: "associate-requirement-image",
                disabledOnReadOnly: true
            });
            
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
                actions.push({
                    label: Constants.getMessage("labelAddNewRequirement"),
                    actionHandler: dojo.hitch(this, "_handleNewRequirementClicked"),
                    iconClass: "add-new-requirement-image",
                    disabledOnReadOnly: true
                });
            }
            
            actions.push({
                label: Constants.getMessage("labelDisassociateRequirements"),
                actionHandler: dojo.hitch(this, "_handleRemoveRequirementClicked"),
                iconClass: "remove-reqirement-image",
                disabledOnReadOnly: true
            });
			
            if (this.getArtifactType() == Constants.TYPE_PLAN) {
                actions.push({
	                label: Constants.getMessage("labelAssociatedTestCases"),
	                actionHandler: dojo.hitch(this, "_handleFindAssociatedTestCaseClicked"),
	                iconClass: "tcases-w-requirmt-icon-image",
	                disabledOnReadOnly: true
            	});
            }
            
            return actions;
        },
        
        showRequirementList: function(){
            if (!this._reqTableViewer) {
                dojox.data.dom.removeChildren(this._tableContainer);
                var args = {
                    requirements: this._requirements,
                    editable: this.isEditable(),
                    actions: this.getTableActions(),
                    parent: this,
					printable: this.isPrintView()
                };
                this._reqTableViewer = new RequirementTableView(args);
                this._tableContainer.appendChild(this._reqTableViewer.domNode);
            }
            
            //load the table
            if (this.getArtifact()) {
                this._loadRequirements();
            }
        },
        
        _refreshRequirementList: function(requirements){
            this._requirements = requirements ? requirements : [];
            this.refresh();
        },
        
        refresh: function(){
            this._reqTableViewer.refresh(this._requirements);
			this._reqTableViewer.disaccosicateButton.setDisabled(true);
			if(this._reqTableViewer.assoicatedTestPlanNode!=null)
				this._reqTableViewer.assoicatedTestPlanNode.setDisabled(true);
			if(this._reqTableViewer.assoicatedTestCaseNode!=null)
				this._reqTableViewer.assoicatedTestCaseNode.setDisabled(true);
		},
        
        _loadRequirements: function(){
            if (this.getArtifact() && this._reqTableViewer) {
                var responseHandler = new ServiceResponseHandler(this, "_refreshRequirementList", "_error");
                switch (this.getArtifactType()) {
                    case Constants.TYPE_PLAN:
                        this.RequirementWorkItemClient.getRequirementWorkItemDTOs(responseHandler, {
                            testPlanItemId: this.getArtifact().versionableItemId
                        });
                        break;
                    case Constants.TYPE_CASE:
                        this.RequirementWorkItemClient.getRequirementWorkItemDTOs(responseHandler, {
                            testCaseItemId: this.getArtifact().versionableItemId
                        });
                        break;
                }
            }
            else {
                //this.RequirementWorkItemClient.getAllRequirements(this, callbackFunction);
            }
        },
        
        _handleAssociateRequirementClicked: function(){
            var self = this;
            var dialogArgs = {
                object: this,
                requirements: this._requirements,
                type: this.getArtifactType(),
                artifactId: this.getArtifact()? this.getArtifact().versionableItemId : null,
                onOK: function(requirements){
                    self.addRequirements(requirements);
                }
            }
            this.requirementDialog = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementSelectionDialog(dialogArgs);
			var args = {
                primaryTitle: Constants.getMessage("labelAssociateRequirement"),
                isClosable: true,
                contentNode: this.requirementDialog.domNode,
				okCallback: dojo.hitch(this.requirementDialog, "_onOK")
            }
            this.floatingPane = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
			this.floatingPane._yesButton.disabled = false;
            dojox.data.dom.removeChildren(this.floatingPane._headerText);
            var span1 = document.createElement("span");
            span1.innerHTML = Constants.getMessage("labelAssociateRequirement");
            this.floatingPane._headerText.appendChild(span1);
            var cshelp = new CSHelp({
                contextID: "rqm_assoc_req.html"
            });
            cshelp.setButtonLabel(Constants.getMessage("labelAssociateReqCSHelp"));
            var span2 = document.createElement("span");
            span2.appendChild(cshelp.domNode);
            this.floatingPane._headerText.appendChild(span2);
            
            this.floatingPane.open(false);
            this.floatingPane.domNode.style.width = "62em";
			this.requirementDialog.initialize();
        },
        
        _handleImportRequirementClicked: function(){
            var self = this;
            var setTitle = function(title){
                self.importFloatingPane.setTitle(title);
            };
            var onCancel = function(){
                self.importFloatingPane.close();
            };
            var onDone = function(requirements){
                self.importFloatingPane.close();
                self.addRequirements(requirements);
            };
            this._requirementsSource = new com.ibm.rqm.requirement.web.ui.internal.view.requirement.RequirementsSource({
                requirements: this._requirements,
                setTitle: setTitle,
                onCancel: onCancel,
                onDone: onDone,
                parent: self
            });
            
            var args = {
                primaryTitle: Constants.getMessage("labelImportRequirementStep1"),
                isClosable: true,
				width: "50%",
                contentNode: this._requirementsSource.domNode
            }
            this.importFloatingPane = new com.ibm.asq.common.web.ui.internal.widgets._CommonDialog(args);
            dojox.data.dom.removeChildren(this.importFloatingPane._headerText);
            var span1 = document.createElement("span");
            span1.innerHTML = args.primaryTitle;
            this.importFloatingPane._headerText.appendChild(span1);
            var cshelp = new CSHelp({
                contextID: "rqm_import_req.html"
            });
            cshelp.setButtonLabel(Constants.getMessage("labelImportReqCSHelp"));
            var span2 = document.createElement("span");
            span2.appendChild(cshelp.domNode);
            this.importFloatingPane._headerText.appendChild(span2);
            
            this.importFloatingPane.open(false);
            dojo.mixin(this.importFloatingPane, {
                "setTitle": function(title){
                    dojox.data.dom.removeChildren(this._headerText);
                    var span1 = document.createElement("span");
                    span1.innerHTML = title;
                    this._headerText.appendChild(span1);
                    var cshelp = new CSHelp({
                        contextID: "rqm_import_req.html"
                    });
                    cshelp.setButtonLabel(Constants.getMessage("labelImportReqCSHelp"));
                    var span2 = document.createElement("span");
                    span2.appendChild(cshelp.domNode);
                    this._headerText.appendChild(span2);
                }
            });
        },
        
        _handleNewRequirementClicked: function(){
            var self = this;
            var args = {
                callback: {
                    method: dojo.hitch(self, "addNewRequirement"),
                    returnAsDTO: true
                },
                workItem: null,
                workItemTypeId: REQ_WORKITEM_TYPE_ID,
                cshelpId: "rqm_add_req.html"
            }
            this._wiPopup = new com.ibm.rqm.planning.web.ui.internal.view.common.QMWorkItemPane(args);
            //this._workitemnumber.appendChild(this._wiPopup.domNode);
        },
        
        addNewRequirement: function(requirement){
            //create the link b/w the artifact and this new requirement
            if (requirement) {
                var newReq = {
                    id: requirement.id,
                    status: "",
                    itemId: requirement.itemId,
                    name: requirement.summary.content,
                    description: requirement.description.content,
                    owner: requirement.attributes.owner,
                    externalId: ""
                };
                this._requirements.push(newReq);
                this._requirementsAdded.push(newReq);
                this.getSaveContext().createLink(this.getParentEditor(), this._linkType, requirement.itemId);
                this.setDirty();
                this.refresh();
            }
        },
        
        _handleRemoveRequirementClicked: function(){
            var selectedRequirements = this._reqTableViewer.getSelectedRows();
            if (selectedRequirements && selectedRequirements.length > 0) {
				var self = this;
				var args = {
					title: Constants.getMessage("labelConfirmation"),
					message: Constants.getMessage("msgConfirmDeleteRequirements"),
					yesCallback: function(){   
						 console.debug("Number of to be removed - " + selectedRequirements.length);
                		self.removeRequirements(selectedRequirements);
						return true;          
					},
					noCallback: function() {
						return false;
					}
				}
				var dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
	        	dialog.setSecondaryTitle(args.title);
	        	dialog.open();
            }
            else {
                this.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, Constants.getMessage("msgNoRequirementDisassociate"));
            }
        },
        
        _handleFindAssociatedTestCaseClicked: function() {
			var selectedReqs = this._reqTableViewer.getSelectedRows();
			if(!selectedReqs || selectedReqs.length != 1) {
				return;
			}
			var selectedReq = selectedReqs[0];
			var args = {};
			args.subAction = "viewAssociatedTestCases";
			args.itemId = selectedReq.itemId;
			args.id = selectedReq.id;
			args.testPlanItemId = this.getArtifact().versionableItemId;
			var uri = "#action=com.ibm.rqm.planning.home.actionDispatcher"; //FIXME removed hard coded action dispatcher id
                
            for(var argName in args) {
                var argVal = args[argName];
                
                if(argName == "tq")
                    argVal = Date.parse(new Date().toString());
                    
                uri += ("&" + argName + "=" + argVal); 
            }
            window.location.hash = uri;
        }, 
        
        addRequirements: function(requirements){
            for (var i = 0; i < requirements.length; i++) {
                if (this._contains(this._requirementsRemoved, requirements[i])) {
                    this.getSaveContext().unremoveLink(this.getParentEditor(), this._linkType, requirements[i].itemId);
                    this._requirementsRemoved = this._removeItem(this._requirementsRemoved, requirements[i]);
                    this._requirements.push(requirements[i]);
                }
                else 
                    if (!this._contains(this._requirements, requirements[i])) {
                        this.getSaveContext().createLink(this.getParentEditor(), this._linkType, requirements[i].itemId);
                        this._requirementsAdded.push(requirements[i]);
                        this._requirements.push(requirements[i]);
                    }
                    else {
                        this.getParentEditor().setHeaderStatus(Constants.getMessage("msgExistRequirement"), Constants.HEADER_STATUS_INFO);
                        this._displayAlertDialog(Constants.getMessage("labelAlert"), Constants.getMessage("msgExistRequirement"));
                    }
                
            }
            this.setDirty();
            this.refresh();
        },
        
        removeRequirements: function(requirements){
            for (var i = 0; i < (requirements ? requirements.length : 0); i++) {
                this._removeRequirement(requirements[i]);
            }
            this.setDirty();
            this.refresh();
        },
        
        _contains: function(items, item){
            var id = item.id;
            var externalId = item.externalId;
            if (id) {
                return this._getRequirementById(items, id) != null;
            }
            else {
                return this._getRequirementByExternalId(items, externalId) != null;
            }
        },
        
        _getRequirementById: function(items, id){
            for (var i = 0; i < items.length; i++) {
                var requirement = items[i];
                var itemId = requirement.id;
                if (itemId == id) {
                    return items[i];
                }
            }
            return null;
        },
        
        _getRequirementByExternalId: function(items, id){
            for (var i = 0; i < items.length; i++) {
                var requirement = items[i];
                var itemId = requirement.externalId;
                if (itemId == id) {
                    return items[i];
                }
            }
            return null;
        },
        
        _removeRequirement: function(item){
            var updatedItems = [];
            var items = this._requirements;
            for (var i = 0; i < items.length; i++) {
                if (items[i].itemId && items[i].itemId != item.itemId) {
                    updatedItems.push(items[i]);
                }
                else {
                    //handle item removal from save context   
                    var saveContext = this.getSaveContext();
                    if (this._contains(this._requirementsAdded, items[i])) {
                        //handle existing items ADDED during this saveContext session
                        this._requirementsAdded = this._removeItem(this._requirementsAdded, items[i]);
                        saveContext.uncreateLink(this.getParentEditor(), this._linkType, items[i].itemId);
                    }
                    else {
                        //handle existing references, add them 'removed'
                        this._requirementsRemoved.push(items[i]);
                        saveContext.removeLink(this.getParentEditor(), this._linkType, items[i].itemId);
                    }
                }
            }
            this._requirements = updatedItems;
        },
        
        _removeItem: function(items, item){
            var updatedArray = [];
            for (var i = 0; i < items.length; i++) {
                if (items[i].itemId != item.itemId) {
                    updatedArray.push(items[i]);
                }
            }
            return updatedArray;
        },
        
        _error: function(e){
            this._displayAlertDialog(Constants.getMessage("labelError"), e.message);
        },
        
        _displayAlertDialog: function(title, message){
            var alertDialog = new AlertDialog({
                'message': message
            });
            alertDialog.setSecondaryTitle(title);
            alertDialog.open();
        }
        
    });
    
})();
