/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView");
 

dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");

dojo.require("com.ibm.rqm.planning.web.client.internal.UserClient");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var ServiceResponseHandler      = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var UserClient                  = com.ibm.rqm.planning.web.client.internal.UserClient;
var Constants                   = com.ibm.rqm.planning.web.client.internal.Constants;


dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.AbstractFilteredTableView", null,
{
	constructor: function() {
		this._typeLabel = "";
		this._searchFunction = null;
		this._searchCallback = null;
		this._searchFilterCallback = null; //allow caller to provide additional filter
		this._searchResponseHandlerType = ServiceResponseHandler;
	},
	
    setTypeLabel: function(/* String */ label) {
        this._typeLabel = label;
    },  
		
	setSearchFunction: function(/* Function */ func) {
		this._searchFunction = func;
	},	
	
    setSearchCallback: function(/* Function */ func) {
        this._searchCallback = func;
    }, 

    setSearchFilterCallback: function(/* Function */ func) {
        this._searchFilterCallback = func;
    },
	
    setResponseHandlerType: function(/* TransportServiceResponseWrapper */ handlerType) {
        this._searchResponseHandlerType = handlerType || ServiceResponseHandler;
    },  
		
    setInProgress: function(msg) {
        if (this.viewHeaderNode) {
			this.viewHeaderNode.setProgressStart(msg);
		}
    },
    
    stopInProgress: function() {
        this.showInitLoadProgress = false;
		if (this.viewHeaderNode) {
			this.viewHeaderNode.setProgressEnd();
		}
		if (this.tableViewer) {
			this.tableViewer.endLoadingProgress();
		}
    },	
    
    // method invoked by the filter "Run" action to query with server side filtering. 
    runQuery: function(filterArgs) {
        this.filterArgs = filterArgs;

        // add the current project area to the args
        filterArgs.processArea = com.ibm.team.process.web.projectArea.itemId;

        var responseHandler = new this._searchResponseHandlerType(this, "showQueryResults", "showQueryError");          
        
        this.showInitLoadProgress = true;
        this.tableViewer.setContent([]);
        this.tableViewer.refresh();
        this.tableViewer.showLoadingProgress();
        
		this.setInProgress(Constants.getMessage("msgQueryTableList", this._typeLabel));
        this._searchFunction(responseHandler, this.filterArgs);   
    },
	
	// run the query automatically if the preference dictates
    initRunQuery: function(userPref) {
        if (userPref && userPref.value) {
            var args = {
                myItemsOnly: true,
                includeArchived: false
            }; 
            this.runQuery(args);
        }       
    },	
	
	showQueryResults: function(resultsDTO) {
        this.stopInProgress();
		if (resultsDTO != null && resultsDTO.status == Constants.ARTIFACT_SEARCH_STATUS_OK) {
            //search ok. show results
			if (this._searchFilterCallback && dojo.isFunction(this._searchFilterCallback)) {
				resultsDTO.results = this._searchFilterCallback(resultsDTO.results);
			}
			this._doSearchCallback(resultsDTO.results);		  	
		} else if (resultsDTO != null && resultsDTO.status == Constants.ARTIFACT_SEARCH_STATUS_MAX_EXCEEDED) {
            //too many results, show warning dialog
            this.showQueryThresholdWarning(resultsDTO);			
		} else {
            //some error occurred
			this.showQueryError({
                message: resultsDTO ? resultsDTO.message : "Unknown Error"  
            });			
		}
	},
	
    _doSearchCallback: function(searchResults) {
        if (dojo.isFunction(this._searchCallback)) {
            this._searchCallback(searchResults);
        } else {
            console.error(this.declaredClass + ": no search callback secified!");
        }
    },

    showQueryError: function(error) {
        this._error(error);
        this.stopInProgress();		
	},
	
	showQueryThresholdWarning: function(resultsDTO) {
        var dialogArgs = {
            primaryTitle: Constants.getMessage("labelSearchLimitWarning", resultsDTO.totalSize),
            yesLabel: Constants.getMessage("labelRefine"),
			yesCallback: dojo.hitch(this, function() {
                //set proper policy; user can refine search
				this.filterArgs.policy = Constants.ARTIFACT_SEARCH_POLICY_ENFORCE;
            }),
			noLabel: Constants.getMessage("labelLoadFirst", resultsDTO.size),
            noCallback: dojo.hitch(this, function() {
				//truncate the search to the threshold
                this.filterArgs.policy = Constants.ARTIFACT_SEARCH_POLICY_TRUNCATE;
				this.runQuery(this.filterArgs);
            }),
			cancelLabel: Constants.getMessage("labelLoadAll"),
            cancelCallback: dojo.hitch(this, function() {
                //run the query anyway, getting all results
                this._dialog.close();
				this.filterArgs.policy = Constants.ARTIFACT_SEARCH_POLICY_IGNORE;
                this.runQuery(this.filterArgs);
            }),		
			showCancel: true,	
            isClosable: false
        };                
        this._dialog = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(dialogArgs);
        this._dialog.setWidth("600px");
		this._dialog._cancelButton.onclick = dialogArgs.cancelCallback;
		
        var messageNode = document.createElement("span");
        messageNode.innerHTML = Constants.getMessage("msgSearchLimitWarning", resultsDTO.totalSize);
		  	
		this._dialog.contentContainer.appendChild(messageNode);
        this._dialog.open();		
	},
	
	addContributorFilter: function(filterObj, filtersCollection, callback) {
        if (dojo.isArray(this.contributors) && this.contributors.length > 0) {
			this._doAddContributorFilter(this.contributors, filterObj, callback);
		}
		//creates a filter with the list of contributors
        var self = this;
        var serviceObj = {
            success: function(contributors) {
				self._doAddContributorFilter(contributors, filterObj, filtersCollection, callback);
            },
            failure: function(e) {
                self._error(e);
				callback();
            }
        };
        var responseHandler = new ServiceResponseHandler(serviceObj, "success", "failure");
        UserClient.getContributors(responseHandler);
    },
	
    _doAddContributorFilter: function(contributors, filterObj, filtersCollection, callback) {
        contributors = contributors || [];
		this.contributors = contributors;
		for (var i = 0; i < this.contributors.length; i++) {
            if (contributors[i].name == "Unassigned") {
                contributors[i].name = Constants.getMessage("labelUnassigned");
            }
            contributors[i].value = contributors[i].itemId;
            contributors[i].checked = false;
        }
        
        var contributorFilter = {
            name: filterObj.name,  // must match key in the arg list of the API
            title: filterObj.label,
            promptTitle: filterObj.prompt,
            type: filterObj.type,
            defaultValue: [],
            params: {options: contributors}
        };
        filtersCollection.push(contributorFilter);
        callback();		
	} 	
    
});

})();
