/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.ResourceTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

//xojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");

dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;
var GlobalEvents   = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;

dojo.declare("ResourceTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    constructor: function(parentView) {
        this.parentView = parentView;
    },
    
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        var selectionCount = tableViewer.getCheckedRows().length;
        if (this.parentView.deleteActionNode)
            this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
    },
    
    // Summary:
    //    A group check box is shown when the tables rows are checkable and the in multi-select mode.
    //    This method is called when the group's check box is checked/unchecked. The affected rows and 
    //    their states are passed.
    //
    groupRowsCheckStateChanged: function(tableViewer, rowObjects, state) 
    {
        this.rowCheckStateChanged(tableViewer, rowObjects, state);
    },
    
    // Summary:
    //    If the client have registered table actions, this message will be fired when an action (button)
    //    is invoked by the user.
    //
    tableContentActionInvoked: function(tableViewer, actionId, callbackData, callback)
    {                
        if (callback != null && dojo.isFunction(callback)) {
            if (callbackData != null) {
                callback(callbackData);
            } else {
                callback();
            }
        } 
    }    
});

dojo.declare("ResourceTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		
    }
});

dojo.declare("ResourceTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = Constants.getMessage("labelAddCustCrtl");
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("ResourceTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
	
     getNumColumns: function(content){
            return 2;
        },
        
        // Summary:
        //
        getColumnHeading: function(content, index){
            var headings = [Constants.getMessage("labelScriptType"), Constants.getMessage("labelSharedLocation")];
            return headings[index];
        },
        
        getIsColumnSortable: function(content, atColumnIndex){
            return true;
        },
        
        getColumnWidth: function(content, atColumnIndex){
            var width = 2;
            switch (atColumnIndex) {
                case 0:
                    width = 30;
                    break;
                case 1:
                    width = 70;
                    break;
            }
            return width;
        },
        getIsCellEditable: function(rowData, atColumnIndex){
            return true;
        },
        
        // Summary:
        //    The only assumptions the table have on the data that it displays
        //    is that it's an array contents. These are the data content/object that
        //    will be displayed in the table rows.
        //
        getContentObjects: function(content){
            // our content is already an array of objects
            return content;
        },
        
        getRowDataId: function(rowData){
            return "table_for_resources_"+rowData.scriptType;
        },
        
        // Summary:
        //    Returns the type of the cell to be displayed. By default, the TableViewer
        //    will support a set of cell types. The type of the cell will map to a specific
        //    cell renderer (to be discussed).
        getCellDisplayType: function(rowData, atColumnIndex){
            var type = TableConstants.RENDER_TYPE_TEXT;
            return type;
        },
        
        // Summary:
        //    Returns the display data of the cell. For cell renderer type of TEXT, this will
        //    be the string. Others cell render type would be Date, or integer value, etc. If the 
        //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
        //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
        //    cell types such as image where it could have multiple values for different states, the 
        //    client also needs to implement the next method.
        //    
        //    Client must implement this method to provide the cell data.
        //    
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "data";
            switch (atColumnIndex) {
                case 0:
                    val = Constants.getExtensionDisplayMessage(rowData.type);
                    break;
                case 1:
                    val = rowData.resolvedPath;
                    break;
            }
            return val;
        },
        
        
        // Summary:
        //    If applicable, returns the extended display information of the cell. The return extended
        //    info must be an object with the following attributes:
        //    
        //    {
        //      /* attribute common for all cell renderer types */
        //      "tooltip": <string>,
        //
        //      /* attributes for IMAGE and ACTION cell renderer types */
        //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
        //      "image": base image URL,  
        //      "mouseOverImage": image URL, (optional)
        //      "mouseOutImage": image URL,  (optional) (overrides "image")
        //
        //      /* attributes for SELECT cell renderer type */
        //      "select" : 'value', the selection in the combo
        //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
        //
        //      /* attributes for CHECKBOX cell renderer type */
        //      "state" : <true, false> initial state of the check box
        //
        //      /* attributes for LINK cell renderer type */
        //      "href" : <string> the href string for the link
        //    }
        //
        //    Client can implement this method to provide extended display infor for the cell.
        //    Certain cell renderer type will require the client to implement this method to
        //    provide needed display information, such as an image ACTION.
        //
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            var val = {};
            return val;
        }
    
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.ResourceTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div class="com-ibm-rqm-test-artifact-table-view" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" "noline"=true dojoAttachPoint="viewHeaderNode">' +
                    '    <div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>' +
                    '  </div>' +
				    '</div>',

    editable: true,
	printable: false,
    widgetsInTemplate: true,

    constructor: function() {
		this.categoryTypes = null;
	 	this.resources = null;
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);

        //create the table viewer
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            //turn off all controls if in print mode 
            showPagingControl: !this.printable,
			showPagingRangeControl: !this.printable,
            showGroupFilterControls: !this.printable,
            showPagingActionsControls: !this.printable,
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: "RQM-RESOURCE-TABLEVIEWER"                                
                      
        });

        // the view is in a query page if there is a query name otherwise it is within
        // some container that don't need the header
        this.isQueryPage = this.queryName;
        
        if (this.isQueryPage) {
            // it is a query page so set header title and add table viewer as the header content
            this.viewHeaderNode.setTitle(this.queryName);
            this._tableViewerNode.appendChild(tableViewer.domNode);
        } else {
            // it is not a query page i.e. within a section without header
            this.viewHeaderNode.domNode.style.display = "none";
            this.domNode.appendChild(tableViewer.domNode);
        }
        
        var content = this.resources ? this.resources : [];
        tableViewer.setContent(content);
        tableViewer.setTableRowsCheckable(this.editable);
        
        this.provider = new ResourceTableView.TableProvider();     
        tableViewer.setContentProvider(this.provider);
		tableViewer.setTableListener(new ResourceTableView.TableListener(this));
        tableViewer.setCellModifier(new ResourceTableView.CellModifier());
        tableViewer.setUnspecifiedGroupTitle(Constants.getMessage("labelUnassigned"));
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelResources")));
        if (this.enableRowSelection) {
            tableViewer.setTableRowsCheckable(this.editable && !this.printable, /* isSingleSelect */ false);
        }
        if (this.refreshCallback && dojo.isFunction(this.refreshCallback)) {
            tableViewer.addAction("Refresh", Constants.getMessage("labelRefresh"), this, "refresh-icon-image", this.refreshCallback);
        }
        
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            var action = this.actions[i];
            if(action.label == Constants.getMessage("labelRemoveResourceLocation")){
				this.deleteActionNode = tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);
				this.deleteActionNode.setDisabled(true);
			}	
			else
				tableViewer.addAction(action.label, action.label, null, action.iconClass, action.actionHandler);	
        }
        
        tableViewer.show();
        tableViewer.setEditable(this.editable);
        this.tableViewer = tableViewer;	
    },
 
    getAction: function(id) {
        for (var i = 0; i < (this.actions ? this.actions.length : 0); i++) {
            if (this.actions[i].label == id) {
                return this.actions[i];
            }
        }
    },
    
     // API
    getSelectedItems: function() {
        return this.tableViewer.getCheckedRows();
    },
    
    refresh: function(content) {
        this.resources = content;
        this.tableViewer.setContent(content);
        this.tableViewer.refresh();
    },

    _error: function(e) {
        console.error(this.declaredClass + ": Error: " + e.message);
    }
    
});

})();
