/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.planning.web.ui.internal.view.table.SnapshotTableView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");

dojo.require("com.ibm.rqm.planning.web.ui.internal.util.Utility");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");

(function() {

var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var Utility        = com.ibm.rqm.planning.web.ui.internal.util.Utility;
var Constants      = com.ibm.rqm.planning.web.client.internal.Constants;

dojo.declare("SnapshotTableView.TableListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
    // Summary:
    //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
    //   called.
    //
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
        //alert("Row check state changed: " + rowObject.col1);        
    }
});

dojo.declare("SnapshotTableView.CellModifier", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier,
{
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    cellValueChanged: function(oldValue, newValue, rowObject, columnIndex)
    {
        alert(Constants.getMessage("labelCellValueChanged",rowObject.col1,newValue));
        
        // store the new values in our internal data model
        if(columnIndex == 6) {
            if(dojo.isString(newValue)) {
                rowObject.selection = newValue;
            }
        }
    },
    
    cellActionInvoked: function(rowObject, columnIndex)
    {
		//handle copy action
		if (columnIndex == 6) {
		   parent.location = rowObject.href;
		}
    }
});

dojo.declare("SnapshotTableView.ListRowRenderer",
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer,
{    
    // Summary:
    //   Override the base list row renderer's create list content method
    //   to supply more custom contents.
    //
    _createListContents: function(viewer, rowObject, listWidget)
    {
        this.inherited("_createListContents", arguments);   
        
        var moreContents = document.createElement("div");
        var toggle = document.createElement("button");
        toggle.innerHTML = Constants.getMessage("labelAddedCustomControl");
        moreContents.appendChild(toggle);
        listWidget.contentNode.appendChild(moreContents);
    }
});

dojo.declare("SnapshotTableView.TableProvider", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider,
{
    // Summary:
    //
    getNumColumns: function(content)
    {
		if (this.itemType==Constants.MODEL_TYPE_PLAN || this.itemType==Constants.MODEL_TYPE_CASE) {
	        return 7; // only testplan and testcase support copy operation
		} else {
	        return 6;
		}
    },
    
    // Summary:
    //
    getColumnHeading: function(content, index)
    {
        var headings = [
		      Constants.getMessage("labelType"),
            Constants.getMessage("labelRevisionTableView"),
				Constants.getMessage("labelName"),
            Constants.getMessage("labelDate"),
				Constants.getMessage("labelOriginator"),
            Constants.getMessage("labelDetailsPlanningBody"),				
            Constants.getMessage("labelCopy")
        ];
        return headings[index];
    },

    // Summary:
    //    Optionally, returns an image src URL to be placed in the column header. If both heading
    //    and image are specified, both are shown.
    //    
    getColumnHeadingImage: function(content, atColumnIndex)
    {
		  /*
        if(atColumnIndex == 4) {
            return ImageCache.REMOVE_ENABLED;
        }
        */		  
        return null;
    },
	 
    // Summary:
    //    Returns the column header horizontal alignment. By default, the columns are
    //    aligned to the left.
    //
    //    Client optionally can indicate the alignment for the column header.
    //
    getColumnHeadingAlignment: function(atColumnIndex)
    {
        return this.getCellAlignment(null, atColumnIndex);
    },	 
    
    getIsColumnGroupable: function(content, atColumnIndex)
    {
        return false;     
    },

    getIsColumnSortable: function(content, atColumnIndex)
    {
        return true;     
    },
    
    getColumnWidth: function(content, atColumnIndex)
    {
        switch(atColumnIndex) {
            case 0: 
                return 2;
            case 1: 
                return 5;
            case 2: 
                return 15;
            case 3: 
                return 15;
            case 4: 
                return 15;	
            case 5: 
                return 50;
            case 6: 
                return 3;
					 
        }
        return 0;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        // our content is already an array of objects
        return content;
    },
    
    getRowDataId: function(rowData)
    {
        return "_table_row_for_item_snapshot" + rowData.id;
    },
    
    // Summary:
    //    Returns the type of the cell to be displayed. By default, the TableViewer
    //    will support a set of cell types. The type of the cell will map to a specific
    //    cell renderer (to be discussed).
    getCellDisplayType: function(rowData, atColumnIndex)
    {
        var type = TableConstants.RENDER_TYPE_TEXT;

        switch(atColumnIndex) {
            case 0: 
                type = TableConstants.RENDER_TYPE_IMAGE;
                break;
            case 1: 
                type = TableConstants.RENDER_TYPE_LINK;
                break;
            case 2: 
                type = TableConstants.RENDER_TYPE_LINK;
                break;
				case 3: 
                type = TableConstants.RENDER_TYPE_DATE;
                break;		
				case 6: 
                type = TableConstants.RENDER_TYPE_IMAGE;
                break;
        }
        return type;
    },

    getCellAlignment: function(rowData, atColumnIndex)
    {
        if(atColumnIndex == 0 || atColumnIndex == 6)
            return TableConstants.ALIGN_CENTER;
        return TableConstants.ALIGN_LEFT;
    },
    
    // Summary:
    //    Returns the display data of the cell. For cell renderer type of TEXT, this will
    //    be the string. Others cell render type would be Date, or integer value, etc. If the 
    //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
    //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
    //    cell types such as image where it could have multiple values for different states, the 
    //    client also needs to implement the next method.
    //    
    //    Client must implement this method to provide the cell data.
    //    
    getCellDisplayData: function(rowData, atColumnIndex)
    {
        var val = "";
        switch(atColumnIndex) {
            case 0: 
                val = ""; //ImageCache.TEST_CASE;
                break;
            case 1: 
                val = rowData.revision; //rowData.col1;
                break;
            case 2: 
                val = rowData.name; //rowData.col2;
                break;
            case 3: 
                val = Utility.dateFromISOString(rowData.created);
                break;
            case 4: 
                val = rowData.originator.name; //rowData.col4;
                break;            
				case 5: 
                val = rowData.description;
                break;
            case 6: 
                val = ""; //ImageCache.COPY;
                break;  					 				 
        }
        return val;
    },

    
    // Summary:
    //    If applicable, returns the extended display information of the cell. The return extended
    //    info must be an object with the following attributes:
    //    
    //    {
    //      /* attribute common for all cell renderer types */
    //      "tooltip": <string>,
    //
    //      /* attributes for IMAGE and ACTION cell renderer types */
    //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
    //      "image": base image URL,  
    //      "mouseOverImage": image URL, (optional)
    //      "mouseOutImage": image URL,  (optional) (overrides "image")
    //
    //      /* attributes for SELECT cell renderer type */
    //      "select" : 'value', the selection in the combo
    //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
    //
    //      /* attributes for CHECKBOX cell renderer type */
    //      "state" : <true, false> initial state of the check box
    //
    //      /* attributes for LINK cell renderer type */
    //      "href" : <string> the href string for the link
    //    }
    //
    //    Client can implement this method to provide extended display infor for the cell.
    //    Certain cell renderer type will require the client to implement this method to
    //    provide needed display information, such as an image ACTION.
    //
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        
        var val = null;
		if (atColumnIndex == 0) {
            val = {};
			val.image = Utility.getArtifactIconURL(this.itemType);
		}
		if(atColumnIndex == 1 || atColumnIndex == 2) {
            val = {};
            val.tooltip = rowData.tooltipHTML;
			val.href = this.getRevisionURI(rowData);	
			if(atColumnIndex == 1)	
				val.linkType = Constants.getMessage("RevisionID");		
        }
        if(atColumnIndex == 6) {
            val = {};
			val.image = ImageCache.COPY_ICON;
            val.href = this.getCopyURI(rowData);          
        }		  
		  /*
        if (atColumnIndex == 6) {
           val = {};
			  val.image = ImageCache.REMOVE_DISABLED;
           val.mouseOutImage = ImageCache.REMOVE_DISABLED;
			  val.mouseOverImage = ImageCache.REMOVE_ENABLED;
        }
        if(atColumnIndex == 6 &&(rowData.col5 == "Jane" || rowData.col5 == "Luke")) {
            val = {};
            val['mouseOverImage'] = ImageCache.TEST_PLAN;
            val['mouseOutImage'] = ImageCache.TEST_SCRIPT;
        }
        if(atColumnIndex == 6 && rowData.col5 == "Bill") {
            val = {};
            val['select'] = (typeof rowData.selection != "undefined") ? rowData.selection : null;
            val['choices'] = [
                {'text':'room 123', 'value':'123'}, 
                {'text':'room 223', 'value':'223'}, 
                {'text':'room 323', 'value':'323'}, 
                {'text':'room 423', 'value':'423'}];
        }    
        */    
        return val;
    },

    setItemId: function(itemId) {     
       this.itemId = itemId;
    },
	 	 
	 setItemType: function(itemType) {	 	
	    this.itemType = itemType;
	 },
	  
    getRevisionURI: function(artifact) {
      var perspective = Utility.getCurrentPerspectiveByURI();
      switch(this.itemType) {
         case Constants.MODEL_TYPE_PLAN:
            return Utility.getTestPlanURI({uid: this.itemId, revision: artifact.revision}, perspective);

         case Constants.MODEL_TYPE_CASE:
            return Utility.getTestCaseURI(this.itemId, artifact.revision, perspective);
            
         case Constants.MODEL_TYPE_SCRIPT:
            return Utility.getTestScriptURI(this.itemId, perspective, artifact.revision, artifact.name);
			
         default:
            return null;
      }     
    },
    
    getCopyURI: function(artifact) {
        var perspective = Utility.getCurrentPerspectiveByURI();
        switch(this.itemType) {
            case Constants.MODEL_TYPE_PLAN:
                return Utility.getCopyTestPlanURI(this.itemId, artifact.revision, perspective);
            case Constants.MODEL_TYPE_CASE:
                return Utility.getCopyTestCaseURI(this.itemId, artifact.revision, perspective);
            default:
                return null;                                     
        }   
    }	 
    
    
});

dojo.declare("com.ibm.rqm.planning.web.ui.internal.view.table.SnapshotTableView", 
    [dijit._Widget, dijit._Templated],
{    
    templateString: '<div dojoAttachPoint="_tableViewerNode" class="com-ibm-rqm-test-artifact-table-view" tabindex="-1"></div>',

    widgetsInTemplate: true,

    constructor: function() {
      //expected params
		this.itemType = null;
		this.itemId = null;
	 	this.snapshots = null;
	 },

    postCreate: function() 
    {
        this.inherited("postCreate", arguments);
        
        var tableIdStr = "RQM-SNAPSHOT" + this.itemType + "-TABLEVIEWER"
        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: tableIdStr                                
        });        
        this._tableViewerNode.appendChild(tableViewer.domNode);
		          
        //validate arguments
        if (!this.itemType) {
		  	  throw new Error("Argument 'itemType' is REQUIRED!");
		  }
		  
		  var content = this.snapshots && dojo.isArray(this.snapshots) ? this.snapshots : [];		  
        tableViewer.setContentPageSize(20);
        tableViewer.setContent(content);
		  
		  var provider = new SnapshotTableView.TableProvider();
		  provider.setItemId(this.itemId ? this.itemId : null);
		  provider.setItemType(this.itemType);
        tableViewer.setContentProvider(provider);
		  
        tableViewer.setTableListener(new SnapshotTableView.TableListener());
        tableViewer.setCellModifier(new SnapshotTableView.CellModifier());
		tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelSnapShot")));
        tableViewer.show();
		  
		  this.tableViewer = tableViewer;		  
    },
	
	setItemId: function(itemId) {
		this.itemId = itemId;
		if (this.tableViewer && this.tableViewer.getContentProvider()) {
			this.tableViewer.getContentProvider().setItemId(itemId);
		}
	},
	 
	refresh: function(refreshedSnapshots) {
        var content = refreshedSnapshots && dojo.isArray(refreshedSnapshots) ? refreshedSnapshots : [];
        this.tableViewer.setContent(content);
        this.tableViewer.show();		  
	}
});

})();
