/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rqm.execution.web.ui.internal.view.table.cellrenderers.TextComboCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");

(function() {
var DELIMITER = ", ";

dojo.declare("com.ibm.rqm.execution.web.ui.internal.view.table.cellrenderers.TextComboCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the cell content. For renderer that can modify the cell value,
    //  
	  
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node;
		var selectLabel = document.createElement("label");
		
        var contentProvider = viewer.getContentProvider();
        var editable = (contentProvider ? contentProvider.isCustomEditable() : false);
        var editableRows = contentProvider.getEditableRows();
		var found = false;
		if (editableRows) {
			for (var j = 0; j < editableRows.length; j++) {
				if (editableRows[j].id == rowObject.id) {
					found = true;
					break;
				}
			}
		}
        if (editable && found) {
            node = document.createElement("select");
            node.className = "combo-cell";
            node.cellModifier = cellModifier;
            node.rowObject = rowObject;
            node.columnIndex = columnIndex;
            node.lastSelected = null;

            if(typeof cellWidthHint != "undefined") {
                node.style.width = cellWidthHint + "px";
            }
			
            node.onchange = function() {
                var val = null;
				var oldValues = this.lastSelected
            
                for(var i = 0; i < this.options.length; i++) {
                    if(this.options[i].selected) {
                        var newValues = this.options[i].value;
						this.lastSelected = newValues;
                    }
                }
                if(this.cellModifier)
                    this.cellModifier.cellValueChanged(oldValues, newValues, this.rowObject, this.columnIndex);
            }
        } else {
            var node = document.createElement("span");
        }
        
        var tooltip = null;
        if (extendedDisplayInfo && extendedDisplayInfo['choices']) {
            var selected = (typeof extendedDisplayInfo['select'] != 'undefined') ? extendedDisplayInfo['select'] : cellData;
            tooltip = extendedDisplayInfo['tooltip'];
            
            if (editable && found) {
                // select combo box node when editable
                this.setEnumChoices(node, extendedDisplayInfo['choices'], selected);
            } else {
                var text = "";
                
                // span text node when not editable
                if (selected) {
                    var choices = extendedDisplayInfo['choices'];
                    // something is selected so get the corresponding text in the choices
                    for (var i = 0; i < choices.length; i++) {
                        if (choices[i].value == selected) {
                            text = choices[i].text;
                            break;
                        }
                    }
					//Possible choice information is not received so check again and put data
					if(text == "")
						text = (typeof extendedDisplayInfo['defaultText'] != 'undefined') ? extendedDisplayInfo['defaultText'] : "";
                }
                if(viewer.isCompactMode && typeof cellWidthHint != "undefined") {
                    text = this._handleCompactCellText(viewer, text, extendedDisplayInfo, node, cellWidthHint);
                }
                node.appendChild(document.createTextNode(text));
            }
        }
        
        // the parent class's attach tooltip will not create a tooltip
        // if the string is null, or undefined. Don't have to check it here.
        if(!viewer.isCompactMode || editable)
            this._attachTooltipToCell(node, tooltip);    
        
		selectLabel.appendChild(node);
        return selectLabel;
    },
    
    setEnumChoices: function(select, choices, value)
    {
        var hasSelect = false;
        
        if(choices && choices.length > 0) {
            for (var i = 0; i < choices.length; i++) {
                select.options[i] = new Option(choices[i].text, choices[i].value);
                
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(value && choices[i].value == value) {
                    select.options[i].selected = true;
                    hasSelect = true;
                }
				if (choices[i].disabled)
					select.options[i].disabled = true;				    
            }
            // select the first item by default.
            if(!hasSelect)
                select.options[0].selected = true;
        }
    }    
});

})();
