/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
console.debug("Load: com.ibm.rqm.process.web.ui.internal.view.timebox.KeyDateTable");
dojo.provide("com.ibm.rqm.process.web.ui.internal.view.timebox.KeyDateTable");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DateTextBox");

dojo.require("dojox.data.dom");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.asq.common.web.client.internal.CommonPreferenceClient");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ListRowRenderer");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.rqm.planning.web.ui.internal.util.EditorUtil");
dojo.require("com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant");
dojo.require("com.ibm.rqm.planning.web.client.internal.Constants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.DatePicker");

(function(){
    var TableConstants 			= com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
    var EditorUtil 				= com.ibm.rqm.planning.web.ui.internal.util.EditorUtil;
    var util 					= com.ibm.team.workitem.web.client.util;
    var RENDER_TYPE_DATETEXT 	= "RenderType.DateText";
	var Constants 				= com.ibm.rqm.planning.web.client.internal.Constants;
    var SaveParticipant 		= com.ibm.rqm.planning.web.ui.internal.model.SaveParticipant;
	
    dojo.declare("KeyDateTable.DateTextBox", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer, {
        // Summary:
        //    Renders the cell content. For renderer that can modify the cell value,
        //    
        renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/ cellModifier, rowObject, columnIndex, cellWidthHint){
            var node;
            var contentProvider = viewer.getContentProvider();
            var editable = viewer.isEditable() && (contentProvider ? contentProvider.getIsCellEditable(rowObject, columnIndex) : false);
            
            if (editable) {
                // display the date in the preferred format.
                var dateFormatLength = com.ibm.asq.common.web.client.internal.CommonPreferenceClient.getDateFormatLength();            
                var args = {
                    isDate: true,
                    containerToggle: "fade",
                    constraints: {
                        formatLength: dateFormatLength
                    },
                    "class": "dateField", // privide a fixed width date box field
					popupCalendarOnFocus: true
                };
                node = new com.ibm.asq.common.web.ui.internal.widgets.DatePicker(args);
                if (cellData) {
                    var date = util.fromISOString(cellData);
                    node.setValue(date);
                }
                node.cellModifier = cellModifier;
                node.rowObject = rowObject;
                node.columnIndex = columnIndex;
				node.domNode.firstChild.firstChild.lastChild.style.display = "none";
				//don't inherit TableViewer's td css
				node.domNode.firstChild.firstChild.firstChild.style.borderBottom = "0px";
				node.domNode.firstChild.firstChild.firstChild.style.padding = "0px";
				if(dojo.isIE)
					dojo.removeClass(node.domNode.firstChild.firstChild.firstChild, "datePickerInputField");
                node.onChange = function(){
                    // convert the date back to the persistent timestamp string value
                    var dateVal = node.getValue();
                    var val = dojo.date.locale.format(dateVal,{datePattern:"yyyy-MM-dd", selector:"date"});
                    if (this.cellModifier) 
                        this.cellModifier.cellValueChanged(null, val, this.rowObject, this.columnIndex);
                }
                return node.domNode;
            }
            else {
                node = document.createElement("span");
                this._renderCellText(viewer, cellData, extendedDisplayInfo, node, cellWidthHint);
                return node;
            }
        }
    });
    
    dojo.declare("KeyDateTable.TableListener", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener, {
         parentView: null,
    
    	constructor: function(testView)
    	{
        	this.parentView = testView;
    	},
	
		// Summary:
        //   If the table's rows are checkable, when a row is checked, or unchecked, this method is
        //   called.
        //
        rowCheckStateChanged: function(tableViewer, rowObject, state){
            rowObject.selected = state;
			var selectionCount = tableViewer.getCheckedRows().length;
        	if (this.parentView.deleteActionNode)
            	this.parentView.deleteActionNode.setDisabled(selectionCount == 0 || !tableViewer.isEditable());
        },
		
		// Summary:
    	//    If the client have registered table actions, this message will be fired when an action (button)
    	//    is invoked by the user.
    	//
    	tableContentActionInvoked: function(tableViewer, actionId, callbackData)
    	{
        	switch(actionId){
				case "add":
					this.parentView._handleAddNewClick();
					break;
				case "remove":
					this.parentView._handleDeleteClick();
					break;
				default:
					break;
			}      
    	}
    });
    
    dojo.declare("KeyDateTable.CellModifier", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellModifier, {
        tableViewer: null,
        
        constructor: function(viewer, parent){
            this.tableViewer = viewer;
			this.parent = parent;
        },
        
        cellValueChanged: function(oldValue, newValue, rowObject, columnIndex){
			if(this.parent.parentSection){
				this.parent.parentSection.setDirty();
			}
			if(rowObject.type == 0) {
				rowObject.isChanged = true;
			}
            switch (columnIndex) {
                case 0:
					rowObject.col1 = newValue;
                    break;
                case 1:
					rowObject.col2 = newValue;
                    break;
                case 2:
					rowObject.col3 = newValue;
                    break;
                default:
                    break;
            }
        },
        
        cellActionInvoked: function(rowObject, columnIndex){
        }
    });
    
    dojo.declare("KeyDateTable.TableProvider", com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider, {
        // Summary:
        //
        getNumColumns: function(content){
            return 3;
        },

		//If data in any of the columns is required indicate it here.
		//cols is an array of column indices.
		//The required column headings will have an asterisk appended to them ie "Name" becomes "Name *"
		//The asterisk will ONLY be applied if there are rows in the table.
		setRequiredColumns:function(cols)
		{
			this.requiredCols = cols;
		},
        
        // Summary:
        //
        getColumnHeading: function(content, index){
            var headings = [Constants.PROCESS_NAME_KEY_DATE, Constants.PROCESS_DESCRIPTION_KEY_DATE, Constants.PROCESS_DATE_KEY_DATE];
			//If any of the columns are required then append an * to the column header to indicate the data is required
			if (this.requiredCols)
			{
				for (var i = 0; i < this.requiredCols.length; i++)
					headings[this.requiredCols[i]] += " *";
			}
            return headings[index];
        },
        
        getIsColumnSortable: function(content, atColumnIndex){
            return true;
        },
        
        getColumnWidth: function(content, atColumnIndex){
            var width = 2;
            switch (atColumnIndex) {
                case 0:
                    width = 30;
                    break;
                case 1:
                    width = 58;
                    break;
                case 2:
                    width = 12;
                    break;
            }
            return width;
        },

        getIsCellEditable: function(rowData, atColumnIndex){
            return true;
        },
        
        // Summary:
        //    The only assumptions the table have on the data that it displays
        //    is that it's an array contents. These are the data content/object that
        //    will be displayed in the table rows.
        //
        getContentObjects: function(content){
            // our content is already an array of objects
            return content;
        },
        
        getRowDataId: function(rowData){
            return rowData.itemId;
        },
        
        // Summary:
        //    Returns the type of the cell to be displayed. By default, the TableViewer
        //    will support a set of cell types. The type of the cell will map to a specific
        //    cell renderer (to be discussed).
        getCellDisplayType: function(rowData, atColumnIndex){
            return (atColumnIndex == 2)? RENDER_TYPE_DATETEXT : TableConstants.RENDER_TYPE_EDITABLE_TEXT;
        },
        
        // Summary:
        //    Returns the display data of the cell. For cell renderer type of TEXT, this will
        //    be the string. Others cell render type would be Date, or integer value, etc. If the 
        //    cell renderer type is image, the client can also supply cell data that can be sortable/groupable. 
        //    The TableViewer's default sorter, filter operates on this returned data. For "complex" 
        //    cell types such as image where it could have multiple values for different states, the 
        //    client also needs to implement the next method.
        //    
        //    Client must implement this method to provide the cell data.
        //    
        getCellDisplayData: function(rowData, atColumnIndex){
            var val = "data";
            switch (atColumnIndex) {
                case 0:
                    val = rowData.col1;
                    break;
                case 1:
                    val = rowData.col2;
                    break;
                case 2:
                    val = rowData.col3;
                    break;
            }
            return val;
        },
        
        
        // Summary:
        //    If applicable, returns the extended display information of the cell. The return extended
        //    info must be an object with the following attributes:
        //    
        //    {
        //      /* attribute common for all cell renderer types */
        //      "tooltip": <string>,
        //
        //      /* attributes for IMAGE and ACTION cell renderer types */
        //      "isHidden": <true, false> the image is hidden if the row is not on the mouse over
        //      "image": base image URL,  
        //      "mouseOverImage": image URL, (optional)
        //      "mouseOutImage": image URL,  (optional) (overrides "image")
        //
        //      /* attributes for SELECT cell renderer type */
        //      "select" : 'value', the selection in the combo
        //      "choices" : [{'text':'display text', 'value':'value'}, ] array of objects with 'text' and 'value' attributes
        //
        //      /* attributes for CHECKBOX cell renderer type */
        //      "state" : <true, false> initial state of the check box
        //
        //      /* attributes for LINK cell renderer type */
        //      "href" : <string> the href string for the link
        //    }
        //
        //    Client can implement this method to provide extended display infor for the cell.
        //    Certain cell renderer type will require the client to implement this method to
        //    provide needed display information, such as an image ACTION.
        //
        getCellDisplayExtendedInfo: function(rowData, atColumnIndex){
            var val = {};
            
            // only specify a fixed size text box in IE. Otherwise,
            // the text box width would grow if the input text is longer than
            // the visible size, which would stretch the table cell's width
            // This only happens in IE. 
            if (dojo.isIE) {
                if (atColumnIndex == 0) {
                    val['size'] = "24";
                }
                else 
                    if (atColumnIndex == 1) {
                        val['size'] = "40";
                    }
            }
            return val;
        }
    });
    
    dojo.declare("com.ibm.rqm.process.web.ui.internal.view.timebox.KeyDateTable", [dijit._Widget, dijit._Templated], {
        templateString: '<div dojoAttachPoint="domNode" tabindex="-1">' +
        '</div>',
        
		editable: true,
		printable: false,		
		widgetsInTemplate: true,
        
		
        constructor: function(args){
            this.phaseResults = null;
            this._itemId = null;
            this.content = null;
            this.owner = null;
            this.deleted = null;
            this.added = null;
        },
        
        postCreate: function(){
            this.inherited("postCreate", arguments);
        },
        
        initialize: function(args){
			this.parentSection = args.parentSection ? args.parentSection : null;
			this.printable = args.printable ? args.printable : false;
			this.editable = this.parentSection && this.parentSection.isEditable();
            this.deleted = new Array();
            this.phaseResults = args.timeboxResults;
			this.parentController = args.parentController;
            this._itemId = args.itemId;
			this._newId = 0;
            this.owner = args.owner;

	        //create the table viewer
	        var tableViewer = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer({
	            //turn off all controls if in print mode 
	            showPagingControl: !this.printable,
				showPagingRangeControl: !this.printable,
	            showGroupFilterControls: !this.printable,
	            showPagingActionsControls: !this.printable,
                allowAdjustSettings: true,
                autoSaveSettings: true,
                tableId: "RQM-PROC-KEYDATE-TABLEVIEWER"          
	        });
		
            this.domNode.appendChild(tableViewer.domNode);
            this.tableViewer = tableViewer;
			
            this.content = this.createContent();
            var contentProvider = new KeyDateTable.TableProvider();
			//The first column (Name) is required
			contentProvider.setRequiredColumns([0]);

            
            var permType = com.ibm.asq.common.web.client.internal.PermissionsClient.OP_PROCESS_PHASE;
            var perm = com.ibm.asq.common.web.client.internal.PermissionsClient.getStoredPermission(permType);
            var canAddDelete = (!perm || perm.create);
            
			tableViewer.setTableSummary(Constants.getMessage("labelTableSummary", Constants.getMessage("labelKeyDate")));
            
			if (canAddDelete)
			{
                tableViewer.addAction("add", Constants.PROCESS_ADD_KEY_DATES_TIME_BOX, null, "add-icon-image");
                this.deleteActionNode = tableViewer.addAction("remove", Constants.PROCESS_REMOVE_KEY_DATES_TIME_BOX, null, "remove-enabled-icon-image");
				this.deleteActionNode.setDisabled(true);
            }
            
			tableViewer.setContent(this.content);
            tableViewer.setContentProvider(contentProvider);
			tableViewer.setTableListener(new KeyDateTable.TableListener(this));
			tableViewer.setCellModifier(new KeyDateTable.CellModifier(tableViewer, this));
			tableViewer.registerCellRenderer(RENDER_TYPE_DATETEXT, new KeyDateTable.DateTextBox());
            tableViewer.show();
			
            //if parent section available and not editable, table should not be editable
            tableViewer.setEditable(this.editable);			
        },
        
        createContent: function(){
            var rows = [];
            if (this.phaseResults == null) {
                return rows;
            }
            for (var i = 0; i < this.phaseResults.length; i++) {
                var res = this.phaseResults[i];
                var obj = {
                    col1: res.name,
                    col2: res.description,
                    col3: res.startDate?res.startDate.split(" ")[0]:"",
                    itemId: res.itemId,
					stateId: res.stateId,
					isChanged: false,
                    type: 0 //type=0: existed; type=1: new;
                };
                rows.push(obj);
				this._addSaveParticipant(obj);
            }
            return rows;
        },
        
        _handleAddNewClick: function(){
            var obj = {
                col1: "",
                col2: "",
                col3: "",
				itemId: ++this._newId,
				stateId: null,
                type: 1
            };
            this.content.push(obj);
			this._addSaveParticipant(obj);
            this.tableViewer.setContent(this.content);
            this.tableViewer.show();
			if(this.parentSection){
				this.parentSection.setDirty();
			}
        },
        
        _handleDeleteClick: function(){
			this.parentSection.getParentEditor().clearStatus();
            var dels = this.tableViewer.getCheckedRows();
            if (dels && dels.length > 0) {
                for (var i = 0; i < dels.length; i++) {
                    this.deleted.push(dels[i]);
                    this.tableViewer.deleteItem(dels[i]);
					this._removeSaveParticipant(dels[i]);
                }
				this.content = this.tableViewer.getContent();
				this.tableViewer.show();
				if(this.parentSection){
					this.parentSection.setDirty();
				}
            }
            else {
				this.parentSection.getParentEditor().setHeaderStatus(Constants.OPERATION_WARNING, Constants.HEADER_STATUS_WARNING, Constants.PROCESS_REMOVE_MSG_TIME_BOX);
			}
        },
        
        changeRowEditable: function(isCheckble){
            if (isCheckble) {
                this.tableViewer.setTableRowsCheckable(this.editable && !this.printable, false);
                this.tableViewer.setEditable(true);
            }
            else {
                this.tableViewer.setTableRowsCheckable(false, false);
                this.tableViewer.setEditable(false);
            }
            this.tableViewer.show();
        },
        
		_addSaveParticipant: function(obj){
            var itemId = (obj == null)? null: obj.itemId; 
            var   saveParticipant = new SaveParticipant();
            
			saveParticipant.getItemId = function () { 
				return (obj.type == 1)?null:itemId; 
			};
			
			saveParticipant.setVersionableItemId(itemId);
			
			saveParticipant.getStateId = function(){
            	return obj.stateId;
        	};
            
			saveParticipant.getItemType = function(){
            	return "com.ibm.rqm.process.TestPhase";
        	};
			
			var self = this;
			saveParticipant.isDirty = function () {
				var isdirty = self._isDirty(itemId); 
				return isdirty;
			};

			saveParticipant.getAttributes = function(){
            	return self._getSaveParam(itemId);
        	};

            this.parentController.getSaveContext().addParticipant(saveParticipant);

            var parent = this.parentController.getParentEditor();
            if (parent != null) {
                this.parentController.getSaveContext().createReference(saveParticipant, "testPlanId", parent);
            }
        },
		
		_removeSaveParticipant: function(obj) {
			var itemId = obj.itemId;
		 	var saveContext = this.parentController.getSaveContext();
			if(obj.type == 1) {
				//new item
				var iter = saveContext._participants.getIterator();
				while (!iter.atEnd()) {
					var participant = iter.get();
					if (participant.getItemType() == "com.ibm.rqm.process.TestPhase" && 
						participant.getVersionableItemId() == itemId) {
						saveContext.removeParticipant(participant);
					}
				}
			}
		},
		
		_getSaveParam: function(itemId) {
			var param = {};
			var deleteArray = new Array();
			var addArray = new Array();
			var updateArray = new Array();
			if (this.deleted && this.deleted.length > 0) {
				for (var i = 0; i < this.deleted.length; i++) {
					if (this.deleted[i].type == 0 && this.deleted[i].itemId == itemId) {
						deleteArray.push({
							itemId: this.deleted[i].itemId
						});
					}
				}
			}
			if (this.content && this.content.length > 0) {
				for (var i = 0; i < this.content.length; i++) {
					if (this.content[i].type == 1 && this.content[i].itemId == itemId) {
						var obj = this.content[i];
						var params = {
							name: obj.col1,
							description: obj.col2,
							startDate: obj.col3,
							tableType: "keydate",
							testPlanId: this._itemId
						};
						addArray.push(params);
					}
				}
				for (var i = 0; i < this.content.length; i++) {
					if (this.content[i].type == 0 && this.content[i].itemId == itemId && this.content[i].isChanged) {
						var obj = this.content[i];
						var params = {
							itemId: obj.itemId,
							stateId: obj.stateId,
							name: obj.col1,
							description: obj.col2,
							startDate: obj.col3,
							tableType: "keydate",
							testPlanId: this._itemId
						};
						updateArray.push(params);	
					}
				}
			}
			param.remove = deleteArray;
			param.update = updateArray;
			param.add = addArray;
			return {operationJson: dojo.toJson(param)};
		},
		
		_isDirty: function(itemId) {
			if (this.deleted && this.deleted.length > 0) {
				for (var i = 0; i < this.deleted.length; i++) {
					if (this.deleted[i].type == 0 && this.deleted[i].itemId == itemId) {
						return true;
					}
				}
			}
			if (this.content && this.content.length > 0) {
				for (var i = 0; i < this.content.length; i++) {
					if (this.content[i].type == 1 && this.content[i].itemId == itemId) {
						return true;
					}
				}
				for (var i = 0; i < this.content.length; i++) {
					if (this.content[i].type == 0 && this.content[i].itemId == itemId && this.content[i].isChanged) {
						return true;
					}
				}
			}
			return false;
		},
		
		//return value is {msg: string, value: boolean}
		validate: function() {
			if (this.content && this.content.length > 0) {
				for (var i = 0; i < this.content.length; i++) {
					var obj = this.content[i];
					if (obj.col1 == "" || obj.col1==null) {
						return {
							msg: Constants.PROCESS_VALIDATE_REQUIRE_MSG_TIME_BOX,
							value: false
						};
					}
				}
			}
			return this._checkUniqueName();
		},
		
		removeAllSaveParticipant: function() {
			var objs = this.tableViewer.getContent();
            for (var i = 0; i < (objs ? objs.length:0); i++) {
				this._removeSaveParticipant(objs[i]);
            }
		},
		
		_checkUniqueName: function() {
			if (this.content && this.content.length > 0) {
				var type = Constants.PROCESS_KEY_DATE_TABLE_TIME_BOX;
				for (var i = 0; i < this.content.length; i++) {
					for (var j = i+1; j < this.content.length; j++) {
						if(this.content[i].col1 == this.content[j].col1) {
							return {
                                msg: Constants.getMessage("msgNameRequireMsgTimeBox", this.content[j].col1, type),
								value: false
							};
						}
					}
				}
			}
			return {value: true};
		},
		
        _error: function(errorObj){
            alert(errorObj.message);
        }
    });
})();