/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: L. To
//
// Summary:
//

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider");
 
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea");

dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ArrayAttributeTitle");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.TemplatedAssetContentProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var SectionHeader = com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader;
var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider", 
    [com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider],
{   
    constructor: function()
    {
        this.sectionCreatedState = {};
        this.subSectionCreatedState = {};
    },
    
    // Summary:
    //    Returns a widget that is the section header.
    //
    constructSectionHeader: function(sectionKey)
    {
        var sectionInfo = this.layoutForm.getContentProvider().getSectionInfo(this.layoutForm.getContent(), sectionKey);   
        var header = null;
        var isOpen = (typeof sectionKey.open != "undefined") ? sectionKey.open : true;
        
        // if we already have shown this section, this's not the initial section creation,
        // show it always.
        if(this.sectionCreatedState[sectionKey.id]) {
            isOpen = true;
        }
        
        // provide a help id for the section if one is defined in the template.
        var helpId = (sectionKey.contextHelpId) ? sectionKey.contextHelpId : (sectionKey.contextHelpIdNew) ? sectionKey.contextHelpIdNew : null;

        header = new com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionHeader({
                title: RtlmUiUtil.getMessage(sectionInfo.title), 
                contextHelpId: helpId,
                csHelpProviderId: com.ibm.rational.test.lm.web.CSHelpID,
                open: isOpen,
                'sectionKey': sectionKey
            });
            
        this.sectionCreatedState[sectionKey.id] = header;
        
        for( var iAction = 0; iAction < sectionKey.actions.length; iAction++ )
        {
        	if(sectionKey.actions[ iAction ] == "add" )
            	header.addAction("add", dojo.hitch(this, "_handleAddAttributeValueFromSection"), sectionKey, "add-icon-image");
        }
        
        return header;
    },

    getSectionHeaderContentNodeName: function()
    {   
        return 'containerNode'; 
    },
    
    // Summary:
    //    Returns a widget that is the header for collection attributes.
    //
    constructCollectionAttributeHeader: function(sectionKey, attributeKey)
    {
        var isArrayVal = (attributeKey.type && attributeKey.type == DefaultAssetContentProvider.TYPE_ATTRIBUTE_MULTI_VALUE);
        
        // Create a sub-section for the attribute if it's declared in the section
        // header. But we need to handle the case where an attribute is an array
        // and it has sub-values with the same attribute name. Ensure that its
        // type is not a multi-value attribute before creating the header for it.
        //

        if(typeof sectionKey.subsections != "undefined") {
            // check if the attribute is in the list of subsections
            var csHelpId = this._inList(sectionKey.subsections, attributeKey.attributeName);
            // csHelpId as string (not just true) is returned if the subsection included
            // the CS Help id
            if (csHelpId && !isArrayVal) {
                var isOpen = false;

                if(this.subSectionCreatedState[attributeKey.id]) {
                    isOpen = true;
                }
                var subsection = new com.ibm.rational.test.lm.web.ui.internal.widgets.ActionTitlePane({
                    title: RtlmUiUtil.getMessage(attributeKey.attributeName + "-section"),
                    open:isOpen});
                
                // set subsection CS Help id if it is returned
                if (typeof csHelpId == 'string') {
                    this.setCollectionAttributeHeaderCSHelpId(subsection, csHelpId);
					var label = RtlmUiUtil.getMessage("labelHelp", RtlmUiUtil.getMessage(attributeKey.attributeName + "-section"), "");
					this.setCollectionAttributeHeaderCSHelpLabel(subsection, label);
                }
            
                var parentPart = attributeKey.parentPart;
                var attrName = attributeKey.attributeName;
                var attr = parentPart.getAttribute(attrName);
                var type = attr.getType();
            
                if(attr.isArray())
                    subsection.addTool(RtlmUiUtil.getMessage("add_button"), dojo.hitch(this, "_handleAddAttributeValue"), attributeKey, "add-icon-image");
                
                this.subSectionCreatedState[attributeKey.id] = subsection;
                return subsection;
            }
        }
        return null;
    },
    
    setCollectionAttributeHeaderCSHelpId: function(subsection, id)
    {
        subsection.setCSHelpId(id);
    },
    
	setCollectionAttributeHeaderCSHelpLabel: function(subsection, label)
    {
        subsection.setCSHelpButtonLabel(label);
    },
	
    _inList: function(list, value)
    {
        for(var i = 0; i < list.length; i++) {
            // the subsection list entries may contain a CS help id
            // in the format of <title>?<CS help id>
            var csHelpPos = list[i].indexOf("?");
            var subsectionTitle = list[i];
            if (csHelpPos > 0) {
                subsectionTitle = list[i].substring(0, csHelpPos);
            }
            if (subsectionTitle == value) {
                if (csHelpPos > 0)
                    // return the CS help id, not just true, if there is one
                    return list[i].substring(csHelpPos + 1);
                else 
                    return true;
            }
        }
        return false;
    },
    
    _handleAddAttributeValueFromSection: function(sectionKey)
    {
        var groups = sectionKey.groups;
        var root = groups[0].root;
        var asset = this.layoutForm.getContent();
        var parentPart = this._getAttributeParent(asset, root);
        var attrName = root.slice(root.lastIndexOf(".") + 1);
        var attr = parentPart.getAttribute(attrName);
        var type = attr.getType();
        var value = "";
        
        if(sectionKey.actions[0].indexOf(".") > -1) {
            var subAttrName = sectionKey.actions[0].slice(sectionKey.actions[0].indexOf(".") + 1);
            return this._addSubAttributeToRoot(sectionKey, attr, subAttrName);
        }
        
        if (type == DefaultAssetContentProvider.TYPE_PART) {
            var rootPart = attr.getRootPart();
            value = rootPart.getName();
        }
        // Add a new empty value to the attribute
        // and invalidate the layout for this attribute
        // to pick up the new value.
        attr.addArrayValue(value);
        this.invalidateSectionLayout(sectionKey);
    },

    _addSubAttributeToRoot: function(sectionKey, parentAttr, attrName)
    {
        var parentPart;
        var attr;
        var type;
        var needUpdateOther = false;
        
        if (parentAttr.isArray()) {
            var values = parentAttr.getArrayValues();
            var parentPart;
            
            if (values && values.length > 0) {
                if (values.length == 1) {
                    parentPart = values[0];
                }
                else {
                    // bring up a dialog to ask the user which parent to add
                    // the sub-attribute to.
                    this._selectParentArrayValue(sectionKey, parentAttr, attrName, values);
                    return;
                }
            }
            else {
                // The parent attribute doesn't have any value,
                // auto add the root part to it.
                parentAttr.addArrayValue(parentAttr.getRootPart().getName());
                
                values = parentAttr.getArrayValues();
                parentPart = values[0];
                needUpdateOther = true;
            }
            attr = parentPart.getAttribute(attrName);
            var value = "";
            
            type = attr.getType();
            if (type == DefaultAssetContentProvider.TYPE_PART) {
                var rootPart = attr.getRootPart();
                value = rootPart.getName();
            }
            
            // Add a new empty value to the attribute
            // and invalidate the layout for this attribute
            // to pick up the new value.
            attr.addArrayValue(value);
            this.invalidateSectionLayout(sectionKey);
        }
        // we also changed the parent part. Update the other
        // section too to pickup the new value.
        if(needUpdateOther)
            this._updateOtherSectionAfterAction(sectionKey.updateOtherSections); 
    },
    
    _updateOtherSectionAfterAction: function(sections) 
    {
        if(sections) {
            for(var i = 0; i < sections.length; i++) {
                this.invalidateSectionLayout(sections[i]);
            }
        }
    },

    _selectParentArrayValue: function(sectionKey, parentAttr, attrName, osValues) {
        this.osSelect = -1;
		var selectLabel = document.createElement("label");
		this.oslist = document.createElement("select");
        this.oslist.multiple = false;
        this.oslist.size = 6;
		
        this.oslist.className = "osSelect";
        this.oslist.onchange = dojo.hitch(this, "_choiceOSselected");
        
        for(var i = 0; i < osValues.length; i++) {
            this.oslist.options[i] = new Option(osValues[i].getName());
        }
        var callbackData = {};
        callbackData["sectionKey"] = sectionKey;
        callbackData["parentAttr"] = parentAttr;
        callbackData["attrName"] = attrName;
        callbackData["values"] = osValues;
        
        var args = {
            width: "30%",
            okCallback: dojo.hitch(this, "_selectOSConfirmed"),
            okCallbackData: callbackData
        };
        var message = document.createElement("div");
        message.className = "osSelectPrompt";
        message.innerHTML = RtlmUiUtil.getMessage("selectOSAddSWPrompt");

		selectLabel.appendChild(this.oslist);
        this.osSelectDialog = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
        this.osSelectDialog.contentContainer.appendChild(message);
        this.osSelectDialog.contentContainer.appendChild(selectLabel);
        this.osSelectDialog.setTitle(RtlmUiUtil.getMessage("selectOSTitle"));
        this.osSelectDialog.open();
        
        return this.osSelect;
    },
    
    _selectOSConfirmed: function(callbackData)
    {
        if(this.osSelect < 0)
            return;
            
        var parentPart = callbackData.values[this.osSelect];
        var attr = parentPart.getAttribute(callbackData.attrName);
        var value = "";
        
        var type = attr.getType();
        if (type == DefaultAssetContentProvider.TYPE_PART) {
            var rootPart = attr.getRootPart();
            value = rootPart.getName();
        }
        
        // Add a new empty value to the attribute
        // and invalidate the layout for this attribute
        // to pick up the new value.
        attr.addArrayValue(value);
        
        this.invalidateSectionLayout(callbackData.sectionKey);
    },
    
    _choiceOSselected: function()
    {
        this.osSelect = -1;
        for(var i = 0; i < this.oslist.options.length; i++) {
            if(this.oslist.options[i].selected) {
                this.osSelect = i;
                break;
            }
        }
        this.osSelectDialog.getOkButton().disabled = (this.osSelect == -1);
    },
    
    _isAttributePathRoot: function(attrPath) 
    {
        return attrPath.indexOf(".") < 0;
    },
    
    _getAttributeParent: function(asset, attributePath)
    {
        var parentPos = attributePath.lastIndexOf(".");
        var parentPath = (parentPos > 0) ? attributePath.substr(0, parentPos) : attributePath;
        return this._getAttributeByPath(asset, parentPath);
    },
    
    _getAttributeByPath: function(asset, attrPath)
    {
        var part = asset.getRoot();
        var paths = attrPath.split(".");
        var i = 0;
        var attr;
        
        while(i < paths.length - 1) {
            try {
                var attrName = paths[i + 1];
                
                attr = part.getAttribute(attrName);
                
                // we're not at the attribute we wanted yet
                // Ensure this is a valid part.
                if((i + 1) < paths.length) {
                    if(attr.getType() != DefaultAssetContentProvider.TYPE_PART) {
                        throw new Error("Invalid attribute Path: " + attrPath);
                    }
                    part = attr;
                }
                i++;
            }
            catch(error) {
                throw new Error("Invalid attribute Path: " + attrPath);
            }
        }
        return part;
    },
    
    // Summary:
    //    convenient method to invalidate the layout of a section. This method
    //    tries to get the section's sub-attributes and invalid the sub-attributes
    //    content. If no section's attributes (very unlikely case), it will invalidate
    //    the entire section's layout.
    //
    invalidateSectionLayout: function(sectionKey) 
    {
        var rootAttrs = this.layoutForm.getSectionAttributes(sectionKey);
        
        if(rootAttrs) {
            for(var i = 0; i < rootAttrs.length; i++) {
                this.layoutForm.invalidateAttributeLayout(rootAttrs[i]);
            }
        }
        else {
            this.layoutForm.invalidateSectionLayout(sectionKey);
        }
    }
});

})();
