/**AssetDetailView**************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: L. To
//
// Summary:
//    The viewer for displaying an RTLM asset. Based on the asset type, it constructs 
//    the appropriate detail view using the generic SectionLayoutForm. This viewer constructs
//    proper content provider specific to the asset type using the "AssetContentProviderFactory"
//    object. An asset type's display content is captured in the provider.
//    
//
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetDetailView");
 
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Dialog");

dojo.require("com.ibm.team.repository.web.client.init.InitializationData");
dojo.require("com.ibm.asq.common.web.client.internal.Catalog");
dojo.require("com.ibm.asq.common.web.client.internal.PermissionsClient");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.CommonDialog");
dojo.require("com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.MoreLessCollapsibleArea");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.EditorSectionContainer");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.RelatedInfo");
dojo.require("com.ibm.asq.common.web.ui.internal.util.GlobalEvents");
dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.Utility");

dojo.require("com.ibm.rational.test.lm.web.client.internal.admin");
dojo.require("com.ibm.rational.test.lm.web.client.internal.RtlmInitializer");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormListener");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetContentProviderFactory");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.MachineReservationsPanel");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.widgets.MachineHistoryPanel");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyEditor");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.InventoryUriUtil");
dojo.require("com.ibm.rational.test.lm.web.client.internal.inventory");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationEditor");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.accessibility.SkipNavigation");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("net.jazz.ajax.ui.PlatformUI");

(function() {

var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;
var ViewHeaderLayout = com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout;
var SkipNavigation = com.ibm.asq.common.web.ui.internal.widgets.accessibility.SkipNavigation;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

// Summary:
// 
dojo.declare("AssetDetailView.HeaderElementProvider", 
             [com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultCustomElementProvider],
{   
    // Summary:
    //    Returns a widget that is the section header.
    //
    constructSectionHeader: function(sectionKey)
    {
        return document.createElement("div");
    },
    getSectionHeaderContentNodeName: function()
    {   
        return null; 
    }
});


// Summary:
//     Custom element provider for the asset detail view's section form. It creates a 
//     editor's section container as the section's header area. The editor section container
//     can display attributes/summary information in its header area.
// 
dojo.declare("AssetDetailView.EditorCustomElementProvider", 
             [com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider],
{   

    isNew: false,
    
    refreshHeadersInfo: function()
    {
        for(var i in this.sectionCreatedState) {
            var header = this.sectionCreatedState[i];
            this._loadSummaryInfo(header['sectionKey'], header);
        }
    },
    
    // Summary:
    //    Returns a widget that is the section header.
    //
    constructSectionHeader: function(sectionKey)
    {
        var sectionInfo = this.layoutForm.getContentProvider().getSectionInfo(this.layoutForm.getContent(), sectionKey);   
        var header = null;
        var isOpen = (typeof sectionKey.open != "undefined") ? sectionKey.open : true;
        
        // if we already have shown this section, this's not the initial section creation,
        // show it always.
        if(this.sectionCreatedState[sectionKey.id]) {
            isOpen = true;
        }
        
        var csHelpId = sectionKey.contextHelpId;
        if (this.isNew) {
            csHelpId = sectionKey.contextHelpIdNew;
            
            if (sectionKey.id == "machine.details" || sectionKey.id == "vm.details")
                isOpen = true;
        }
            
        header = new com.ibm.asq.common.web.ui.internal.view.common.EditorSectionContainer({
                title: RtlmUiUtil.getMessage(sectionInfo.title), 
                contextHelpId: csHelpId,
                csHelpProviderId: com.ibm.rational.test.lm.web.CSHelpID,
                open: isOpen,
                editable: true,
                headerActionsAlign: "right", // align the action to the right
                'sectionKey': sectionKey
            });
        header.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", "", RtlmUiUtil.getMessage(sectionInfo.title)));
		    
        this.sectionCreatedState[sectionKey.id] = header;
        		
		var skip = new SkipNavigation( {
			linkname: RtlmUiUtil.getMessage(sectionInfo.title),
			parent: this.layoutForm.formListener.chainedListener.domNode,
			insertReference: this.layoutForm.formListener.chainedListener.viewHeaderNode.domNode,
			onActivate: function(){
				if (!header.open) 
					header.toggle();
			},
			onFocus: function(){
				header.setFocus();
			}
		} );

        if(sectionKey.actions) {
            var label = RtlmUiUtil.getMessage("add-" + sectionKey.id);
            header.addAction(label, dojo.hitch(this, "_handleAddAttributeValueFromSection"), sectionKey, null, "add-icon-image");
        }
        this._loadSummaryInfo(sectionKey, header);
        return header;
    },
    
    _loadSummaryInfo:function(sectionKey, header)
    {
        var groups = sectionKey.groups;
        var root = groups[0].root;
        var asset = this.layoutForm.getContent();
        var isAssetRoot = this._isAttributePathRoot(root);
        var part = this._getAttributeParent(asset, root);
        var rootAttrName = null;
        var rootAttr = null;
        
        // clear the info area first.
        header.clearAllInfoAttributes();
        header.setSummary("");
        
        if(!isAssetRoot) {
            rootAttrName = root.slice(root.lastIndexOf(".") + 1);
            rootAttr = part.getAttribute(rootAttrName);
            
            if(rootAttr.getType() != DefaultAssetContentProvider.TYPE_PART)
                return;
        }
        
        if(groups[0].infoAttributes) {
            for(var i = 0; i < groups[0].infoAttributes.length; i++) {
                var attrName = groups[0].infoAttributes[i];
                var title = RtlmUiUtil.getMessage(attrName);
                var val = null;
                
                if(rootAttr) {
                    val = this._getAttributeDisplayValue(rootAttr, attrName);
                }
                else {
                    val = this._getPartAttributeValue(part, attrName);
                }
                
                // only add the summary info if the attr has value.
                //
                if(val && val.length > 0) {
                    header.addInfoAttribute(title, "<b>" + val + "</b>", null, null, false, null);
                }
            }
        }
        if(groups[0].summaryAttribute && part) {
            var attrName = groups[0].summaryAttribute[0];
            var attr = part.getAttribute(attrName);
            var title = RtlmUiUtil.getMessage(attrName);
            var val = (attr.getValue()) ? attr.getValue() : "";
            header.setSummary(val);
        }
    },
    
    _getAttributeDisplayValue: function(attr, attrName)
    {
        var val = null;
        
        if(attr.isArray()) {
            var arrayValue = attr.getArrayValues();
            var rootPartName = attr.getRootPart().getName();
            
            if(arrayValue.length > 0) {
                var val = "";
                var tmp = null;
                
                for(var i = 0; i < arrayValue.length; i++) {
                    var attrPart = arrayValue[i];
                    
                    if(attrName == attr.getName()) {
                        // don't show this part's value, if it's the root part.
                        // We use a blank to denote the root. It might be confusing
                        // for the user to see the info displaying the root value
                        // such as 'OperatingSystem'.
                        if (rootPartName != attrPart.getName()) {
                            tmp = attrPart.getName();
                            val = val + attrPart.getName();
                        }
                    }
                    else {
                        tmp = this._getPartAttributeValue(attrPart, attrName);
                        if(tmp)
                            val = val + tmp;
                    }
                    if(i + 1 < arrayValue.length && tmp)
						val = val + ", ";
                }
                if(val.length == 0)
                    val = null;
            }
        }
        else {
            var part = attr.getValue();
            val = this._getPartAttributeValue(part, attrName);
        }
        return val;
    },
    
    _getPartAttributeValue: function(part, attrName)
    {
        var attrVal = "";
        var attr = part.getAttribute(attrName);
        var type = attr.getType();
        var isPart = (type == DefaultAssetContentProvider.TYPE_PART);
        
        if (attr.isArray()) {
            var arrayValue = attr.getArrayValues();
            var rootPartName = isPart ? attr.getRootPart().getName() : null;
            
            for (var i = 0; i < arrayValue.length; i++) {
                var attrPart = arrayValue[i];
                
                // don't show this part's value, if it's the root part.
                // We use a blank to denote the root. It might be confusing
                // for the user to see the info displaying the root value
                // such as 'OperatingSystem'.
                if(!rootPartName || rootPartName != attrPart.getName())
                    attrVal = attrVal + ((i > 0) ? ", " : "") + attrPart.getName();
            }
        }
        else {
            attrVal = attr.getValue();
            
            if (attrVal) {
                attrVal = (type == DefaultAssetContentProvider.TYPE_POINTER) ? attrVal.getDisplayString() : attrVal;
            }
        }                
        return attrVal;
    },
    
    // override the method to show CS help only if it is new
    //
    setCollectionAttributeHeaderCSHelpId: function(subsection, id)
    {
        if (this.isNew) {
            this.inherited("setCollectionAttributeHeaderCSHelpId", arguments);
        }
    }
});

var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var ServiceResponseHandler = com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper;
var LOADING = RtlmUiUtil.getMessage("appWorking");
var AssetContentProviderFactory = com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetContentProviderFactory;
var CustomElementProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider;
var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
var InventoryUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.InventoryUriUtil;
var GlobalEvents = com.ibm.asq.common.web.ui.internal.util.GlobalEvents;
var ConfigurationUriUtil = com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationsUriUtil;
var ConfigurationEditor = com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationEditor;
var PermissionsClient = com.ibm.asq.common.web.client.internal.PermissionsClient;
var YesNoDialog = com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog;

var EDIT = "canEdit";
var ADD_TO_BF = "canAddToBf";
var AUTOMATE = "canAutomate";
var DISCOVER = "canDiscover";
var RESERVE = "canReserve";


dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetDetailView", 
    [dijit._Widget, dijit._Templated,
     com.ibm.asq.common.web.ui.internal.view.common.RelatedInfoMixin,
     com.ibm.rational.test.lm.web.ui.internal.widgets.layout.FormListener],
{    
    templateString: '<div class="asset-detail-view" dojoAttachPoint="domNode" tabindex="-1">' +
                    '  <div dojoType="com.ibm.asq.common.web.ui.internal.view.layout.ViewHeaderLayout" titleEditable="true" dojoAttachPoint="viewHeaderNode">' +
                    '    <div dojoAttachPoint="leftFormNode" class="section-form">' +
                    '    </div>' +
                    '  </div>' +
                    '</div>',

    widgetsInTemplate: true,
	
	// initialize the view's labels from the message catalog.
	saveButtonLabel: RtlmUiUtil.getMessage("save_button"),
    discardButtonLabel: RtlmUiUtil.getMessage("discard_button"),
    
    discardButtonNode: null,
    saveButtonNode: null,

	// initial status message to display when the view is done loading.
	statusMessage: null,
	
	// the asset ID being viewed.
    assetId: null,
	
	// if the asset is new (doesn't exist yet)
	isNew: false,
	
    // if the asset's archive state is deleted.
    isDeleted: false,
    
	// the type of the assset
	assetType: null,
	
	// the actual asset object
	asset: null,
	
	// interal catalog object
	theCatalog: null,
	
	// additional info display panels of the asset (only shown if the asset already exists)
	historyPanel: null,
	reservationsPanel: null,
	
    staticRelatedInfo: null,
    staticRelatedVMInfo: null,
	_isDirty: false,
    
    getStaticRelatedInfo: function() {
        if (this.assetType == "Machine")
            return this.staticRelatedInfo;
        else
            return this.staticRelatedVMInfo;
    },
        
    getStaticRelatedInfoId: function() {
        return "com.ibm.rational.test.lm.web.ui.internal.views.machines.AssetDetailView." + this.assetType
    },

    postCreate: function() 
	{
        this.inherited("postCreate", arguments);

        this._updateHeader();
		
        this.discardButtonNode = this.viewHeaderNode.addRightViewAction(this.discardButtonLabel, dojo.hitch(this, "_onDiscardActionClick"), null, null);
        this.saveButtonNode = this.viewHeaderNode.addRightViewAction(this.saveButtonLabel, dojo.hitch(this, "_onSaveActionClick"), null, null);

        this.theCatalog = new com.ibm.asq.common.web.client.internal.Catalog();

        this.viewHeaderNode.setSummaryEditable(true);
        
        this.viewHeaderNode.onTitleEdited = dojo.hitch(this, "_nameChanged");
        this.viewHeaderNode.onSummaryEdited = dojo.hitch(this, "_descriptionChanged");
		
        this._resetButtonStates();

        if(this.isNew) {
            this.asset = this.theCatalog.createAsset(this.assetType);
            this._constructForm();
            // for new lab resource set focus to the title
            this.viewHeaderNode.viewEditableTitle.focusWithPrompt();
        }
        else {
            this._initResourcePermissions();
        }
    },
    
    _initResourcePermissions: function(resHandler)
    {
        var adminClient = new com.ibm.rational.test.lm.web.client.internal.admin();
        var responseHandler = (resHandler) ? resHandler : new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(
            {
                parent: this,
                _success: function(response) 
                {
                    this.parent.permissions = response[0];
                    this.parent._loadAssetById(this.parent.assetId);
                },
                _failure: function(errObj) 
                {
                    this.parent._loadAssetById(this.parent.assetId);
                }
            },
            "_success",
            "_failure"
        );
        adminClient.getResourcePermissions(responseHandler, this.assetId);
    },
    
    _canPerformAction: function(actionName)
    {
        // if the asset is deleted, no actions can be performed.
        if (this.isDeleted) {
            return false;
        }
        
        // if no permission, or the type of the permission
        // is not present, optimistically, allow the action.
        // B/E will guard the action.
        return !this.permissions || this.permissions[actionName] != false; 
    },
    
    _resetButtonStates: function()
    {
        this.saveButtonNode.disabled = true;
        this.discardButtonNode.disabled = true;        
    },
    
    _nameChanged: function(value)
    {
		var oldvalue = this.asset.getRoot().getAttributeValue( "name" );
        this.asset.getRoot().setAttributeValue("name", value);
        this.attributeValueChanged( "name", oldvalue, value );
    },

    _descriptionChanged: function(value)
    {
		var oldvalue = this.asset.getRoot().getAttributeValue( "Description" );
        this.asset.getRoot().setAttributeValue("Description", value);
        this.attributeValueChanged("Description", oldvalue, value);
    },
    
    _hasHeaderEmptyRequireField: function()
    {
        var nameAttr = this.asset.getRoot().getAttribute("name");
        var nameVal = this.asset.getRoot().getAttributeValue("name");
        
        return (nameAttr.isRequired() && (!nameVal || nameVal.length == 0));
    },
    
    _updateHeader: function()
    {
        if (this.assetType == "Machine") {
            this.viewHeaderNode.setIconClass("machine-icon-48");
            this.viewHeaderNode.setCSHelpId(this.isNew ? "rtlm_new_machine.html" : "rtlm_machine_details.html", com.ibm.rational.test.lm.web.CSHelpID);
        } else if (this.assetType == "VirtualImage") {
            this.viewHeaderNode.setIconClass("vm-icon-48");
            this.viewHeaderNode.setCSHelpId(this.isNew ? "rtlm_create_virtual_image.html" : null, com.ibm.rational.test.lm.web.CSHelpID);
        }
		if(this.isNew)
			this.viewHeaderNode.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", RtlmUiUtil.getMessage("labelNew"), this.assetType));
		else
			this.viewHeaderNode.setCSHelpButtonLabel(RtlmUiUtil.getMessage("labelHelp", this.assetType, RtlmUiUtil.getMessage("labelDetail")));
        var typeName = RtlmUiUtil.getMessage(this.assetType);
        this.viewHeaderNode.setTypeLabel(typeName);
        this.viewHeaderNode.setNoValueEditableTitleLabel(RtlmUiUtil.getMessage("newAssetPrompt", typeName));
		
		this._updateBrowserTitle();
    },
	
	_loadDataSuccess: function(response)
	{
        this.viewHeaderNode.setProgressEnd();
		
		this.asset = response;
        var name = this.asset.getRoot().getAttributeValue("name");
		this.resourceName = name;
		this._constructForm();
		this._loadAssetInfoPanels();
		
        this.viewHeaderNode.setTitle(name);
        this.viewHeaderNode.setSummary(this.asset.getRoot().getAttributeValue("Description"));
        
        this._updateHeader();
        this._resetButtonStates();
        
        // show the proper asset's name in its parent container.
        var iconClass = this._getIconForAssetType();
        this._updateTabLabel(false);
        
        if(!this._canPerformAction(EDIT)) {
            var msg = (this.isDeleted) ? RtlmUiUtil.getMessage("statusNoEditDeleted") : RtlmUiUtil.getCommonMessage("statusNoEditPerm");
            var details = (this.isDeleted) ? RtlmUiUtil.getMessage("statusNoEditDeletedDetails") : RtlmUiUtil.getCommonMessage("msgNoEditPermDetails", RtlmUiUtil.getMessage(this.assetType));
            
            this.viewHeaderNode.setStatus(msg, ViewHeaderLayout.WARNING, details);            
        }        
        else if(this.statusMessage) {
            // last operation may want to display a message, after a page load.
            // show it and clear the message once it's displayed.
            this.viewHeaderNode.setStatus(this.statusMessage, ViewHeaderLayout.INFO);
            this.statusMessage = null;
        }
	},
	
	_updateTabLabel: function(dirty)
	{
		if (this.isNew)
			return;  //Never update the label of a new machine.
			
        var name = this.asset.getRoot().getAttributeValue("name");
        var iconClass = this._getIconForAssetType();
        dojo.publish(GlobalEvents.UpdateTabLabel, [window.location.hash, name, iconClass, dirty]);	
	},
	
    _loadDataFailure: function(response)
    {
        this.viewHeaderNode.setProgressEnd();
        this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("fetchAssetFailed"), ViewHeaderLayout.ERROR);    

        // call the common artifact data load error handler.  
        var details = (response && response.message) ? response.message : null;
        com.ibm.asq.common.web.ui.internal.util.Utility.handleArtifactLoadError(RtlmUiUtil.getMessage(this.assetType), details);
    },
	
	// Summary:
	//    Retrieves the asset's full content based on the specified Id. This method invokes
    //    an asynchrous call to the server to get the asset contents.
	// 
	_loadAssetById: function(id)
	{
        this.viewHeaderNode.setProgressStart(LOADING, /*hideContent*/true);
		var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_loadDataSuccess", "_loadDataFailure");
		this.theCatalog.getAsset(responseHandler, id, this.assetType);	
	},
	
	_constructForm: function()
	{
        this._constructHeaderAttributeFields();     

		// construct the detail form for the asset's attribute. Use the appropriate
		// content provider for the asset type.
		//
        this.formLayout = new com.ibm.rational.test.lm.web.ui.internal.widgets.layout.SectionLayoutForm(this.leftFormNode);
        this.formElementProvider = new AssetDetailView.EditorCustomElementProvider(this.formLayout);
		this.provider = AssetContentProviderFactory.getAssetContentProvider(this.assetType, this.formLayout, this);

        this.formElementProvider.isNew = this.isNew;
		this.formLayout.setContentProvider(this.provider);
		this.formLayout.setCustomElementProvider(this.formElementProvider);
		
		this.formLayout.setContent(this.asset);
		this.formLayout.doLayout();
		this.formLayout.loadDataToForm();
        
        // the history & the reservation panels now is a part of
        // the form's widget. If we are reconstructing the form,
        // we need to recreate these panels as well.
        if(this.historyPanel)
            delete this.historyPanel;
        if(this.reservationsPanel)            
            delete this.reservationsPanel;
    },

    _constructHeaderAttributeFields: function()
    {
        var label;
        var attr;

        this.viewHeaderNode.getInfoAttributesNode().innerHTML = "";        

        label = document.createElement("label");
        label.setAttribute("for", "asset-group-text");
        
        label.innerHTML = RtlmUiUtil.getMessage("group");
        attr = document.createElement("label");
        
        var group = this.asset.getRoot().getAttributeValue("group");
        attr.innerHTML = group ? group.getDisplayString() : RtlmUiUtil.getMessage("none");
        this.viewHeaderNode.addInfoAttribute(label, attr);
        
        label = document.createElement("label");
		label.setAttribute("for", "asset-detail-select");
		
        label.innerHTML = RtlmUiUtil.getMessage("owner");
        this.usersAttr = document.createElement("select");
		this.usersAttr.name = "asset-detail-select";
		this.usersAttr.id = "asset-detail-select";
        this.viewHeaderNode.addInfoAttribute(label, this.usersAttr);
        
        this.usersAttr.onchange = dojo.hitch(this, "_setOwner");
        this.ownerField = this.usersAttr;

        var getUsersHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "setUsersSelectContents", "setUsersSelectContents");
        com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.updateAllUsers(getUsersHandler);
        
        if(this.assetType == "Machine") {
            label = document.createElement("label");
            label.innerHTML = RtlmUiUtil.getMessage("Operational Status");;
			label.setAttribute("for", "asset-operational-select");
			
            attr = document.createElement("select");
			attr.name = "asset-operational-select";
			attr.id = "asset-operational-select";
		
            this.viewHeaderNode.addInfoAttribute(label, attr);
            var s_attr = this.asset.getRoot().getAttribute("Operational Status");
            var status = s_attr.getValue();
            this.setSelectContents(attr, s_attr.getEnumValues(), true, status);     
            
            attr.onchange = dojo.hitch(this, "_setOperationalStatus");
            this.opStatusField = attr;            
        }
    },

    setUsersSelectContents: function()
    {
        var users = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUsers();
        var owner = this.asset.getRoot().getAttributeValue("owner");
        this._loadUsersControl(this.usersAttr, users, true, ((owner) ? owner.getDisplayString() : null));
    },
    
    _loadAssetInfoPanels: function()
	{
		if (!this.initActions) {
            var tePerm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_TEST_ENVIRONMENT);  
            var virtualizePerm = PermissionsClient.getStoredPermission(PermissionsClient.OP_LM_VIRTUALIZATION);  
            
            if (this.assetType == "Machine") {
                if (this._canPerformAction(AUTOMATE)) {
                    this.automationButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("Automation"), dojo.hitch(this, this._onRunAutomationActionClick), null, "configure-automation-icon-image");
                    this.automationButton.setDisabled(true);
                }
                if (this._canPerformAction(DISCOVER)) {
                    this.discoverButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("Discover"), dojo.hitch(this, this._onDiscoverActionClick), null, "discover-icon-image");
                    this.discoverButton.setDisabled(true);
                }
				var name = this.asset.getRoot().getName();
				if( name == "VirtualMachine" && (!virtualizePerm || virtualizePerm[PermissionsClient.ACTION_DEPLOY]) )
					this.undeployButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("Undeploy"), dojo.hitch(this, this._onUnDeployActionClick), null, "undeploy-icon-image");
				
                if(this._canPerformAction(ADD_TO_BF))
				    this.addtobldfrgButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("Add to provider"), dojo.hitch(this, this._onAddToBuildForgeActionClick), null, "addtobldfrg-icon-image");
                    
                if(!tePerm || tePerm[PermissionsClient.ACTION_CREATE])    
	           	   this.createConfigButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("newConfiguration"), dojo.hitch( this, this._onCreateConfigActionClick ), null, "config-from-machine-icon-image" );			
			}
			else 
				if (this.assetType == "VirtualImage"  && (!virtualizePerm || virtualizePerm[PermissionsClient.ACTION_DEPLOY]) ) 
					this.deployButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("Deploy"), dojo.hitch(this, this._onDeployActionClick), null, "configure-deploy-icon-image");
            
            if( !this.isNew )
            	this.refreshButton = this.viewHeaderNode.addLeftViewAction(RtlmUiUtil.getMessage("Refresh"), dojo.hitch( this, this._onRefreshActionClick ), null, "refresh-icon-image" );
            
            this.initActions = true;
		}
		
        if(!this.historyPanel) {
            this.historyPanel = new com.ibm.rational.test.lm.web.ui.internal.widgets.MachineHistoryPanel();
			var histhdr = this.historyPanel.getCollapsibleSection();
			var skip = new SkipNavigation( {linkname: this.historyPanel.getCollapsibleSection().getTitle(),
							    parent: this.domNode,
								insertReference: this.viewHeaderNode.domNode,
								onActivate: function() {if( !histhdr.open )histhdr.toggle();},
								onFocus: function() {histhdr.setFocus();} });

            // append the panel to the root table of the section layout form
            var tr = this.formLayout.rootTable.insertRow(this.formLayout.rootTable.rows.length);
            var td = tr.insertCell(0);
            tr.tabIndex = -1;
            td.tabIndex = -1;
            td.appendChild(this.historyPanel.domNode);
		}
        if(!this.reservationsPanel) {
            this.reservationsPanel = new com.ibm.rational.test.lm.web.ui.internal.widgets.MachineReservationsPanel({'permissions':this.permissions});
			var reshdr = this.reservationsPanel.getCollapsibleSection();
			var skip = new SkipNavigation( {linkname: this.reservationsPanel.getCollapsibleSection().getTitle(),
							    parent: this.domNode,
								insertReference: this.viewHeaderNode.domNode,
								onActivate: function() {if( !reshdr.open )reshdr.toggle();},
								onFocus: function() {reshdr.setFocus();} });
								
            // append the panel to the root table of the section layout form
            var tr = this.formLayout.rootTable.insertRow(this.formLayout.rootTable.rows.length);
            var td = tr.insertCell(0);
            tr.tabIndex = -1;
            td.tabIndex = -1;
            td.appendChild(this.reservationsPanel.domNode);
        }
        
        if(this._canPerformAction(AUTOMATE) && this.assetId && this.assetId != "" && this.assetType == "Machine" )
        {
        	var inven = new com.ibm.rational.test.lm.web.client.internal.inventory();
			var checkHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
			(
				{
					parent: this,
					_success: function( response )
					{
                        if(this.parent.automationButton)
                           this.parent.automationButton.setDisabled( ( !response || ( response.length == 0 ) ) );
					},
					_failure: function( errObj )
					{
					}
				},
				"_success",
				"_failure"
			);
			inven.getAutomationsForAsset( checkHandler, this.assetId );
        }
		
        if(this._canPerformAction(DISCOVER) && this.assetId && this.assetId != "" && this.assetType == "Machine" )
        {
        	var inven = new com.ibm.rational.test.lm.web.client.internal.inventory();
			var checkHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper
			(
				{
					parent: this,
					_success: function( response )
					{
                        if(this.parent.discoverButton)
                           this.parent.discoverButton.setDisabled( ( response && ( response == "false" ) ) );
					},
					_failure: function( errObj )
					{
					}
				},
				"_success",
				"_failure"
			);
			inven.isInventoryAvailable( checkHandler, this.assetId );
        }

		// load the panels' data.
        this.historyPanel.load(this.assetId);
        this.reservationsPanel.load(this.assetId, this.resourceName);
	},
    
	// Summary:
	//     implements the form listener's interface. Parameters are not used.
	// 
    attributeValueChanged: function(attributeKey, oldValue, newValue)
    {
		var oVal = ( oldValue ) ? oldValue : null;
		var nVal = ( newValue ) ? newValue : null;
		this._isDirty = ( oVal != nVal );
        // the user doesn't have permission to edit
        // don't update the button state/message, etc.
        if(!this._canPerformAction(EDIT)) {
             return;
        }
		
        var disableAction = false;
		var msg = null;
		var status = ViewHeaderLayout.INFO;
        
		if(this._hasHeaderEmptyRequireField())
		{
			disableAction = true;
			msg = RtlmUiUtil.getMessage("msgSpecifyName", RtlmUiUtil.getMessage(this.assetType));
            status = ViewHeaderLayout.WARNING;
		}

		if(this.formLayout.hasEmptyRequiredFields()) {
			disableAction = true;
			msg = RtlmUiUtil.getMessage("msgRequiredValueNotPresent");
            status = ViewHeaderLayout.WARNING;
		}

		if(!disableAction && this.formLayout.hasErrorFields()) {
            disableAction = true;
            msg = RtlmUiUtil.getMessage("msgErrorValuesPresent");			
            status = ViewHeaderLayout.WARNING;
		}
		
		// something changed. Enable the save button and clear and status.
        this.saveButtonNode.disabled = disableAction;
        
        // once there's editing, always allows discard.
        this.discardButtonNode.disabled = false;
        
        // value changed update the header's info sections to pick up
        // new values.
        this.formElementProvider.refreshHeadersInfo();
        
        // No error messages, show a save reminder message
        if(!msg) {
            var msgId = (this.isNew) ? "saveNewReminderMsg" : "saveChangeReminderMsg";
            msg = RtlmUiUtil.getMessage(msgId);           
        }
        this._updateSaveStatus(status, msg);

		this._updateTabLabel(this._isDirty);
	},
	
    // Summary:
    //     implements the form listener's interface
    // 
    attributeDeleteActionInvoked: function(attributeKey, deleteFunction )
    {
		// deleting an attribute is just like editing 
		// values in the form, handle the form value changed.
		//
		var self = this;
		var args = {
			title: RtlmUiUtil.getMessage("msgConfirmTitle"),
			message: RtlmUiUtil.getMessage("msgConfirmRemoveLMSection"),
			yesCallback: function(){
				if( deleteFunction )
					deleteFunction( attributeKey );
				self.attributeValueChanged( attributeKey, null, null );
				return true;          
			},
			noCallback: function() {
				return false;
			}
		}
		var dialog = new YesNoDialog(args);
        dialog.setSecondaryTitle(args.title);
        dialog.open();
    },
    
    _onRefreshActionClick: function()
    {
    	if( !this.isNew )
			this._loadAssetById( this.assetId );

        // reset the save status
        this._updateSaveStatus(-1, null);
        
		this._updateTabLabel(false /*dirty*/);
    },

	// Summary:
	//    Handles save action button click. This method invokes an asynchrous call
	//    to the server to save the asset's new attributes. Based on the viewer's "isNew"
	//    state, it invokes the appropriate service to either create a new asset, or to
	//    update it.
	//
	_onSaveActionClick: function()
	{
        var responseHandler = new ServiceResponseHandler(this, "_saveSuccess", "_failureHandler");
		
		if(this.isNew) {
			this.theCatalog.postCreateAsset(responseHandler, this.asset);
		}
		else {
			this.theCatalog.updateAsset(responseHandler, this.asset);
		}
		this.viewHeaderNode.setProgressStart(RtlmUiUtil.getMessage("savingAsset"));
        
        // update the save status too
        this._updateSaveStatus(ViewHeaderLayout.INFO, RtlmUiUtil.getMessage("savingAsset"));
        
        // don't let the user click on action at this point.
        this._resetButtonStates();
	},
	
    _onDiscardActionClick: function()
    {
        var args = {
            'message': RtlmUiUtil.getMessage("confirmDiscard"),
            'yesCallback': 
                function(self) {
                    if(self.isNew) {
                        self.asset = self.theCatalog.createAsset(self.assetType);
				        var typeName = RtlmUiUtil.getMessage(self.assetType);
				        self.viewHeaderNode.setNoValueEditableTitleLabel(RtlmUiUtil.getMessage("newAssetPrompt", typeName));
						self.viewHeaderNode.setTitle( "" );
                        self._constructForm();
                    }
                    else {
                        // reset the save status
                        self._loadAssetById(self.assetId);
						self._updateTabLabel(false /*dirty*/);
                    }      
                    self._updateSaveStatus(-1, null); 
                    self._resetButtonStates();
               },
            'yesCallbackData': this,
            'noCallback': null,
            'noCallbackData': null
        };
        var confirmDlg = new com.ibm.asq.common.web.ui.internal.widgets.YesNoDialog(args);
        confirmDlg.setSecondaryTitle(RtlmUiUtil.getMessage("msgConfirmTitle"));
		confirmDlg.open();
    },
    
	// Summary:
	//   asynchrous success callback handler for the save action. 
	//
	_saveSuccess: function(response)
	{
		this._isDirty = false;
        this.viewHeaderNode.setProgressEnd();
        this.asset = response;
		
        var oldAssetId = this.assetId;
		this.assetId = this.asset.getId();
		
        var type = (this.assetType == "Machine") ? RtlmUiUtil.TYPE_MACHINE : (this.assetType == "VirtualImage") ? RtlmUiUtil.TYPE_VM : this.assetType;

        try {
            if(this.isNew) {
                dojo.publish(GlobalEvents.UpdateTabUri, [window.location.hash, AssetUriUtil.getViewAssetDetailURI(this.assetType, this.assetId)]);
                dojo.publish(GlobalEvents.ArtifactsAdded, [ type, [ this.asset ], AssetUriUtil.getViewAssetDetailURI(this.assetType, this.assetId) ]);
            } else {
                dojo.publish(GlobalEvents.ArtifactsUpdated, [ type, [ this.asset ], AssetUriUtil.getViewAssetDetailURI(this.assetType, this.assetId) ]);
				this._updateTabLabel(false /*dirty*/);
            }
        }
        catch(e) {
            // ignore any exceptions happened in the listener.    
        }
        
        if (this.isNew) {
            this.isNew = false;
            
            // before we reload the page with additional sections and actions,
            // for this newly created machine, fetch the resource's permissions 
            // first. 
            var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(
                {
                    parent: this,
                    _success: function(response) 
                    {
                        this.parent.permissions = response[0];
                        this.parent._loadDataSuccess(this.parent.asset);
                    },
                    _failure: function(errObj) 
                    {
                        this.parent._loadDataSuccess(this.parent.asset);
                    }
                },
                "_success",
                "_failure"
            );
            this._initResourcePermissions(responseHandler);
        }
        else {
            // The investory system may update the asset after a save/update, so we need to
            // refresh the asset to insure we have the correct state id.
            this._loadDataSuccess(this.asset);
        }
        var msg = RtlmUiUtil.getMessage("savedSuccessMsg", RtlmUiUtil.getCurrentTimeString());
        this.viewHeaderNode.setStatus(msg, ViewHeaderLayout.SUCCESS);
		
		//Check whether we have exceeded the limit on the number of CPUs
		this._checkCPULimit();
        
        // refresh the title too, in case this was new or there's a name change
        this._updateBrowserTitle();
	},
	
    _failureHandler: function(response)
    {
        this.viewHeaderNode.setProgressEnd();
        if (response.status == 409) {
			// Stale Data
			this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("saveAssetFailedModifiedAsset"), ViewHeaderLayout.ERROR);
		}
		else if (response.message && response.message.toLowerCase().indexOf('license') > -1) {
				RtlmUiUtil.showServerErrorMessage(MessageUtil.getMessage(MessageBundle.COMMON, "serverLicenseErrMsg"));
				var details = (response && response.message) ? response.message : null;
				this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("saveFailedMsg", RtlmUiUtil.getCurrentTimeString()), ViewHeaderLayout.ERROR, details);
		}
		else {
			var details = (response && response.message) ? response.message : null;
			this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("saveFailedMsg", RtlmUiUtil.getCurrentTimeString()), ViewHeaderLayout.ERROR, details);
		}
    },
    
    _updateSaveStatus: function(status, msg, details)
    {
        if(msg)
            this.viewHeaderNode.setStatus(msg, status, details);
        else
            this.viewHeaderNode.clearStatus();        
    },
    
    _ReleaseAssetsSuccess: function(response) {
        this.viewHeaderNode.setProgressEnd();
        this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("resourceReleased"), ViewHeaderLayout.SUCCESS);
    },
    
    _ReleaseAssetsFailure: function() {
        this.viewHeaderNode.setProgressEnd();
        this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("resourceReleaseFailed"), ViewHeaderLayout.ERROR);
    },

	// Summary:
	//    Handles forwarding of machine to automation page 
	//
	_onRunAutomationActionClick: function()
	{
		var machines = { "Data": [[this.assetId,  this.formLayout.getContent().getRoot().getAttributeValue( "name" )]], "Header": ["machine.id", "machine.name"] };
			
		dojo.setObject( "com.ibm.rational.test.lm.web.machines", machines );
		
		window.location.href = InventoryUriUtil.getAutomationsURI();
	},
	
	_onDeployActionClick: function()
	{
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_virtualizationAvailableSuccess", "_virtualizationAvailableFailure");
		var inventory = new com.ibm.rational.test.lm.web.client.internal.inventory();
		this.operationFunction = this.deployLabResource;
		this.operation = "Deploy";
		inventory.isVirtualizationAvailable( responseHandler, this.assetId )
	},

	_onUnDeployActionClick: function()
	{
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_virtualizationAvailableSuccess", "_virtualizationAvailableFailure");
		var inventory = new com.ibm.rational.test.lm.web.client.internal.inventory();
		this.operationFunction = this.undeployLabResource;
		this.operation = "Undeploy";
		inventory.isVirtualizationAvailable( responseHandler, this.assetId )
	},
	
	_virtualizationAvailableSuccess: function( results )
	{
		if( results && ( results.length == 1 ) && ( results[ 0 ] == this.assetId ) )
		{
			this.operationFunction();
			this.operationFunction = null;
		}
		else
		{
			var oper = RtlmUiUtil.getMessage(this.operation);
        	this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("deployOperationsNotAvailable", oper, this.resourceName), ViewHeaderLayout.ERROR);			
		}
	},
	
	_virtualizationAvailableFailure: function( errorObj )
	{	
	},
	
	deployLabResource: function()
	{
		dojo.setObject( "com.ibm.rational.test.lm.web.virtualimages", this.assetId );
		window.location.href = InventoryUriUtil.getDeploymentURI();		
	},
	
	undeployLabResource: function()
	{
		dojo.setObject( "com.ibm.rational.test.lm.web.virtualmachines", this.assetId );
		window.location.href = InventoryUriUtil.getDeploymentURI();				
	},
	
    setSelectContents: function(selectNode, choices, createEmpty, defaultVal)
    {
        var pos = 0;

        // create an empty selection
        if(createEmpty) {
            selectNode.options[0] = new Option("");
            pos = 1;
        }
        
        if(choices && choices.length > 0) {
            for (var i = 0; i < choices.length; i++) {
                // the display text for the combo option
                // is the same as the value that we want to
                // retrieve. We can just use the option's 'text'
                // property, i.e., don't need to set the option's
                // 'value' property.
                selectNode.options[i+pos] = new Option(choices[i]);
                
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(defaultVal && choices[i] == defaultVal)
                    selectNode.options[i+pos].selected = true;
            }
        }
        // don't select anything if we don't have a default value
        // and there's not an empty option.
        if(!defaultVal && !createEmpty)
            selectNode.selectedIndex = -1;
    },


    _loadUsersControl: function(selectNode, users, createEmpty, defaultVal)
    {
        var pos = 0;

        // create an empty selection
        if(createEmpty) {
            selectNode.options[0] = new Option("");
            pos = 1;
        }
        
        if(users && users.length > 0) {
            for (var i = 0; i < users.length; i++) {
                // the display text for the combo option
                // is the same as the value that we want to
                // retrieve. We can just use the option's 'text'
                // property, i.e., don't need to set the option's
                // 'value' property.

				//Don't include archived users UNLESS the archived user is the default value
				if (users[i].archived)
				{
					if (!defaultVal || users[i].name != defaultVal)
						continue;
				}

				var option = new Option(users[i].name);
				option.disabled = users[i].archived;
                selectNode.options[pos] = option;
                
                // make this option selected (shown) if a 
                // selectValue is specified and it matches this
                // option.
                if(defaultVal && users[i].name == defaultVal)
                    selectNode.options[pos].selected = true;
				pos++;
            }
        }
        // don't select anything if we don't have a default value
        // and there's not an empty option.
        if(!defaultVal && !createEmpty)
            selectNode.selectedIndex = -1;
    },

    _setOwner: function() 
    {
        var val = null;
        
        for(var i = 0; i < this.ownerField.options.length; i++) {
            if(this.ownerField.options[i].selected) {
                val = this.ownerField.options[i].text;
            }
        }
        if(val != null) {
            var users = com.ibm.rational.test.lm.web.client.internal.RtlmInitializer.getUsers();
            if(users) {
                var uid = "";
				// if the user selected the empty (clear) value, don't have
				// look for it in the user list. We'll just use the empty uid.
				// to clear the owner attribute.
				if (val.length > 0) {
					for (var i = 0; i < users.length; i++) {
						if (users[i].name == val) {
							uid = users[i].itemId;
							break;
						}
					}
				}
                var user = {
                    "ds": val,
                    "uuid": uid
                }; 
				var oldvalue = this.asset.getRoot().getAttributeValue("owner");
                this.asset.getRoot().setAttributeValue("owner", user);
                this.attributeValueChanged("owner", ( oldvalue ) ? oldvalue.uuid : null, uid);             
            }
        }
    },
    
    _setOperationalStatus: function() 
    {
        var val = null;
        
        for(var i = 0; i < this.opStatusField.options.length; i++) {
            if(this.opStatusField.options[i].selected) {
                val = this.opStatusField.options[i].text;
            }
        }
        if (val != null) {
			var oldvalue = this.asset.getRoot().getAttributeValue("Operational Status");
            this.asset.getRoot().setAttributeValue("Operational Status", val);
            this.attributeValueChanged("Operational Status", oldvalue, val);             
        }
    },
    
    _getIconForAssetType: function()
    {
        if(this.assetType == "Machine")
            return "machine-icon-16";
        else
            return "vm-icon-16";
        return null;
    },
	
	_onDiscoverActionClick: function()
	{
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_inventoryAvailableSuccess", "_inventoryAvailableFailure");
		var inventory = new com.ibm.rational.test.lm.web.client.internal.inventory();
		inventory.isInventoryAvailable( responseHandler, this.assetId )
	},
	
	_inventoryAvailableSuccess: function( results )
	{
		if (results == "true") {
			dojo.setObject("com.ibm.rational.test.lm.web.discoverimages", this.assetId);
			window.location.href = InventoryUriUtil.getDeploymentURI();
		}
		else{
        	this.viewHeaderNode.setStatus(RtlmUiUtil.getMessage("nonDiscoverableAsset", this.resourceName), ViewHeaderLayout.ERROR);
			
		}						
	},
	
	_inventoryAvailableFailure: function( errorObj )
	{
	},
	
	_onAddToBuildForgeActionClick: function()
	{
    	var panel =	'<div dojoType="dijit.layout.LayoutContainer" >' +
					' <div dojoType="dijit.layout.ContentPane" layoutAlign="client" dojoAttachPoint="textNode">' +
					'  <table class="add-to-bld-frg" dojoAttachPoint="editorTableNode" border="0" cellspacing="10px" align="center">' +
					'   <tr>' +
					'    <td dojoAttachPoint="usernameNode"></td>' +
					'    <td><label><input dojoAttachPoint="userInputNode" type="text" size="50"/></label></td>' +
					'   </tr>' +
					'   <tr>' +
					'    <td dojoAttachPoint="passwordNode"></td>' +
					'    <td><label><input dojoAttachPoint="passwordInputNode" type="password" size="50"/></label></td>' +
					'   </tr>' +
					'   <tr dojoAttachPoint="selectProviderNode">' +
					'    <td dojoAttachPoint="selectLabelNode"></td>' +
					'    <td><label><select dojoAttachPoint="selectInputNode" style="width: 325px"/></label></td>' +
					'   </tr>' +
					'   <tr>' +
					'    <td dojoAttachPoint="bldfrgpathNode"></td>' +
					'    <td><label><input dojoAttachPoint="pathInputNode" type="text" size="50"/></label></td>' +
					'   </tr>' +
                    '   <tr>' +
                    '    <td dojoAttachPoint="portNode"></td>' +
                    '    <td><label><input dojoAttachPoint="portInputNode" type="text" size="50"/></label></td>' +
                    '   </tr>' +
                    '   <tr>' +
					'    <td colspan="2" class="bf-dlg-properties" dojoAttachPoint="propertiesNode" style="visibility:hidden"></td>' +
					'   </tr>' +
					'  </table>' +
					' </div>' +
                    '  <div dojoAttachPoint="statusNode" class="bf-dlg-status-area">' +
                    '    <img dojoAttachPoint="statusImageNode" class="bf-dlg-status-icon"></img>' +
                    '    <span dojoAttachPoint="statusLabelNode" class="bf-dlg-status-label">&nbsp;</span>' +
                    '  </div>' +
                    '  <div dojoAttachPoint="progressNode" class="bf-dlg-status-area" style="display:none">' +
                    '    <div class="bf-dlg-progress-image">' +
                    '      <span dojoAttachPoint="progressLabelNode" class="bf-dlg-progress-label"></span>' +
                    '    </div>' +
                    '  </div>' +
					'</div>';
    	
		var dom = new com.ibm.asq.common.web.ui.internal.util.DomConstructor( panel );
		
		dom.portNode.innerHTML = RtlmUiUtil.getMessage("labelPort");
		dom.usernameNode.innerHTML = RtlmUiUtil.getMessage("labelUserName");
		dom.passwordNode.innerHTML = RtlmUiUtil.getMessage("labelPassword");
		dom.selectLabelNode.innerHTML = RtlmUiUtil.getMessage("labelProvider");
		dom.bldfrgpathNode.innerHTML = RtlmUiUtil.getMessage("labelTempPath");
		// Listening the selected provider change
		dom.selectInputNode.onchange = dojo.hitch(this, "changeSelProvider");
		
		this.properieswidget = new com.ibm.asq.common.web.ui.internal.widgets.propertyeditor.PropertyEditor();
		dom.propertiesNode.appendChild( this.properieswidget.domNode );
        
        // attach validations to required fields to enable/disable the save button.
        dom.userInputNode.onkeyup = dojo.hitch(this,"validateBFInputs");
        dom.passwordInputNode.onkeyup = dojo.hitch(this,"validateBFInputs");
        dom.pathInputNode.onkeyup = dojo.hitch(this,"validateBFInputs");
        
        this.isBFProvider = false;
        
        var args = {
            primaryTitle: RtlmUiUtil.getMessage("Add to provider"),
            csHelpID: "rtlm_addtobuildforge.html",
            csHelpProviderId: com.ibm.rational.test.lm.web.CSHelpID,
            isClosable: true,
            okLabel: RtlmUiUtil.getMessage("Save"),
            okCallback: dojo.hitch( this, this._saveAction ),
            cancelCallback: dojo.hitch( this, this._cancelAction )
        }
        this.addToBldFrgDlg = new com.ibm.asq.common.web.ui.internal.widgets.SelectionDialog(args);
        this.addToBldFrgDlg.contentContainer.appendChild(dom.domNode);
		
		this.addToBldFrgDOM = dom;

        this.addToBldFrgDlg.getOkButton().disabled = true;
        dojo.style(this.addToBldFrgDOM.progressNode, "display", "block");
        this.addToBldFrgDOM.progressLabelNode.innerHTML = RtlmUiUtil.getMessage("fetchingInventoryProviders");
        this.addToBldFrgDlg.open();
		
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_providersSuccess", "_providersFailure");
		var inventory = new com.ibm.rational.test.lm.web.client.internal.inventory();
		inventory.getInventoryProviders( responseHandler, this.assetId );
	},
	
    validateBFInputs: function()
    {
        var hasInput = false;        
        
        if (!this.addToBldFrgDlg.hasBFError && this.isBFProvider) {
            var uid = this.addToBldFrgDOM.userInputNode.value;
            var pwd = this.addToBldFrgDOM.passwordInputNode.value;
            var tmpPath = this.addToBldFrgDOM.pathInputNode.value;
            
            hasInput = uid && uid.length > 0 && pwd && pwd.length > 0 && tmpPath && tmpPath.length > 0;
            
            if (!hasInput) {
                dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "visible");
                this.addToBldFrgDOM.statusImageNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/info_obj.gif");
                this.addToBldFrgDOM.statusLabelNode.innerHTML = RtlmUiUtil.getMessage("msgBFNeedInputs");
            }
            else {
                dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "hidden");
            }
        }
        this.addToBldFrgDlg.getOkButton().disabled = ((this.isBFProvider == true) 
        												&& (hasInput == false || this.addToBldFrgDlg.hasBFError));     
    },
    
    changeSelProvider: function()
    {
    	if (!this.addToBldFrgDlg.hasBFError) {
    		var typeIndex = dojo.fromJson(
    						this.addToBldFrgDOM.selectInputNode.options[ this.addToBldFrgDOM.selectInputNode.selectedIndex ]
    						.value).typeIndex;
    		if (typeIndex == 0) {
	    		// The provider is Build Forge provider
	    		this.isBFProvider = true;
	    	} else {
	    		this.isBFProvider = false;
	    		dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "hidden");
	    	}
	    	if (typeIndex == -1) {
	    		// The type of provider is unknown for RQM, display generic properties editor
	    		dojo.style(this.addToBldFrgDOM.propertiesNode, "visibility", "visible");
	    	} else {
	    		dojo.style(this.addToBldFrgDOM.propertiesNode, "visibility", "hidden");
	    	}
    		this.validateBFInputs();
    	}
    },
    
	_providersSuccess: function( response )
	{
        // the user could have cancelled while we're fetching the info.
        // Do nothing if this happened.
        if(this.addToBldFrgDlg == null)
            return;

        dojo.style(this.addToBldFrgDOM.progressNode, "display", "none");
        
        // just in case there's no prvider in the response...
        if(!response || response.length == 0) {
            dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "visible");
            this.addToBldFrgDOM.statusLabelNode.innerHTML = RtlmUiUtil.getMessage("msgNoInventoryProvider"); 
            this.addToBldFrgDOM.statusImageNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
            this.addToBldFrgDlg.hasBFError = true;                   
            return;
        }    
        
		//Add drops down options
		for (var iOption = 0; response && iOption < response.length; iOption++) {
			var name = response[ iOption ].instanceId + ( ( response[ iOption ].isMapped ) ? " (Update)" : " (Add)" );
			var value = 
				{
					"uuid": response[ iOption ].uuid, 
					"typeIndex": response[ iOption ].typeIndex
				};
			this.addToBldFrgDOM.selectInputNode.options[this.addToBldFrgDOM.selectInputNode.options.length] = 
				new Option(name, dojo.toJson(value));
		}
        this.changeSelProvider();
	},
	
	_providersFailure: function( errorObj )
	{
        // the user could have cancelled while we're fetching the info.
        // Do nothing if this happened.
        if(this.addToBldFrgDlg == null)
            return;

        dojo.style(this.addToBldFrgDOM.progressNode, "display", "none");
        dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "visible");
        this.addToBldFrgDOM.statusLabelNode.innerHTML = RtlmUiUtil.getMessage( "errGetInventoryProviders"); 
        this.addToBldFrgDOM.statusImageNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
        
        // Show the error has errors. Only thing the user can do
        // is to cancel.
        this.addToBldFrgDlg.hasBFError = true;                   
	},
	
	_saveAction: function()
	{
        dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "hidden");
        dojo.style(this.addToBldFrgDOM.progressNode, "display", "block");
		this.addToBldFrgDOM.progressLabelNode.innerHTML = RtlmUiUtil.getMessage("Saving");
        
        this.addToBldFrgDlg.getOkButton().disabled = true;
        
		var inventory = new com.ibm.rational.test.lm.web.client.internal.inventory();
	   	var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(this, "_saveToBuildForgeSuccess", "_saveToBuildForgeFailure");
		
		var parameters = {};
		if (this.addToBldFrgDOM.propertiesNode.style.visibility == "visible") {
			// Fecth the properties value only if the propertiesNode is visible
			parameters = this.properieswidget.getProperties();
		}
		
		var requiredParams = 
			{
				"username": this.addToBldFrgDOM.userInputNode.value,
				"password": this.addToBldFrgDOM.passwordInputNode.value,
				"Path": this.addToBldFrgDOM.pathInputNode.value,
				"port": this.addToBldFrgDOM.portInputNode.value 
			};

		for (var param in requiredParams) {
	    	if (requiredParams[param] && requiredParams[param].length > 0) {
	    		parameters[param] = requiredParams[param];
	    	}
	    }

		var optionValue = dojo.fromJson(this.addToBldFrgDOM.selectInputNode.options[ this.addToBldFrgDOM.selectInputNode.selectedIndex ].value);
		inventory.addAssetToProvider(  responseHandler, 
										this.assetId,
										optionValue.uuid,
										optionValue.typeIndex,
										parameters );
        
        // we don't popup down this dialog. We're still using it
        // to show progress/status
        return false;								
	},
	
	_saveToBuildForgeSuccess: function( response )
	{
        // the user could have cancelled while we're fetching the info.
        // Do nothing if this happened.
        if(this.addToBldFrgDlg == null)
            return;

        dojo.style(this.addToBldFrgDOM.progressNode, "display", "none");
        dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "visible");

		if ((response && response.rc == "SPIRC_NO_ERROR") || (!response)) {
            this.addToBldFrgDOM.statusImageNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/success_obj.gif");
			this.addToBldFrgDOM.statusLabelNode.innerHTML = RtlmUiUtil.getMessage("savedBldFrgSuccessMsg");

            // remove the ok button and leave the cancel to close.
            if(this.addToBldFrgDlg._noButton)
                this.addToBldFrgDlg._noButton.innerHTML = RtlmUiUtil.getMessage("close_button");
            dojo.style(this.addToBldFrgDlg.getOkButton(), "display", "none");
                            
        	this._loadAssetById(this.assetId);
		}
		else if( response )
		{
			var localMsg = RtlmUiUtil.getMessage(response.rc, this.addToBldFrgDOM.selectInputNode.options[ this.addToBldFrgDOM.selectInputNode.selectedIndex ].text);
			// Display error message from adapter except Build Forge, 
			// because error message from BF adapter isn't wrapped well currently.
			if ((this.isBFProvider == false) 
				&& (response.message != null) 
				&& (response.message.length > 0)) {
				localMsg = localMsg + " : " + response.message;
			}
            this.addToBldFrgDOM.statusImageNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
			this.addToBldFrgDOM.statusLabelNode.innerHTML = localMsg;			
		}
	},
	
	_saveToBuildForgeFailure: function( error )
	{
        // the user could have cancelled while we're fetching the info.
        // Do nothing if this happened.
        if(this.addToBldFrgDlg == null)
            return;

        dojo.style(this.addToBldFrgDOM.progressNode, "display", "none");
        dojo.style(this.addToBldFrgDOM.statusNode, "visibility", "visible");

		this.addToBldFrgDOM.statusLabelNode.innerHTML = RtlmUiUtil.getMessage("savedBldFrgFailureMsg"); 
        this.addToBldFrgDOM.statusImageNode.src = dojo.moduleUrl("com.ibm.asq.common.web", "ui/internal/view/layout/images/error_obj.gif");
        
        // allow the user do try again.
        this.addToBldFrgDlg.getOkButton().disabled = false;
 	},
	
	_cancelAction: function()
	{
		this.addToBldFrgDlg.destroy();
        this.addToBldFrgDlg = null;
	},
	
	_onCreateConfigActionClick: function()
	{
		//Create a config from the current machine
		var config = this.theCatalog.createConfigurationFromAsset(this.asset);
		var configString = config.toJson();
		
		//Set the config string into the Config Editor Class
		com.ibm.rational.test.lm.web.ui.internal.views.configurations.ConfigurationEditor.setInitialConfigString(configString);
		
		//Launch the New Config action.
		var URI = ConfigurationUriUtil.getNewConfigurationURI();
		window.location.href = URI;		
	},

    // Summary:
    //    Determines whether the resource's deleted state has been changed 
    //    as indicated by the global event.
    //
    _isDeleteStateChanged: function(changeType)
    {
        if(!this.isDeleted) {
            return changeType == GlobalEvents.ArtifactsDeleted;
        }
        return !(changeType == GlobalEvents.ArtifactsUndeleted);
    },
    
    // handler on artifacts changed
    onArtifactChanged: function(changeType, artifactType, artifactArray) 
    {
        // this is a new asset being defined. Don't care about existing asset changes.
        if(this.isNew || !artifactArray || artifactArray.length == 0)
            return;

        var displayAssetType = (this.assetType == "Machine") ? RtlmUiUtil.TYPE_MACHINE : (this.assetType == "VirtualImage") ? RtlmUiUtil.TYPE_VM : this.assetType;
        
        // only care about machines or vm that is our asset type 
        // and the deleted state has changed.
        if (artifactType == displayAssetType) { 
            for(var i = 0; i < artifactArray.length; i++) {
                
                // This event concern this asset, matches the id,
                // update the deleted state of the page.
                if(this.assetId == artifactArray[i]) {                    
                    var newDeleteState = this._isDeleteStateChanged(changeType);
                    
                    if (this.isDeleted != newDeleteState) {
                        this.isDeleted = newDeleteState;
                        
                        if (this.isDeleted) {
                            var msg = RtlmUiUtil.getMessage("statusNoEditDeleted");
                            var details = RtlmUiUtil.getMessage("statusNoEditDeletedDetails");
                            
                            this.viewHeaderNode.setStatus(msg, ViewHeaderLayout.WARNING, details);
                            this._resetButtonStates();
                        }
                        else {
                            this.viewHeaderNode.clearStatus();
                        }
                    }
                    break;
                }
            }
        }
    },
	
	isDirty: function()
	{
		return( this._isDirty );
	},
	
	hasConfirmDeleteAttribute: function( attributeKey )
	{
		return( true );
	},
	
	//Checks to see if the number of CPUs in the system exeeds the licensed limit.
	//If it does, it displays a warning.
	_checkCPULimit: function()
	{
        var adminClient = new com.ibm.rational.test.lm.web.client.internal.admin();
        var responseHandler = new com.ibm.asq.common.web.ui.internal.TransportServiceResponseWrapper(
            {
                parent: this,
                _success: function(response) 
                {
					var cpuLimit = response.globalCpuSoftLimit;
					var numCPUs = response.numGlobalCpus;
					if (numCPUs > cpuLimit)
						alert(RtlmUiUtil.getMessage("msgCPULimit", numCPUs, cpuLimit));
                },
                _failure: function(errObj) 
                {
                    //Ignore failure
                }
            },
            "_success",
            "_failure"
        );
        adminClient.getSystemOptions(responseHandler);
	},
	
	_updateBrowserTitle: function(){
		if (this.page) {
			var type;
			var name;
			
            type = RtlmUiUtil.getMessage(this.assetType);
			if (this.resourceName) {
				name = this.resourceName;
			}
			else {
				name = RtlmUiUtil.getMessage("labelCapitalNew");
			}
			this.page.browserTitle = RtlmUiUtil.getMessage("pageTitleAsset", type, name);
			net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this.page.browserTitle );
			net.jazz.ajax.ui.PlatformUI.getWorkbench().browserTitle = this.page.browserTitle;
		}
	}
});

})();
