/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
//

//
//
// Author: Paul Beaulieu
//
// Summary:
//

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.machines.FilteringCustomElementProvider");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ParameterDialog");
dojo.require( "com.ibm.asq.common.web.ui.internal.widgets.OptionChooser" );
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider");

(function() {
var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
var DefaultAssetContentProvider = com.ibm.rational.test.lm.web.ui.internal.views.machines.DefaultAssetContentProvider;
	
dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.machines.FilteringCustomElementProvider", 
             [com.ibm.rational.test.lm.web.ui.internal.views.machines.ActionCustomElementProvider],
{
	contentProvider: null,
	savedAttributeSelection: {},
	firstTime: true,
	
	constructSectionHeader: function(sectionKey)
	{
		var elements = {};
		var actionHdr = this.inherited( "constructSectionHeader", arguments );
		
		actionHdr.addAction( RtlmUiUtil.getMessage( "editparms" ), dojo.hitch( this, this.dlgFunction ),
                              sectionKey, "edit-icon");
		
        var contentProvider = this.layoutForm.getContentProvider();
		for( var iGroup = 0; iGroup < sectionKey.groups.length; iGroup++ )
		{
			var shown = contentProvider.getAttributeDisplayListForSection( sectionKey, sectionKey.groups[ iGroup ].root );
			
			if( shown && ( shown.length == 0 ) )
			{
				var button = actionHdr.getActionButton( "add" );
				if( button )
					button.setDisabled( true );
			}
		}
		
		return( actionHdr );
	},
	
	dlgFunction: function( sectionKey )
	{
		var dlg = new com.ibm.rational.test.lm.web.ui.internal.views.machines.ParameterDialog( {title:"Select Attributes To Show", "sectionKey": sectionKey });
		var elements = {};
        var contentProvider = this.layoutForm.getContentProvider();

		for( var iGroup = 0; iGroup < sectionKey.groups.length; iGroup++ )
		{
			var attrs = contentProvider.getAttributeDisplayListForSection( sectionKey, sectionKey.groups[ iGroup ].root, true );
			if(attrs) {
	            for( var iAttr = 0; iAttr < attrs.length; iAttr++ )
	            {
	                elements[ attrs[ iAttr ] ] = {};
	                elements[ attrs[ iAttr ] ].id = attrs[ iAttr ];
	                elements[ attrs[ iAttr ] ].selected = true;
	            }
			}
			var hiddenAttrs = contentProvider.getAttributeHiddenListForSection( sectionKey, sectionKey.groups[ iGroup ].root );
            
			if(hiddenAttrs) {
	            for( var iAttr = 0; iAttr < hiddenAttrs.length; iAttr++ )
	            {
	                elements[ hiddenAttrs[ iAttr ] ] = {};
	                elements[ hiddenAttrs[ iAttr ] ].id = hiddenAttrs[ iAttr ];
	                elements[ hiddenAttrs[ iAttr ] ].selected = false;
	            }				
			}
		}

		dlg.setOptions( elements );
		dlg.setOKaction( dojo.hitch( this, this.parametersChanged ) );
		
		dlg.show();
	},
	
	_orderAttributeList: function( elements, selected )
	{
		var list = {};
		
		for( var names in elements )
		{
			list[ elements[ names ] ] = {};
			list[ elements[ names ] ].selected = selected;
			list[ elements[ names ] ].id = names;
		}
		
		return( list );		
	},
	
	getAttributeList: function( rootPart, list )
	{
		var parts = rootPart.getAttributeNames();
		
		for( var i = 0; i < parts.length; i++ )
		{
			list[ parts[ i ] ] = {};
			list[ parts[ i ] ].selected = false;
			var attr = rootPart.getAttribute( parts[ i ] );
			var path = rootPart.getAttribute( parts[ i ] ).getPathName();
			list[ parts[ i ] ].id = path;
			
			if( attr.getType() == "ConfigurationPartType" )
			{
				this.getAttributeList( attr.getRootPart(), list );
			}
		}		
	},
	
	getSectionAttributes: function( attrKeys, sectionKey, elements )
	{
        var contentProvider = this.layoutForm.getContentProvider();

		for( var i = 0; i < attrKeys.length; i++ )
		{
			var attrInfo = contentProvider.getAttributeInfo( contentProvider.layoutForm.content, sectionKey, attrKeys[ i ] );
			
			for( var elems = 0; attrInfo.collection && elems < attrInfo.collection.length; elems++ )
			{
				var item = {};
				if( attrInfo.collection[ elems ].isAttribute )
				{
					elements[ attrInfo.collection[ elems ].catPath ] = attrInfo.collection[ elems ].attributeName;
					if( attrKeys[i].parentPart )
					{
						var attr = attrKeys[ i ].parentPart.getAttribute( attrInfo.collection[ elems ].attributeName );
						if( attr.getType() == DefaultAssetContentProvider.TYPE_PART && attr.isArray() )
						{
							var root = attr.getRootPart();
							var value = root.getName();
							attr.addArrayValue( value );
							
							var vals = attr.getArrayValues();
							
							for( var iVal = 0; iVal < vals.length; iVal++ )
							{
								var attrNames = vals[ iVal ].getAttributeNames();
								
								for( var iAttr = 0; iAttr < attrNames.length; iAttr++ )
								{
									var id = vals[ iVal ].getAttribute( attrNames[ iAttr ] ).getPathName();
									
									elements[ id ] = attrNames[ iAttr ];
								}
							}
							
							attr.removeArrayValue( vals.length - 1 );
						}
					}
				}
				else if( attrInfo.collection[ elems ].parentPart )
				{
					try
					{
						var attr = attrInfo.collection[ elems ].parentPart.getAttribute( attrInfo.collection[ elems ].attributeName );
						elements[ attr.getPathName() ] = attrInfo.collection[ elems ].attributeName;
					}
					catch( e ){}
				}
			}
			
			if( attrInfo.collection && dojo.isArray( attrInfo.collection ) )
			{
				this.getSectionAttributes( attrInfo.collection, sectionKey, elements );
			}
		}		
	},
		
	parametersChanged: function( sectionKey, parameters )
	{
		var shown = [];
		var hidden = [];
		
		for( var elem in parameters )
		{
			if( parameters[ elem ] == true )
			{
				shown.push( elem );
			}
			else
			{
				hidden.push( elem );				
			}
		}
		
        var contentProvider = this.layoutForm.getContentProvider();
		for( var iGroup = 0; iGroup < sectionKey.groups.length; iGroup++ )
		{
			contentProvider.setShowHideAttributeForSection( sectionKey, sectionKey.groups[ iGroup ].root, shown, hidden );
		}
/*
        // this code is broken as getSectionHeader does not exist.     
		var sHdr = this.getSectionHeader( sectionKey );
		//Get the add button and disable it if nothing is going to be shown anyways
		var button = sHdr.getActionButton( "add" );
		
		if( ( shown.length == 0 ) && button )
		{
			button.setDisabled( true );
		}
		else if( button )
		{
			button.setDisabled( false );
		}
*/		
	}
});

})();
