/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.AutomationsMachinesTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	
	var column_types = {
		'Machine': [TableConstants.RENDER_TYPE_LINK ],
		'VirtualImage': [TableConstants.RENDER_TYPE_LINK ]
	};
    var SEARCH_ATTRIBUTE_HEADINGS = {
        'Machine' : [
		  {title: RtlmUiUtil.getMessage("Name"), native_column: "machine\\.name", isSortable: true, includeInFilter: true, width: 100}
		],
		'VirtualImage' : [
          {title: "Image Name", native_column: "name", isSortable: true, includeInFilter: true, width: 100}
		]
    };
    
    var ATTRIBUTE_ID_FIELDS = {
        'Machine' : "machine\\.id",
        'VirtualImage' : "id"
    };
    
    var SEARCH_ATTRIBUTE_CRITERIA = {
        'Machine' : "machine.id, machine.name, machine.ipAddress, machine.Manufacturer, machine.operatingsystem, machine.cpu",
        'VirtualImage' : "VirtualImage.id, VirtualImage.name, VirtualImage.ImageType, VirtualImage.operatingsystem, VirtualImage.VMFile"
    };
    

com.ibm.rational.test.lm.web.ui.internal.views.providers.AutomationsMachineInfo = {
	getSearchAttributeHeadings : function()
	{
		return( SEARCH_ATTRIBUTE_HEADINGS );
	},
	
	getIdFields: function()
	{
		return( ATTRIBUTE_ID_FIELDS );
	},
	
	getSearchAttributeCriteria: function()
	{
		return( SEARCH_ATTRIBUTE_CRITERIA );
	}
};

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.AutomationsMachinesTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider,
{
	constructor: function( assetType )
	{
		this.setParameters( { "column_headings" : SEARCH_ATTRIBUTE_HEADINGS,
					  "id_fields" : ATTRIBUTE_ID_FIELDS,
					  "column_types" : column_types,
					  "column_section" : assetType } );
	}
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.AutomationMachinesListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	constructor: function( params )
	{
		for( var param in params )
		{
			this[ param ] = params[ param ];
		}
	},
	
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
    	if( this.receiver )
    	{
    		if( this.automationstbl )
    		{
   				this.receiver._fetchMachinePermissionsAndCheckButtonState();
    		}
    	}
    }
});

})();
