/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider"); 

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil");
dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");

(function() {

	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;
	var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
	
	var column_types = {
		'Machine': [TableConstants.RENDER_TYPE_LINK, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT ],
		'VirtualImage': [TableConstants.RENDER_TYPE_LINK, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT, TableConstants.RENDER_TYPE_TEXT ]
	};
    var SEARCH_ATTRIBUTE_HEADINGS = {
        'Machine' : [
		  {title: RtlmUiUtil.getMessage("Name"), native_column: "machine\\.name", isSortable: true, includeInFilter: true, width: 25}, 
		  {title: RtlmUiUtil.getMessage("pl_Manufacturer"), native_column: "machine\\.Manufacturer", isSortable: true, includeInFilter: true, width: 25  },
		  {title: RtlmUiUtil.getMessage("Operating System"), native_column: "machine\\.operatingsystem", isSortable: true, includeInFilter: true, width: 25  },
		  {title: RtlmUiUtil.getMessage("Processor Type"), native_column: "machine\\.cpu", isSortable: true, includeInFilter: true, width: 25  }
		],
		'VirtualImage' : [
          {title: "Image Name", native_column: "VirtualImage\\.name", isSortable: true, includeInFilter: true, width: 25}, 
          {title: "Image Type", native_column: "VirtualImage\\.ImageType", isSortable: true, includeInFilter: true, width: 25  },
          {title: "Operating System", native_column: "VirtualImage\\.operatingsystem", isSortable: true, includeInFilter: true, width: 25  },
          {title: "VM File", native_column: "VirtualImage\\.VMFile", isSortable: true, includeInFilter: true, width: 25  }
		]
    };
    
    var ATTRIBUTE_ID_FIELDS = {
        'Machine' : "machine\\.id",
        'VirtualImage' : "VirtualImage\\.id"
    };
    
    var SEARCH_ATTRIBUTE_CRITERIA = {
        'Machine' : "machine.id, machine.name, machine.ipAddress, machine.Manufacturer, machine.operatingsystem, machine.cpu",
        'VirtualImage' : "VirtualImage.id, VirtualImage.name, VirtualImage.ImageType, VirtualImage.operatingsystem, VirtualImage.VMFile"
    };
    

com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineInfo = {
	getSearchAttributeHeadings : function()
	{
		return( SEARCH_ATTRIBUTE_HEADINGS );
	},
	
	getIdFields: function()
	{
		return( ATTRIBUTE_ID_FIELDS );
	},
	
	getSearchAttributeCriteria: function()
	{
		return( SEARCH_ATTRIBUTE_CRITERIA );
	}
};

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachineTableProvider", 
             com.ibm.rational.test.lm.web.ui.internal.views.providers.DefaultTableProvider,
{
	constructor: function( assetType )
	{
		this.setParameters( { "column_headings" : SEARCH_ATTRIBUTE_HEADINGS,
					  "id_fields" : ATTRIBUTE_ID_FIELDS,
					  "column_types" : column_types,
					  "column_section" : assetType } );
	},
	    
    
    _normalizeOSDisplay: function(osName)
    {
        if( osName == null || osName == "OperatingSystem" )
        {
             return "";
        }
        var indx = osName.indexOf("OperatingSystem");
        if( indx > -1) 
        {
            return osName.replace(/OperatingSystem/, RtlmUiUtil.getMessage("unknownOsDisplay"));
        }
        return osName;
    },
    
    // Summary:
    //    The only assumptions the table have on the data that it displays
    //    is that it's an array contents. These are the data content/object that
    //    will be displayed in the table rows.
    //
    getContentObjects: function(content)
    {
        var rows = [];
        for( var iRow = 0; iRow < content.Data.length; iRow++ )
        {
            var aRow = {};
            
            for( var iField = 0; iField < content.Data[ iRow ].length; iField++ )
            {
                if( content.Header[ iField ] == "machine.operatingsystem" )
                {
                    content.Data[ iRow ][ iField ] = this._normalizeOSDisplay(content.Data[ iRow ][ iField ]);
                }
                
                aRow[ content.Header[ iField ] ] = content.Data[ iRow ][ iField ];
            }
            
            rows.push(aRow);
        }
        
        return( rows );
    },
    
    getCellDisplayExtendedInfo: function(rowData, atColumnIndex)
    {  
        var val = null;
        if(atColumnIndex == 0) {
            val = {};
 			var AssetUriUtil = com.ibm.rational.test.lm.web.ui.internal.pages.AssetUriUtil;
            val['href'] = AssetUriUtil.getViewAssetDetailURI( this.columns_section, this.getRowDataId( rowData ) );
        }

        return val;
    }       
});

dojo.declare("com.ibm.rational.test.lm.web.ui.internal.views.providers.MachinesListener", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableListener,
{
	constructor: function( params )
	{
		for( var param in params )
		{
			this[ param ] = params[ param ];
		}
	},
	
    // Summary:
    //    Method to be called when a cell renderer changes the cell value. This notifies
    //    the cell's value change event. 
    //    
    //    
    rowCheckStateChanged: function(tableViewer, rowObject, state)
    {
    	if( this.receiver )
    	{
    		var rows = this.receiver.tableViewer.getCheckedRows();
    		this.receiver.configureButton.setDisabled( rows.length == 0 );
    	}
    }
});

})();