/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.date.locale");

dojo.require("com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableContentProvider");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer");

(function() 
{   
	var RtlmUiUtil = com.ibm.rational.test.lm.web.ui.internal.RtlmUiUtil;

    dojo.declare("com.ibm.rational.test.lm.web.ui.internal.widgets.QueryResultWidget", 
          [dijit._Widget, dijit._Templated], 
    {
        //This line is NECESSARY fot the attachPoints in the PageLayout to get instantiated
        widgetsInTemplate: true,
        
        //The layout of the page.
        templateString: 
            '<div id="${id}" dojoAttachPoint="domNode">' +
            '  <!-- table resultarea --> '+
            '  <div dojoAttachPoint="queryResultDiv"></div> '+
            '</div>',		
		
        /*label for query select combo*/
		label: null,
        
        /*whether to load the query select combo box at creation time*/
        loadQueryAtCreate: true,
        
        /*Result table*/
        resultTable: null,
        
        isMultiSelectTable: false,
		tableSortColumn: -1,
		tableSortDirection: 1,
        tableId: null,
        

//===========================================================
        
    postCreate: function()
    {
		this.inherited("postCreate", arguments);
  
        //Create the table that will hold the query result
        this.createTable();
        
        //Load the combo box containing the query choices.
		if(this.loadQueryAtCreate)
            this.loadQueryCombo();  
    },
	
    createTable: function()
    {
        var args = {
            allowAdjustSettings: true,
            autoSaveSettings: true,
            tableId: this.tableId                        
        }; 
                       
        this.resultTable = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableViewer(this.tableId ? args : {});
        this.resultTable.setContentPageSize( 10 );
        this.resultTable.setTableRowsCheckable( this.isMultiSelectTable );
        this.resultTable.setContentProvider( this.getTableContentProvider() );
		this.resultTable.setTableSummary(RtlmUiUtil.getMessage("labelTableSummary", RtlmUiUtil.getMessage("labelQueryResult")));
        this.queryResultDiv.appendChild( this.resultTable.domNode );
    },
    
    /**
    *   Loads the queries for the query select combo. Subclass should override.
    **/
    loadQueryCombo: function()
    {                    
    },
    
	/**
	 * Returns the provider for the table:  
	 * Subclass should override.
	 */
	getTableContentProvider: function() {
		return( null );		
	},
	
    /**
     * Run selected query and populate the result table. Subclass should override.
     */
    runSelectedQuery: function() 
    {
    },
    
	getQuerySelectionValue: function() {
	   return this.resultTable.contentSelect.text;
	},
	
    getQuerySelectionId: function() 
    {
        if(this.resultTable.contentSelect.options) 
        {
            for(var j = 0; j < this.resultTable.contentSelect.options.length; j++) 
            {
                if(this.resultTable.contentSelect.options[j].selected) 
                {
                    return this.resultTable.contentSelect.options[j].value;
                }
            }
        }
    },

    getDateDisplay: function(dateValue, dateOnly)
    {
        // Constant that represents the date equal to "-1".  This is the
        // convention used to represent a null date.  This convention is needed
        // so that we can indicate that we want to remove a date from a date field.
        var _NODATE = "1969-12-31T18:59:59.999-0500";
        
        if(dateValue == null)
            return "";
            
        // Check if the date is -1.  If it is we have no date specified
        // and we should display a blank.
        if( dateValue == _NODATE ) {
            return "";
        }
        var date = RtlmUiUtil.convertTimeStampToDate(dateValue);
        
        if(dateOnly)
            return dojo.date.locale.format(date, {selector:'date', formatLength:"short"});
        return RtlmUiUtil.getDisplayDateTimeString(date);
    }
});

})();
