/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util");

dojo.require("dojo.date.locale"); 

(function() {

com.ibm.asq.common.web.ui.internal.widgets.tableViewer.Util = {   
    
    
    // Summary:
    //    Normalizes the timestamp string to the timezone format expected by
    //    dojo.date.stamp.fromISOString. The server returned timestamp contains timezone
    //    format as [-+][hhmm]. The dojo timestamp expects [-+][hh:mm]. 
    //
    normalizeTimeStampString: function(/* timestamp string */timestamp)
    {
        var timeZone = new RegExp("[-+][0-9]{4}");
        
        if(timestamp && timestamp.length) {
            var indx = timestamp.search(timeZone);
        
            if(indx > -1) {
                var str = timestamp.substring(0, indx + 3) + ":" + timestamp.substring(indx + 3, indx + 5);
                return str;
            }
        }
        return timestamp;
    },
    
    // Summary:
    //    Converts a timestamp returned by the server into a Date object.
    //    If the timestamp is null, or empty, today's date object is returned.
    //
    convertTimeStampToDate: function(/* timestamp string */timestamp)
    {   
        var date = null;
             
        if(timestamp && timestamp.length) {
            var str = this.normalizeTimeStampString(timestamp);
            date = dojo.date.stamp.fromISOString(str);
        }
        return date; 
    }
};

})();
